from flask import Flask, jsonify, make_response, request, send_file
from flask_mysqldb import MySQL

from flask_limiter import Limiter
from flask_limiter.util import get_remote_address

import time

from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.serialization import load_der_public_key
from cryptography.fernet import Fernet
import hashlib, os, base64, json
import bcrypt
import json
import datetime;
from datetime import datetime
import dateutil.parser as dt

from DecimalEncoder import DecimalEncoder
from decimal import Decimal
from entities.Brute import Brute
from entities.User import User
from entities.Encrypted import Encrypted
from controllers.BruteController import BruteController
from controllers.UserController import UserController
from controllers.TimeRecordController import TimeRecordController
from controllers.EncryptionController import EncryptionController


app = Flask(__name__)

limiter = Limiter(
    get_remote_address,
    app=app,
    default_limits=["10000 per day", "10000 per hour"],
)

app.config['MYSQL_HOST'] = 'mysql'
app.config['MYSQL_USER'] = 'admin'
app.config['MYSQL_PASSWORD'] = 'admin'
app.config['MYSQL_DB'] = 'mydb'
app.config["MYSQL_CURSORCLASS"] = "DictCursor"
app.config['DEBUG'] = True

mysql = MySQL(app)

bruteController = BruteController(mysql)
userController = UserController(mysql)
timeController = TimeRecordController(mysql)
encryptionController = EncryptionController(mysql)

def createAESKey():
    key = os.urandom(32)
    return key


def createAESIv():
    iv = os.urandom(16)
    return iv


def dataDBDecrypted(key, iv, data):
    cipher = Cipher(algorithms.AES(key), modes.CFB(iv))
    decryptor = cipher.decryptor()
    ct = decryptor.update(data) + decryptor.finalize()
    return ct


def createResponseData(data):
    data= {
        "data": {
            "message": data            
        },
    }
    return data

def encryptionCheck(dataEncrypted):
    data = json.loads(str(dataEncrypted, 'utf-8'))
    enc = Encrypted(data['integrity'], data['request'], data['key'], data['iv'])
    decryptedData = encryptionController.prepareAes(enc)

    #check if data has been corrupted that means that integrity check has failed
    if decryptedData == None:
        response = make_response(jsonify(createResponseData("Data has been corrupted")), 401)
        response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
        return response

    return json.loads(decryptedData)

def checkifpublicKeyExists(email:str):
    if userController.getPublicKey(email) == "null" or userController.getPublicKey(email) == None:
        response = make_response(jsonify(createResponseData("Uživateľ nemá nastavený verejný RSA kľúč")), 401)
        response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
        return response
    else:
        return None

#returns None if token is valid, returns response if token is not valid
def checkIfTokenIsValidAndUserExists(user:User, token:str):
    if user == None:
        response = make_response(jsonify(createResponseData("User does not exist")), 401)
        response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
        return response

    if user.token != token:
        response = make_response(jsonify(createResponseData("Token is not valid")), 401)
        response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
        return response
    else:
        current_time = Decimal(datetime.utcnow().timestamp())
        if Decimal(user.timestamp) <= current_time:
            response = make_response(jsonify(createResponseData("Token has expired")), 401)
            response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
            return response
        else:
            userController.insertCurrentTimestamp(user.email)
            return None


@app.route('/api/arrival', methods=['POST', 'OPTIONS'])
@limiter.limit("1/second", override_defaults=False)
def arrival():    
    data = encryptionCheck(request.data)
    user = userController.getEntity(data['email'])
    
    data['start'] = dt.parse(data['start'], dayfirst=True)
    data['date'] = dt.parse(data['date'], dayfirst=True)

    response = checkifpublicKeyExists(data['email'])
    if response != None:
        return response

    response = checkIfTokenIsValidAndUserExists(user, data['token'])
    if response != None:
        return response

    if timeController.checkIfTimeRecordExists(data['email'], data['date']):
        response = make_response(jsonify(createResponseData("Start time record already exists")), 401)
        response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
    else:
        timeController.createEntityRegular(data['date'], data['email'], data['start'])
        response = make_response(jsonify(createResponseData("Start time record has been added")), 200)
        response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'

    return response


@app.route('/api/departure', methods=['POST', 'OPTIONS'])
@limiter.limit("1/second", override_defaults=False)
def departure():
    data = encryptionCheck(request.data)

    user = userController.getEntity(data['email'])
    data['end'] = dt.parse(data['end'], dayfirst=True)
    data['date'] = dt.parse(data['date'], dayfirst=True)

    response = checkifpublicKeyExists(data['email'])
    if response != None:
        return response
    
    response = checkIfTokenIsValidAndUserExists(user, data['token'])
    if response != None:
        return response

    if not timeController.checkIfTimeRecordExists(data['email'], data['date']):
        response = make_response(jsonify(createResponseData("Time record does not exist")), 401)
    elif timeController.checkIfTimeRecordEndExists(data['email'], data['date']):
        response = make_response(jsonify(createResponseData("End time record already exists")), 401)
    else:
        timeController.setEndTime(data['email'], data['date'], data['end'])
        response = make_response(jsonify(createResponseData("End time record has been added")), 200)
    
    response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
    return response


@app.route('/api/vacation', methods=['POST', 'OPTIONS'])
@limiter.limit("1/second", override_defaults=False)
def vacation():
    data = encryptionCheck(request.data)

    user = userController.getEntity(data['email'])

    data['from'] = dt.parse(data['from'])
    data['to'] = dt.parse(data['to'])

    response = checkifpublicKeyExists(data['email'])
    if response != None:
        return response
    
    response = checkIfTokenIsValidAndUserExists(user, data['token'])
    if response != None:
        return response
    
    timeController.createEntityVacation(user.email, data['from'], data['to'], data['description'])
    response = make_response(jsonify(createResponseData("Vacation has been added")), 200)
    response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
    return response


@app.route('/api/getAllData', methods=['POST', 'OPTIONS'])
# @limiter.limit("0.1/second", override_defaults=False)
def getAllData():
    data = encryptionCheck(request.data)

    time.sleep(1)
    user = userController.getEntity(data['email'])

    response = checkifpublicKeyExists(data['email'])
    if response != None:
        return response
    
    response = checkIfTokenIsValidAndUserExists(user, data['token'])
    if response != None:
        return response

    data = timeController.getMonthTimeRecords(user.email, data['month'], data['year'])
    string_list = [str(element) for element in data]
    delimiter = ","
    result_string = delimiter.join(string_list)
    result_string = result_string.encode('utf-8')

    integrity = hashlib.sha256(result_string).digest()
    integrity = base64.b64encode(integrity)

    key = createAESKey()
    iv = createAESIv()
    encryptedData = encryptionController.encryptAES(key, iv, result_string)
    encryptedData = base64.b64encode(encryptedData).decode('utf-8')
    pubKey = userController.getPublicKey(user.email)
    pubKey = encryptionController.getPublicKeyClientFormat(pubKey)
    key = encryptionController.encryptRSA(key, pubKey)
    iv = encryptionController.encryptRSA(iv, pubKey)
    integrity = encryptionController.encryptRSA(integrity, pubKey)

    finalData = {
        "key": key,
        "iv": iv,
        "response": encryptedData,
        "integrity": integrity
    }


    response = make_response(finalData, 200)
    response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
    return response


@app.route('/api/searchVacations', methods=['POST', 'OPTIONS'])
# @limiter.limit("0.1/second", override_defaults=False)
def searchVacations():
    data = encryptionCheck(request.data)

    time.sleep(1)
    user = userController.getEntity(data['email'])

    response = checkifpublicKeyExists(data['email'])
    if response != None:
        return response
    
    response = checkIfTokenIsValidAndUserExists(user, data['token'])
    if response != None:
        return response

    data = timeController.getMonthTimeRecordsByDescription(user.email, data['description'])
    
    string_list = [str(element) for element in data]
    delimiter = ","
    result_string = delimiter.join(string_list)
    result_string = result_string.encode('utf-8')

    integrity = hashlib.sha256(result_string).digest()
    integrity = base64.b64encode(integrity)

    key = createAESKey()
    iv = createAESIv()
    encryptedData = encryptionController.encryptAES(key, iv, result_string)
    encryptedData = base64.b64encode(encryptedData).decode('utf-8')
    pubKey = userController.getPublicKey(user.email)
    pubKey = encryptionController.getPublicKeyClientFormat(pubKey)
    key = encryptionController.encryptRSA(key, pubKey)
    iv = encryptionController.encryptRSA(iv, pubKey)
    integrity = encryptionController.encryptRSA(integrity, pubKey)

    finalData = {
        "key": key,
        "iv": iv,
        "response": encryptedData,
        "integrity": integrity
    }

    response = make_response(finalData, 200)
    response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
    return response


@app.route('/api/deleteRecord', methods=['POST', 'OPTIONS'])
# @limiter.limit("1/second", override_defaults=False)
def deleteRecord():
    data = encryptionCheck(request.data)

    # data['token'] = str(encryptionController.decryptRSA(data['token']), 'utf-8')
    user = userController.getEntity(data['email'])

    data['date'] = dt.parse(data['date'], dayfirst=True)

    response = checkifpublicKeyExists(data['email'])
    if response != None:
        return response

    response = checkIfTokenIsValidAndUserExists(user, data['token'])
    if response != None:
        return response

    if timeController.checkIfTimeRecordExists(data['email'], data['date']):
        timeController.deleteTimeRecord(data['email'], data['date'])
        response = make_response(jsonify(createResponseData("Time record has been deleted")), 200)
    else:
        response = make_response(jsonify(createResponseData("Time Record does not exist")), 401)

    response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
    return response


@app.route('/api/signin', methods=['POST', 'OPTIONS'])
@limiter.limit("1/second", override_defaults=False)
def signin():
    #getting ip address
    ip = request.remote_addr
    #checking if ip is in database
    ip_database:Brute = bruteController.getEntity(ip)
    if ip_database != None:
        current_time = Decimal(datetime.utcnow().timestamp())
        #checking if timestamp is still valid if it is block user and do not increase login attempts as it is unnecessary
        #if we do not allow to increase login attempts, user still wont be able to login as he will be blocked by timestamp
        #if we let user to update login attempts he may go over integer limit and it will cause problems
        if Decimal(ip_database.timestamp) >= current_time:
            difference = Decimal(ip_database.timestamp) - current_time
            difference = datetime.utcfromtimestamp(float(difference)).strftime('%M:%S')
            data = createResponseData("Too many login attempts, try again later")
            data['data']['timer'] = str(difference)
            response = make_response(jsonify(data), 401)
            response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
            return response
        else:
            difference = current_time - Decimal(ip_database.timestamp) 
            if difference > 1800:
                bruteController.resetLoginAttempts(ip)
    else:
        #if ip is not in database, insert it
        bruteController.insertEntity(ip)

    #loading data from request and decrypting it from RSA
    data = json.loads(str(request.data, 'utf-8'))
    data['email'] = str(encryptionController.decryptRSA(data['email']), 'utf-8')
    data['password'] = str(encryptionController.decryptRSA(data['password']), 'utf-8')
    #checking if user exists
    user:User = userController.getEntity(data['email'])
    if user == None:
       response = make_response(jsonify(createResponseData("Nesprávny email alebo heslo")), 401)
       bruteController.incrementLoginAttempts(ip)
    else:
        #checking if password is correct
        if encryptionController.checkPassword(data['password'], user.password.encode('utf-8')):
            loggedData = createResponseData("User has been authenticated")
            loggedData['data']['token'] = userController.createAndInsertTimestampToUser(data['email'], data['password'])

            response = make_response(jsonify(loggedData), 200)
        else:
            bruteController.incrementLoginAttempts(ip)
            response = make_response(jsonify(createResponseData("Nesprávny email alebo heslo")), 401)

    #checking if ip has more than 3 login attempts and if so, create timestamp for 10 minutes
    ip_updated = bruteController.getEntity(ip)
    if ip_updated.loginAttempts >= 3:
        bruteController.createAndInsertTimestamp(ip)

    response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
    return response


@app.route('/api/signup', methods=['POST', 'OPTIONS'])
@limiter.limit("1/second", override_defaults=False)
def signup():
    #loading data from request and decrypting it from RSA
    data = json.loads(str(request.data, 'utf-8'))
    data['email'] = str(encryptionController.decryptRSA(data['email']), 'utf-8') 
    data['password'] = str(encryptionController.decryptRSA(data['password']), 'utf-8')
    data['name'] = str(encryptionController.decryptRSA(data['name']), 'utf-8')

    if encryptionController.checkCommonPasswords(data['password']):
        response = make_response(jsonify(createResponseData("Password is too common")), 401)
        response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
        return response
    
    #checking if user already exists
    user:User = userController.getEntity(data['email'])
    if user != None:
        response = make_response(jsonify(createResponseData("Používateľ so zadaným emailom už existuje.")), 401)
    else:
        #creating new user and hashing and salting password
        userController.createEntity(data['email'], encryptionController.hashPassword(data['password']), data['name'])
        response = make_response(jsonify(createResponseData("Vytvorenie používateľa úspešné.")), 200)
    
    response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
    return response


@app.route('/api/uploadKey', methods=['POST', 'OPTIONS'])
@limiter.limit("1/second", override_defaults=False)
def uploadKey():
    #loading data from request and decrypting it from RSA
    data = json.loads(str(request.data, 'utf-8'))

    #decrypting token
    data['token'] = str(encryptionController.decryptRSA(data['token']), 'utf-8')
    
    #check token and timestamp
    user:User = userController.getEntity(data['email'])
    if user == None:
        response = make_response(jsonify(createResponseData("User does not exist")), 401)
        response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
        return response
    else:
        if user.token != data['token']:
            response = make_response(jsonify(createResponseData("Token is not valid")), 401)
            response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
            return response
        else:
            current_time = Decimal(datetime.utcnow().timestamp())
            if Decimal(user.timestamp) <= current_time:
                response = make_response(jsonify(createResponseData("Token has expired")), 401)
                response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
                return response
            else:
                userController.insertPublicKey(data['email'], data['publicKey'])
                userController.insertCurrentTimestamp(data['email'])
                response = make_response(jsonify(createResponseData("Public key has been uploaded")), 200)
                response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
                return response


@app.route('/api/sendText', methods=['POST', 'OPTIONS'])
def sendText():
    #loading data from request and decrypting it from RSA
    data = json.loads(str(request.data, 'utf-8'))

    #decrypting token
    data['token'] = str(encryptionController.decryptRSA(data['token']), 'utf-8')
    
    #check token and timestamp
    user = userController.getEntity(data['email'])
    if user == None:
        response = make_response(jsonify(createResponseData("User does not exist")), 401)
        response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
        return response
    else:
        if user.token != data['token']:
            response = make_response(jsonify(createResponseData("Token is not valid")), 401)
            response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
            return response
        else:
            current_time = Decimal(datetime.utcnow().timestamp())
            if Decimal(user.timestamp) <= current_time:
                response = make_response(jsonify(createResponseData("Token has expired")), 401)
                response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
                return response
            else:
                decryptedText = encryptionController.decryptRSA(data['sendText'])
                textPubKey = userController.getPublicKey(data['email'])
                pubKey = encryptionController.getPublicKeyClientFormat(textPubKey)
                encryptedText = encryptionController.encryptRSA(decryptedText, pubKey)

                #check if client has set public key
                
                if userController.getPublicKey(data["email"]) == "null":
                    response = make_response(jsonify(createResponseData("Client has not set public key")), 401)
                    response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
                    return response

                userController.insertCurrentTimestamp(data['email'])
                response = make_response(jsonify(createResponseData(encryptedText)), 200)
                response.headers['Access-Control-Allow-Origin'] = 'http://localhost:3000'
                return response
