from controllers.BaseEntityController import BaseEntityController
from entities.Brute import Brute
from flask_mysqldb import MySQL
import json
from DecimalEncoder import DecimalEncoder
from datetime import datetime

class BruteController(BaseEntityController):
    def __init__(self, sql: MySQL):
        super().__init__(sql)

    def getEntity(self, ip: str):
        cur = self.sql.connection.cursor()
        cur.execute("SELECT * FROM brute WHERE ip = %s", (ip,))
        rv = cur.fetchone()

        if rv is None:
            return None
        
        json_string = json.dumps(rv, cls=DecimalEncoder)
        brute_json = json.loads(json_string)
        brute:Brute = Brute(brute_json["ip"], brute_json["timestamp"], brute_json["login_attempts"])
        return brute
    
    def insertEntity(self, ip: str):
        brute:Brute = Brute(ip)
        cur = self.sql.connection.cursor()
        cur.execute("INSERT INTO brute (ip, timestamp, login_attempts) VALUES (%s, %s, %s)", (brute.id, brute.timestamp, brute.loginAttempts))
        self.sql.connection.commit()

    def incrementLoginAttempts(self, ip: str):
        cur = self.sql.connection.cursor()
        cur.execute("UPDATE brute SET login_attempts = login_attempts + 1 WHERE ip = %s", (ip,))
        self.sql.connection.commit()
    
    def resetLoginAttempts(self, ip: str):
        cur = self.sql.connection.cursor()
        cur.execute("UPDATE brute SET login_attempts = 0 WHERE ip = %s", (ip,))
        self.sql.connection.commit()

    def createAndInsertTimestamp(self, ip: str):
        timeStamp = datetime.utcnow().timestamp() + 600
        cur = self.sql.connection.cursor()
        cur.execute("UPDATE brute SET timestamp = %s WHERE ip = %s", (timeStamp, ip,))
        self.sql.connection.commit()
        cur.close()
        