from controllers.BaseEntityController import BaseEntityController
from flask_mysqldb import MySQL
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
import bcrypt
# from cryptography.hazmat.primitives import padding
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.serialization import load_der_public_key
import base64
import os
import hashlib

from entities.Encrypted import Encrypted



class EncryptionController(BaseEntityController):
    def __init__(self, sql: MySQL):
        super().__init__(sql)

    def checkIntegrity(self, recievedIntegrity, recievedRequest):
        #checking integrity of request
        integrity = hashlib.sha256(recievedRequest.encode()).digest().hex()
        rec = base64.b64decode(recievedIntegrity)
        rec = rec.decode('utf-8')

        if  rec == integrity:
            return True
        else:
            return False

    def prepareAes(self, enc: Encrypted):
        enc.key = self.decryptRSA(enc.key)

        enc.iv = self.decryptRSA(enc.iv)

        enc.integrity = self.decryptRSA(enc.integrity)

        enc.request = base64.b64decode(enc.request)
        pepek = self.decryptAES(enc.key, enc.iv, enc.request)

        if(self.checkIntegrity(enc.integrity, pepek.decode('utf-8'))):
            return pepek.decode('utf-8')
        else:
            return None        

    def encryptAES(self, key, iv, data):
        cipher = Cipher(algorithms.AES(key), modes.CFB(iv))
        encryptor = cipher.encryptor()
        ct = encryptor.update(data) + encryptor.finalize()
        return ct
    
    def decryptAES(self, key:bytes, iv:bytes, data:bytes):
        cipher = Cipher(algorithms.AES(key), modes.CFB(iv))
        decryptor = cipher.decryptor()
        pt = decryptor.update(data) + decryptor.finalize()
        return pt
    
    def hashPassword(self, password):
        return bcrypt.hashpw(password.encode('utf-8'), bcrypt.gensalt(5)).decode('utf-8')

    def checkPassword(self, password, hashedPassword):
        if bcrypt.checkpw(password.encode('utf-8'), hashedPassword):
            return True
        else:
            return False
        
    def decryptRSA(self, data):
        privateKey = self.getPrivateKey()
        plaintext = privateKey.decrypt(
        base64.b64decode(data),
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        ))
        return plaintext
    
    def encryptRSA(self, data, publicKey):
        encryptedText = publicKey.encrypt(
            data,
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None
            )
        )
        return base64.b64encode(encryptedText).decode('utf-8')
    
    def getPrivateKey(self):
        #Getting private key of server
        file_path = "/backend/serverKeys/Private_Server_KEY"
        with open(file_path, "rb") as key_file:
            private_key = serialization.load_pem_private_key(
                key_file.read(),
                password=None,
        )
        return private_key
    
    def getPublicKeyClientFormat(self, pubKey):
        b64data = '\n'.join(pubKey.splitlines()[1:-1])
        public_key_client = load_der_public_key(base64.b64decode(b64data), default_backend())
        return public_key_client
    
    def checkCommonPasswords(self, password):
        #checking if password is in database of common passwords
        file_path = "/backend/passwordDatabase/10-million-password-list-top-1000000.txt"
        commonPasswords = open(file_path, "r").read().split()
        if(password in commonPasswords):
            return True
        else:
            return False
        
        