from entities.TimeRecord import TimeRecord
from controllers.BaseEntityController import BaseEntityController
from flask_mysqldb import MySQL
from datetime import datetime, timedelta
from entities.FrontendTimeRecord import FrontendTimeRecord
import dateutil.parser as dt
class TimeRecordController(BaseEntityController):
    def __init__(self, sql: MySQL):
        super().__init__(sql)

    def checkIfTimeRecordExists(self, email:str, date:datetime):
        cur = self.sql.connection.cursor()
        cur.execute("SELECT * FROM time WHERE user_id = %s AND date = %s", (email, date))
        result = cur.fetchone()
        cur.close()
        if result is not None:
            return True
        else:
            return False
        
    def deleteTimeRecord(self, email:str, date:datetime):
        cur = self.sql.connection.cursor()
        cur.execute("DELETE FROM time WHERE user_id = %s AND date = %s", (email, date))
        self.sql.connection.commit()
        cur.close()
    
    def checkIfTimeRecordEndExists(self, email:str, date:datetime):
        cur = self.sql.connection.cursor()
        cur.execute("SELECT * FROM time WHERE user_id = %s AND date = %s AND end IS NOT NULL", (email, date))
        result = cur.fetchone()
        cur.close()

        if result is not None:
            return True
        else:
            return False
        
    def setEndTime(self, email:str, date:datetime, end:datetime):
        cur = self.sql.connection.cursor()
        cur.execute("UPDATE time SET end = %s WHERE user_id = %s AND date = %s", (end, email, date))
        self.sql.connection.commit()
        cur.close()

    def createEntityRegular(self, date:datetime, userId, start:datetime):
        cur = self.sql.connection.cursor()
        cur.execute("INSERT INTO time (date, user_id, start, priority, month, year) VALUES (%s, %s, %s, %s, %s, %s)", (date, userId, start, 1, date.month, date.year))
        self.sql.connection.commit()
        cur.close()
        
    def createEntityVacation(self, userId, start:datetime, end:datetime, description):
        #get dates between start and end
        dates = []
        dates.append(start)
        while start < end:
            # start = datetime.strptime(start, "%Y-%m-%d")  # replace "%Y-%m-%d" with your date format
            start += timedelta(days=1)
            dates.append(start)
        dates.append(end)

        for vacDate in dates:
            if self.checkIfTimeRecordExists(userId, vacDate) or vacDate.weekday() >= 5:
                continue
            cur = self.sql.connection.cursor()
            cur.execute("INSERT INTO time (date, user_id, priority, month, year, description) VALUES (%s, %s, %s, %s, %s, %s)", (vacDate, userId, 2, vacDate.month, vacDate.year, description))
            self.sql.connection.commit()
            cur.close()

    def getMonthTimeRecords(self, userId: str, month: int, year: int):
        cur = self.sql.connection.cursor()
        cur.execute("SELECT * FROM time WHERE user_id = %s AND month = %s AND year = %s", (userId, month, year))
        result = cur.fetchall()
        cur.close()
        dateFormat = '%Y-%m-%d %H:%M:%S'
        dateTimeFormat = '%Y-%m-%d %H:%M:%S'
        

        timeRecords = []
        for row in result:
            date:str = row['date']
            start:str = row['start']
            end:str = row['end']
            description:str = row['description']
            
            date = datetime.strptime(date, dateFormat)
            start = row['start']
            end = row['end']
            description = row['description']
            timeRecords.append(FrontendTimeRecord(date, start, end, description).getJSON())
        return timeRecords
    
    def getMonthTimeRecordsByDescription(self, userId: str, description: str):
        cur = self.sql.connection.cursor()
        cur.execute("SELECT * FROM time WHERE user_id = %s AND description LIKE %s", (userId, f"%{description}%"))
        result = cur.fetchall()
        cur.close()
        dateFormat = '%Y-%m-%d %H:%M:%S'
        dateTimeFormat = '%Y-%m-%d %H:%M:%S'

        timeRecords = []
        for row in result:
            date:str = row['date']
            start:str = row['start']
            end:str = row['end']
            description:str = row['description']
            
            date = datetime.strptime(date, dateFormat)
            start = row['start']
            end = row['end']
            description = row['description']
            timeRecords.append(FrontendTimeRecord(date, start, end, description).getJSON())
        return timeRecords