from entities.User import User
from controllers.BaseEntityController import BaseEntityController
from controllers.EncryptionController import EncryptionController
from flask_mysqldb import MySQL
import json
from DecimalEncoder import DecimalEncoder
from datetime import datetime
import os
import base64
import hashlib

class UserController(BaseEntityController):
    def __init__(self, sql: MySQL):
        super().__init__(sql)
        self.encrypt = EncryptionController(sql)

    def createEntity(self, email, password, name):
        cur = self.sql.connection.cursor()
        cur.execute("INSERT INTO users (email, password, name) VALUES (%s, %s, %s)", (email, password, name))
        self.sql.connection.commit()
        cur.close()
        
    def getEntity(self, email:str):
        cur = self.sql.connection.cursor()
        cur.execute("SELECT * FROM users WHERE email = %s", (email,))
        rv = cur.fetchone()
        cur.close()

        if rv is None:
            return None

        json_string = json.dumps(rv, cls=DecimalEncoder)
        user_json = json.loads(json_string)
        
        return User(user_json['id'], user_json['email'], user_json['name'], user_json['token'], user_json['timestamp'], user_json['public_key'], user_json['password']) 
    
    def createAndInsertTimestampToUser(self, email: str, password):
        timeStamp = datetime.utcnow().timestamp() + 600

        token = os.urandom(32)
        token = base64.b64encode(token).decode('utf-8')

        cur = self.sql.connection.cursor()
        cur.execute("UPDATE users SET timestamp = %s, token = %s WHERE email = %s", (timeStamp, token, email,))
        self.sql.connection.commit()
        cur.close()

        # preparing and encrypting data from database via AES CFB
        iv = hashlib.sha256(email.encode()).digest()[:16] 
        aesKey = hashlib.sha256(password.encode()).digest()
        bytes_object = bytes(token, 'utf-8')
        encryptedAESdata = self.encrypt.encryptAES(aesKey,iv, bytes_object)

        return base64.b64encode(encryptedAESdata).decode('utf-8')
    
    def insertPublicKey(self ,email: str, publicKey):
        cur = self.sql.connection.cursor()
        cur.execute("UPDATE users SET public_key = %s WHERE email = %s", (publicKey, email,))
        self.sql.connection.commit()
        cur.close()
    
    def getPublicKey(self, email:str):
        cur = self.sql.connection.cursor()
        cur.execute("SELECT public_key FROM users WHERE email = %s", (email,))
        result = cur.fetchone()
        cur.close()
        if result is None or result == 'null':
            return None
        else:
            return result['public_key']

    def insertCurrentTimestamp(self, email: str):
        timeStamp = datetime.utcnow().timestamp() + 600

        cur = self.sql.connection.cursor()
        cur.execute("UPDATE users SET timestamp = %s WHERE email = %s", (timeStamp, email,))
        self.sql.connection.commit()
        cur.close()