import React, { useState, useEffect } from "react"
import { useNavigate } from 'react-router-dom';
var forge = require('node-forge');

export default function AuthForm(props) {
    const navigate = useNavigate();
    //Public key for server
    const [publicKeyServer,setPublicKeyServer] = useState(
      `-----BEGIN PUBLIC KEY-----
      MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAxhPA0dEouX2Obuhag9H2
      wh6L/Gj9llKcuCsf3b/ndbRqxLbAYmpCQwYRvhUmb3HYttJ7pHJLoBlO1/i0cM4p
      PaJTMc4mmwX7FB5JgvqvYXl4KRu/TOnmPImZBdaCghnbqKfFK1gFDtmb3eABXoN+
      3Q49qzJqS4akui4H+ZLWKGrjO08XwvMpGE651ZOBMufMkkzUwwN8L2XfLKMJJiEJ
      jVC7ape+Giq1yL4oK9XAXslu9wZ0zlrCdbfueznLJZGo3O2l6bIv6Wj81ZFBJuGh
      s+LQMohTxStnjhmIY74H65ElcO0PlDo8dU5ZTfHf3IVULcrElbRXFFd/N3l3rQ/p
      IWtNLI83gjuiFC05Y8aH3ahK4mVV2UKJXxIeCBHTAvkCKxOsVG780/O51rksBN0c
      EoivUN4N54s94NpaLSSiA2wdvhA4Io9UKXuMrbDu1BsAw2++AJ9j9JGHeWqB40Hr
      zMnMCcO/3KUL7BNJ9xYgJ9dJOAGDOMxZJ43BtQyk+bRg/xzuCJRDoH98jD9c0i+b
      u4OCzEL2MYpFTxkha57I+kK7HhY0layrrRhU1Xn9aWA6w3QvVl+pHqa7d9olZveW
      PWAJvl+3Vqzj7AVRaVlJEI686Hs8+9Tjq6cGH3aY7ZOIoBu9Y2sKGyr48reZe2ig
      dyPh2x3+s58xLfds1iwfgjMCAwEAAQ==
      -----END PUBLIC KEY-----`)
  let [authMode, setAuthMode] = useState("signin");

  const [formData, setFormData] = useState({
    name: "",
    email: "",
    password: ""
  });

  const [passwordError, setPasswordError] = useState('Heslo musí obsahovať viac ako 12 znakov.');

  const [emailError, setEmailError] = useState('Email nesmie byť prázdny.');

  const [nameError, setNameError] = useState('Meno a priezvisko nesmia byť prázdne.');

  const [responseMessage, setResponseMessage] = useState('');

  const [timer, setTimer] = useState('');

  const [redirect, setRedirect] = useState(false);
  const [token, setToken] = useState('');

   //loading public key from server for encrypting FormData
   var publicKeyServerEnc = forge.pki.publicKeyFromPem(publicKeyServer);

  const rsaEncryptString = (data) => {
    
    var encrypted = publicKeyServerEnc.encrypt(data, 'RSA-OAEP', {
      md: forge.md.sha256.create(),
      mgf1: forge.mgf1.create()
    });
    return forge.util.encode64(encrypted);
  }

  const encryptFormData = (formData) => {
    const encryptedFormData = {
      email: rsaEncryptString(formData.email),
      name: formData.name !== "" ? rsaEncryptString(formData.name):"",
      password: rsaEncryptString(formData.password)
    }
    return encryptedFormData;

  }

  const changeAuthMode = () => {
    setAuthMode(authMode === "signin" ? "signup" : "signin");
    setResponseMessage("");
    setTimer("");
  };

  const handleChange = (e) => {
    const { name, value } = e.target;
    setFormData((prev) => ({ ...prev, [name]: value }));
    if (name === "password")
      passwordHandleBlur(value);
    else if (name === "email")
      emailHandleBlur(value);
    else if (name === "name")
      nameHandleBlur(value);
  };

  const nameHandleBlur = (name) => {
      const error = validateName(name);
      setNameError(error);
  };

  const passwordHandleBlur = (password) => {
      const error = validatePassword(password);
      setPasswordError(error);
  };

  const emailHandleBlur = (email) => {
      const error = validateEmail(email);
      setEmailError(error);
  };

  const validatePassword = (password) => {
      const errorMessages = {
          length: 'Heslo musí obsahovať viac ako 12 znakov.',
          number: 'Heslo musí obsahovať aspoň jedno číslo.',
          letterCase: 'Heslo musí obsahovať veľké aj malé písmená.',
          specialCharacter: 'Heslo musí obsahovať aspoň jeden špeciálny znak.',
      };

      if (password.length < 13) {
          return errorMessages.length;
      } else if (!/\d/.test(password)) {
          return errorMessages.number;
      } else if (!/[A-Z]/.test(password) || !/[a-z]/.test(password)) {
          return errorMessages.letterCase;
      } else if (!/[^a-zA-Z0-9]/.test(password)) {
          return errorMessages.specialCharacter;
      } else {
          return '';
      }
  };

  const validateEmail = (email) => {
      const errorMessages = {
          emptiness: 'Email nesmie byť prázdny.',
          format: 'Neplatný formát emailu.'
      };

      const emailRegex = /^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Za-z]{2,}$/;

      if (email.length == 0) {
          return errorMessages.emptiness;
      } else if (!emailRegex.test(email)) {
          return errorMessages.format;
      } else {
          return '';
      }
  };

  const validateName = (name) => {
      const errorMessages = {
          emptiness: 'Meno a priezvisko nesmia byť prázdne.',
      };

      if (name.length == 0) {
          return errorMessages.emptiness;
      } else {
          return '';
      }
  };

  const handleSubmit = async (e) => {
    e.preventDefault();
    const { name, value } = e.target;
    setFormData((prev) => ({ ...prev, [name]: value }));

    const url = authMode === "signin" ? "http://localhost:5000/api/signin" : "http://localhost:5000/api/signup";
    const requestOptions = {
      method: 'POST',
      headers: {
        'Accept': 'application/json',
      },
      body: JSON.stringify(encryptFormData(formData)),
    };
    
    try {
      const response = await fetch(url, requestOptions);
      const responseBody = await response.json();
      
      if (responseBody.data) {

        //if too many login attempts, set timer to display
        if(responseBody.data.message === "Príliš veľa pokusov o prihlásenie."){
          setTimer(responseBody.data.timer);
        }else{
          setTimer('');
        }

        //set response message displayed to user
        setResponseMessage(responseBody.data.message);

        if(responseBody.data.message === "User has been authenticated"){

          //prepare data fro decryption from base64 to ByteStringBuffer
          var unpreparedData = forge.util.decode64(responseBody.data.token)
          var buffer = forge.util.createBuffer(unpreparedData, 'utf-8')

          //Hash password for AES key
          var md = forge.md.sha256.create();
          md.update(formData['password']);
          var key = md.digest();

          //Hash email for AES IV
          var md = forge.md.sha256.create();
          md.update(formData['email']);
          var iv = md.digest().getBytes(16);
    
           //Decrypt Symetrically encrypted data from server in AES CFB - mode
          var cipher = forge.aes.createDecryptionCipher(key, 'CFB');
          cipher.start(iv);
          cipher.update(buffer);
          cipher.finish();

          let tokenDecrypted = cipher.output.toString()

          setToken(tokenDecrypted);
          setRedirect(true);
        }
      } else {
        setResponseMessage("unsuccessful");
        console.log("unsuccessful");
      }
      
    } catch (error) {
      setResponseMessage("Chyba počas autentifikácie.");
      console.error("Error during authentication:", error);
    }
  };

  useEffect(() => {
    if (redirect) {
      const data = {
        email: formData.email,
        token: token,
        privateKey: ""
      }
  
      navigate('/dashboard', { state: { data } });
    }
  }, [redirect]);


  if (authMode === "signin") {
    return (
      <div className="Auth-form-container">
        <form className="Auth-form">
          <div className="Auth-form-content">
            <h3 className="Auth-form-title">Prihlásenie</h3>
            <div className="form-group mt-3">
              <label>Email</label>
              <input
                type="email"
                name="email"
                onChange={handleChange}
                value={formData.email}
                className="form-control mt-1"
                maxLength="200"
                placeholder=""
              />
            </div>
            <div className="form-group mt-3">
              <label>Heslo</label>
              <input
                type="password"
                name="password"
                value={formData.password}
                onChange={handleChange}
                className="form-control mt-1"
                maxLength="200"
                placeholder=""
              />
              </div>
            <div className="d-grid gap-2 mt-3">
             <button onClick={handleSubmit} type="submit" className="btn btn-primary" style={{backgroundColor: 'rgb(33,37,41)', borderColor: 'rgb(33,37,41)'}}>
                Prihlásiť
              </button>
            </div>
            {(typeof responseMessage === "undefined" || responseMessage === "") ? (
              <p style={{color: 'red'}}></p>
            ): (
              <p className="text-center d-grid gap-2 mt-3" style={{color: 'red'}}>{responseMessage}</p>
            )}
            {(typeof timer === "undefined" || timer === "") ? (
              <p style={{color: 'red'}}></p>
            ): (
              <p className="text-center d-grid gap-2 mt-3" style={{color: 'red'}}>{timer}</p>
            )}
            <div className="text-center d-grid gap-2 mt-3">
              {"Nemáte vytvorený účet? "}
              <span className="link-primary" onClick={changeAuthMode} >
                Registrovať
              </span>
            </div>
          </div>
        </form>
      </div>
    )
  }

  return (
    <div className="Auth-form-container">
      <form className="Auth-form">
        <div className="Auth-form-content">
          <h3 className="Auth-form-title">Registrácia</h3>
          <div className="form-group mt-3">
            <label>Meno a priezvisko</label>
            <input
              type="text"
              name="name"
              value={formData.name}
              onChange={handleChange}
              className="form-control mt-1"
              maxLength="50"
              placeholder="Maximálna dĺžka 50 znakov"
            />
          </div>
            {nameError && <p className="text-danger">{nameError}</p>}
          <div className="form-group mt-3">
            <label>Email</label>
            <input
              type="email"
              name="email"
              onChange={handleChange}
              value={formData.email}
              className="form-control mt-1"
              maxLength="50"
              placeholder="Maximálna dĺžka 50 znakov"
            />
          </div>
            {emailError && <p className="text-danger">{emailError}</p>}
          <div className="form-group mt-3">
            <label>Heslo</label>
            <input
              type="password"
              name="password"
              value={formData.password}
              onChange={handleChange}
              className="form-control mt-1"
              maxLength="50"
              placeholder="Maximálna dĺžka 50 znakov"
            />
            {passwordError && <p className="text-danger">{passwordError}</p>}
          </div>
          <div className="d-grid gap-2 mt-3">
            <button onClick={handleSubmit} disabled={passwordError !== '' || nameError !== '' || emailError !== ''} type="submit" className="btn btn-primary" style={{backgroundColor: 'rgb(33,37,41)', borderColor: 'rgb(33,37,41)'}}>
              Registrovať
            </button>
          </div>
          {(typeof responseMessage === "undefined" || responseMessage === "") ? (
            <p className= "text-center d-grid gap-2 mt-3" style={{color: 'red'}}></p>
          ): (
            <p className="text-center d-grid gap-2 mt-3" style={{color: 'red'}}>{responseMessage}</p>
          )}
          <div className="text-center d-grid gap-2 mt-3">
            Máte vytvorený účet?{" "}
            <span className="link-primary" onClick={changeAuthMode}>
              Prihlásiť
            </span>
          </div>
        </div>
      </form>
    </div>
  )
}