import React, { useEffect, useState, useRef } from 'react'
import { useLocation } from 'react-router-dom';
import moment from 'moment';
import { sk } from 'date-fns/locale'
import {
    Container,
    Row,
    Col,
    ListGroup,
    Button,
    Card,
    Nav,
    OverlayTrigger,
    Tooltip,
    Modal,
    Table
} from 'react-bootstrap';

import CustomNavbar from './NavBar';
import html2canvas from 'html2canvas';
import jsPDF from 'jspdf';

import {
  Calendar,
  dateFnsLocalizer,
} from 'react-big-calendar'
import format from 'date-fns/format'
import parse from 'date-fns/parse'
import startOfWeek from 'date-fns/startOfWeek'
import getDay from 'date-fns/getDay' 
import Datepicker from 'react-datepicker'
import 'react-big-calendar/lib/css/react-big-calendar.css';
import { set } from 'date-fns';
import { fi } from 'date-fns/locale';

var forge = require('node-forge');


function convertTimeFormat(inputTime) {
  try {
      const dateObj = new Date(inputTime);

      const day = ('0' + dateObj.getDate()).slice(-2);
      const month = ('0' + (dateObj.getMonth() + 1)).slice(-2);
      const year = dateObj.getFullYear();
      
      const outputTime = `${day}.${month}.${year}`;

      return outputTime;

  } catch (error) {
    if (error.response && error.response.data && error.response.data.message) {
        console.log(error.response.data.message);
    } else {
        console.log(error);
    }
        return "Invalid input time format";
  }
}

function Logged(props) {
    let location = useLocation();
    const data = location.state.data ? location.state.data : location.state;

    const [newEvent, setNewEvent] = useState({ title: '', start: '', end: '', description: '' })
    const [allEvents, setallEvents] = useState([])
    const [allEventsRecord, setallEventsRecord] = useState([])
    const [showModal, setShowModal] = useState(false);
    const [vacationDescription, setVacationDescription] = useState('');
    const [searchVacationDescription, setSearchVacationDescription] = useState('');
    const [selectedDay, setSelectedDay] = useState(new Date());
    const [getAllData, setgetAllData] = useState(true)
    const [privateKey, setprivateKey] = useState("")
    const [inputProps, setInputProps] = useState(data);
    const [dataJson, setDataJson] = useState("empty")
    const [err, setErr] = useState('');
    const [publicKeyServer, setPublicKeyServer] = useState(
        `-----BEGIN PUBLIC KEY-----
    MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAxhPA0dEouX2Obuhag9H2
    wh6L/Gj9llKcuCsf3b/ndbRqxLbAYmpCQwYRvhUmb3HYttJ7pHJLoBlO1/i0cM4p
    PaJTMc4mmwX7FB5JgvqvYXl4KRu/TOnmPImZBdaCghnbqKfFK1gFDtmb3eABXoN+
    3Q49qzJqS4akui4H+ZLWKGrjO08XwvMpGE651ZOBMufMkkzUwwN8L2XfLKMJJiEJ
    jVC7ape+Giq1yL4oK9XAXslu9wZ0zlrCdbfueznLJZGo3O2l6bIv6Wj81ZFBJuGh
    s+LQMohTxStnjhmIY74H65ElcO0PlDo8dU5ZTfHf3IVULcrElbRXFFd/N3l3rQ/p
    IWtNLI83gjuiFC05Y8aH3ahK4mVV2UKJXxIeCBHTAvkCKxOsVG780/O51rksBN0c
    EoivUN4N54s94NpaLSSiA2wdvhA4Io9UKXuMrbDu1BsAw2++AJ9j9JGHeWqB40Hr
    zMnMCcO/3KUL7BNJ9xYgJ9dJOAGDOMxZJ43BtQyk+bRg/xzuCJRDoH98jD9c0i+b
    u4OCzEL2MYpFTxkha57I+kK7HhY0layrrRhU1Xn9aWA6w3QvVl+pHqa7d9olZveW
    PWAJvl+3Vqzj7AVRaVlJEI686Hs8+9Tjq6cGH3aY7ZOIoBu9Y2sKGyr48reZe2ig
    dyPh2x3+s58xLfds1iwfgjMCAwEAAQ==
    -----END PUBLIC KEY-----`)

    const fullCalendarRef = useRef();

    if (data.privateKey != '' && privateKey == '') {
        setprivateKey(data.privateKey)
    }

    var publicKey = forge.pki.publicKeyFromPem(publicKeyServer);

    const rsaEncryptString = (data) => {

        var encrypted = publicKey.encrypt(data, 'RSA-OAEP', {
            md: forge.md.sha256.create(),
            mgf1: forge.mgf1.create()
        });
        return forge.util.encode64(encrypted);
    }

    const handleVacationDescriptionModalClose = () => {
        setShowModal(false);
    };

    function checkIfPrivateKeyIsNotSet() {
        if (privateKey == '' || privateKey == null || privateKey == undefined){
            alert("Nastavte si svoj private key v nastaveniach");
            return true;
        }
    }

    const handlePrichod = async (e) => {
        if (checkIfPrivateKeyIsNotSet()) {
            return;
        }

        const start = new Date();
        start.setHours(start.getHours() + 1);
        const day = start.getDate();
        const month = start.getMonth() + 1;
        const year = start.getFullYear();
        const todayFormatted = `${day}.${month}.${year}`;

        const url = 'http://localhost:5000/api/arrival';

        var requestBody = handleSendingEncryptedData({
            email: data.email,
            token: data.token,
            start: start,
            date: todayFormatted
        })

        const requestOptions = {
            method: 'POST',
            headers: {
                'Accept': 'application/json',
            },
            body: JSON.stringify(requestBody),
        };

        try {
            const response = await fetch(url, requestOptions);
            const responseBody = await response.json();

            if (responseBody.data) {
            }
            setgetAllData(true)
        } catch (error) {
            if (error.response && error.response.data && error.response.data.message) {
                console.log(error.response.data.message);
            } else {
                console.log(error);
            }
        }
    }

    const handleOdchod = async () => {
        if (checkIfPrivateKeyIsNotSet()) {
            return;
        }
        const end = new Date();
        end.setHours(end.getHours() + 1);
        const day = end.getDate();
        const month = end.getMonth() + 1;
        const year = end.getFullYear();
        const todayFormatted = `${day}.${month}.${year}`;

        var requestBody = handleSendingEncryptedData({
            email: data.email,
            token: data.token,
            end: end,
            date: todayFormatted
        })

        const url = 'http://localhost:5000/api/departure';

        const requestOptions = {
            method: 'POST',
            headers: {
                'Accept': 'application/json',
            },
            body: JSON.stringify(requestBody),
        };

        try {
            const response = await fetch(url, requestOptions);
            const responseBody = await response.json();

            if (responseBody.data) {
            }
            setgetAllData(true)
        } catch (error) {
            if (error.response && error.response.data && error.response.data.message) {
                console.log(error.response.data.message);
            } else {
                console.log(error);
            }
        }
    }

    const handlePrintPDF = async () => {
        try {
            const calendarElement = document.getElementById('timeRecord');

            if (calendarElement) {
                const canvas = await html2canvas(calendarElement);
                const imgData = canvas.toDataURL('image/png');

                 // Convert the dimensions from pixels to mm
                const widthInMm = canvas.width * 0.264583;
                const heightInMm = canvas.height * 0.264583;

                const pdf = new jsPDF({
                    orientation: 'landscape',
                    unit: 'mm',
                    format: [widthInMm, heightInMm],
                });

                const imgWidth = pdf.internal.pageSize.getWidth();
                const imgHeight = (canvas.height * imgWidth) / canvas.width;

                pdf.addImage(imgData, 'PNG', 0, 0, widthInMm, heightInMm);
                pdf.save('monthly-calendar.pdf');
            } else {
                console.error("TimeRecord element not found.");
            }
        } catch (error) {
            console.error("Error exporting calendar:", error);
        }
    }

    const handleDovolenka = async () => {
        if (checkIfPrivateKeyIsNotSet()) {
            return;
        }

        setShowModal(true);
    }

    //Create vacation with/without description
    const handleModalSubmit = async () => {
        const from = document.getElementById('start_date').value;
        const to = document.getElementById('end_date').value;

        if (!from || !to) {
            console.log('Please fill in both start and end dates');
            return;
        }

        var requestBody = handleSendingEncryptedData({
            email: data.email,
            token: data.token,
            from: from,
            to: to,
            description: vacationDescription
        })

        const url = 'http://localhost:5000/api/vacation';

        const requestOptions = {
            method: 'POST',
            headers: {
                'Accept': 'application/json',
            },
            body: JSON.stringify(requestBody),
        };

        try {
            const response = await fetch(url, requestOptions);
            const responseBody = await response.json();

            setgetAllData(true)
        } catch (error) {
            if (error.response && error.response.data && error.response.data.message) {
                console.log(error.response.data.message);
            } else {
                console.log(error);
            }
        }

        setShowModal(false);
    }

    const handleSearchSubmit = async () => {
        const url = 'http://localhost:5000/api/searchVacations';

        var requestBody = handleSendingEncryptedData({
            email: data.email,
            token: data.token,
            description: searchVacationDescription
        })

        const requestOptions = {
            method: 'POST',
            headers: {
                'Accept': 'application/json',
            },
            body: JSON.stringify(requestBody)
        };

        try {
            const response = await fetch(url, requestOptions);
            const responseBody = handleReceivingEncryptedData(await response.json());

            var tempArr = []
            let temp = null

            responseBody.forEach((element) => {
                let date = new Date(element.date)
                

                if (element.end != null) {
                    temp = {
                        title: "Práca",
                        start: new Date(element.start),
                        end: new Date(element.end),
                        description: element.description == "" ? "Práca" : element.description
                    }
                } else if (element.start != null) {
                    temp = {
                        title: "Práca",
                        start: new Date(element.start),
                        end: new Date(element.start),
                        description: element.description == "" ? "Práca" : element.description
                    }
                } else {
                    temp = {
                        title: "Dovolenka",
                        start: new Date(date.setHours(8)),
                        end: new Date(date.setHours(date.getHours() + 8)),
                        description: element.description == "" ? "Dovolenka" : element.description
                    }
                }

                if(temp != null && temp != undefined){
                    console.log(temp)
                    tempArr.push(temp)
                    
                }

            });

            setallEventsRecord(tempArr)
        } catch (error) {
            if (error.response && error.response.data && error.response.data.message) {
                console.log(error.response.data.message);
            } else {
                console.log(error);
            }
        }
    }

    useEffect(() => {
        if (getAllData) {
            if (checkIfPrivateKeyIsNotSet()) {
                return;
            }
            const fetchData = async () => {

                var requestBody = handleSendingEncryptedData({
                    email: data.email,
                    token: data.token,
                    month: selectedDay.getMonth() + 1,
                    year: selectedDay.getFullYear(),
                })

                const url = 'http://localhost:5000/api/getAllData';
                const requestOptions = {
                    method: 'POST',
                    headers: {
                        'Accept': 'application/json',
                    },
                    body: JSON.stringify(requestBody),
                };

                try {
                    const response = await fetch(url, requestOptions);
                    const responseBody = await response.json();
                    if (responseBody && response.status == 200 ) {
                        let response = handleReceivingEncryptedData(responseBody)
                        if (response.length == 0) {
                            setallEvents([])
                            return
                        }
                        
                        var tempArr = []
                        let temp = null

                        response.forEach((element) => {
                            let date = new Date(element.date)
                            

                            if (element.end != null) {
                                temp = {
                                    title: "Práca",
                                    start: new Date(element.start),
                                    end: new Date(element.end),
                                    description: element.description == "" ? "Práca" : element.description
                                }
                            } else if (element.start != null) {
                                temp = {
                                    title: "Práca",
                                    start: new Date(element.start),
                                    end: new Date(element.start),
                                    description: element.description == "" ? "Práca" : element.description
                                }
                            } else {
                                temp = {
                                    title: "Dovolenka",
                                    start: new Date(date.setHours(8)),
                                    end: new Date(date.setHours(date.getHours() + 8)),
                                    description: element.description == "" ? "Dovolenka" : element.description
                                }
                            }

                            if(temp != null && temp != undefined){
                                console.log(temp)
                                tempArr.push(temp)
                            }

                        });

                        setallEventsRecord(tempArr)
                        setallEvents(tempArr)
                    }
                } catch (error) {
                    if (error.response && error.response.data && error.response.data.message) {
                        console.log(error.response.data.message);
                    } else {
                        console.log(error);
                    }
                    
                }
            };

            fetchData();

            if (data.privateKey != '' && privateKey == '') {
                setprivateKey(data.privateKey)
            }

            setgetAllData(false);
        }

    }, [privateKey, getAllData]);

    const locales = {
        sk: sk    
    }

    const localizer = new dateFnsLocalizer({
        format,
        parse,
        startOfWeek,
        getDay,
        locales,
    })

    function handleAddEvent() {
        setallEvents([...allEvents, newEvent])
    }

    const handleDeleteEvent = async (event) => { 
        if (checkIfPrivateKeyIsNotSet()) {
            return;
        }
        
        const deleteDate = new Date(event)
        const day = deleteDate.getDate();
        const month = deleteDate.getMonth() + 1;
        const year = deleteDate.getFullYear();
        const todayFormatted = `${day}.${month}.${year}`;

        var requestBody = handleSendingEncryptedData({
            email: data.email,
            token: data.token,
            date: todayFormatted,
        })

        fetch('http://localhost:5000/api/deleteRecord', {
                method: 'POST',
                headers: {
                'Accept': 'application/json'
                },
                body: JSON.stringify(requestBody)
            })
            .then(response => {
                if (response.ok) {
                    response.json().then(json => {
                        console.log(json)
                    })
                    
                // handle successful response
                } else {
                // handle error response
                }
            })
            .catch(error => {
                if (error.response && error.response.data && error.response.data.message) {
                    console.log(error.response.data.message);
                } else {
                    console.log(error);
                }
        });
    }

    function handleSendingEncryptedData(reqBody) {
        var finalData = {
            integrity: null,
            request: null,
            key: null,
            iv: null,
        }

        if(reqBody){
            reqBody = JSON.stringify(reqBody)

            var integrity = forge.md.sha256.create();
            integrity.update(reqBody, 'utf8');
            integrity = integrity.digest().toHex();
            integrity = forge.util.encode64(integrity)

            finalData.integrity = rsaEncryptString(integrity)

            var key = forge.random.getBytesSync(32);
 
            var iv = forge.random.getBytesSync(16);

            var cipher = forge.cipher.createCipher('AES-CFB', key);
            cipher.start({iv: iv});
            cipher.update(forge.util.createBuffer(reqBody));
            cipher.finish();
            var output = cipher.output.getBytes();

            finalData.request = forge.util.encode64(output);
            finalData.key = rsaEncryptString(key)
            finalData.iv = rsaEncryptString(iv)
        }

        return finalData;
    }

    function handleReceivingEncryptedData(responseBody) {
        if(responseBody){
            var prvKey = forge.pki.privateKeyFromPem(privateKey);
            var key = prvKey.decrypt(forge.util.decode64(responseBody.key), 'RSA-OAEP', {
                md: forge.md.sha256.create(),
            });

            var iv = prvKey.decrypt(forge.util.decode64(responseBody.iv), 'RSA-OAEP', {
                md: forge.md.sha256.create(),
            });

            var decryptedIntegrity = prvKey.decrypt(forge.util.decode64(responseBody.integrity), 'RSA-OAEP', {
                md: forge.md.sha256.create(),
            });

            decryptedIntegrity = forge.util.decode64(decryptedIntegrity)
            
            var unpreparedData = forge.util.decode64(responseBody.response)

            var decipher = forge.cipher.createDecipher('AES-CFB', key);
            decipher.start({iv: iv});
            decipher.update(forge.util.createBuffer(unpreparedData));
            
            var output = decipher.output.getBytes();

            var integrity = forge.md.sha256.create();
            integrity.update(output, 'utf8');
            integrity = integrity.digest().getBytes();

            if(decryptedIntegrity !== integrity){
                alert("Integrity check failed")
                return []
            }

            var pepek = []

            output = output.split("},")

            if(output.length == 1 && output[0] == ""){
                return []
            }

            output.forEach((element) => {   
                if(element[element.length - 1] !== '}'){
                    element = element += '}'
                }
                element = element.replace(/'/g, '"')
                element = element.replace(/None/g, 'null');
                element = JSON.parse(element)
                pepek.push(element)
            });

            console.log(pepek)
            return pepek
        }
        return []
    }

    return (
        <div className='row bg-light text-dark' style={{height: `${document.documentElement.scrollHeight}px`}}>
            <Container>
                <CustomNavbar
                    email={data.email}
                    token={data.token}
                    privateKey={privateKey != '' ? privateKey : ''}
                />
            
                <Row style={{marginTop: '20px', marginLeft: '30px'}}>
                    <Col className="justify-content-center">
                    <Button variant="primary" onClick={handlePrichod} style={{backgroundColor: 'rgb(33,37,41)', color: 'rgb(248, 249, 250)', borderColor: 'rgb(33,37,41)'}}>
                        Príchod
                    </Button>{' '}
                    <Button variant="primary" onClick={handleOdchod} style={{backgroundColor: 'rgb(33,37,41)', color: 'rgb(248, 249, 250)', borderColor: 'rgb(33,37,41)'}}>
                        Odchod
                    </Button>{' '}
                    <Button variant="primary" onClick={handleDovolenka} style={{backgroundColor: 'rgb(33,37,41)', color: 'rgb(248, 249, 250)', borderColor: 'rgb(33,37,41)'}}>
                        Dovolenka
                    </Button>{' '}
                    <Button variant="primary" onClick={handlePrintPDF} style={{backgroundColor: 'rgb(33,37,41)', color: 'rgb(248, 249, 250)', borderColor: 'rgb(33,37,41)'}}>
                        Mesačný výpis
                    </Button>
                    </Col> 
                        <Modal show={showModal} onHide={handleVacationDescriptionModalClose}>
                            <Modal.Header closeButton>
                                <Modal.Title>Dovolenka</Modal.Title>
                            </Modal.Header>
                            <Modal.Body>
                                <div style={{ width: '100%', display: 'flex', justifyContent: 'space-between', alignItems: 'center' }}>
                                    <div class="form-group" style={{marginBottom: '20px'}}>
                                        <label for="start_date">OD:</label>
                                        <input type="date" id='start_date' name='start_date' style={{padding: '10px', border: '1px solid #ccc', borderRadius: '4px', boxSizing: 'border-box', width: '100%'}}/>                                </div>
                                    <div class="form-group" style={{marginBottom: '20px'}}>
                                        <label for="end_date">DO:</label>
                                        <input type="date" id='end_date' name='end_date' style={{padding: '10px', border: '1px solid #ccc', borderRadius: '4px', boxSizing: 'border-box', width: '100%'}}/>
                                    </div>
                                </div>
                                <input
                                    type="text"
                                    placeholder="Popis dovolenky"
                                    value={vacationDescription}
                                    onChange={(e) => setVacationDescription(e.target.value)}
                                    maxLength={200}
                                    style={{padding: '10px', border: '1px solid #ccc', borderRadius: '4px', boxSizing: 'border-box', width: '100%'}}
                                />

                            </Modal.Body>
                            <Modal.Footer>
                                <Button variant="secondary" onClick={handleVacationDescriptionModalClose}>
                                    Zatvoriť
                                </Button>
                                <Button variant="primary" onClick={handleModalSubmit} style={{backgroundColor: 'rgb(33,37,41)', color: 'rgb(248, 249, 250)', borderColor: 'rgb(33,37,41)'}}>
                                    Odoslať
                                </Button>
                            </Modal.Footer>
                        </Modal>
                </Row>
                
                <Row style={{padding: '30px'}}>
                    <div >
                        <Calendar
                            selectable
                            culture='sk'
                            localizer={localizer}
                            events={allEvents}
                            startAccessor="start"
                            endAccessor="end"
                            tooltipAccessor="description"
                            onNavigate={(e) => {
                                setSelectedDay(e)
                                setgetAllData(true)
                            }}
                            onSelectEvent={(e) => {
                                handleDeleteEvent(e.start.toString())
                                setgetAllData(true)
                            }}
                            style={{ height: 500 }}
                        />
                    </div>
                </Row>

                <Row style={{marginTop: '5px', marginLeft: '30px'}}>
                    <Col className="justify-content-center">
                        <input
                            id="vacation_search_bar"
                            type="text"
                            placeholder="Vyhľadaj popis dovolenky"
                            value={searchVacationDescription}
                            onChange={(e) => setSearchVacationDescription(e.target.value)}
                            maxLength={200}
                            style={{ padding: '10px', width: '270px', border: '1px solid #ccc', borderRadius: '4px', boxSizing: 'border-box' }}
                        />

                        <Button variant="primary" style={{ padding: '10px 12px', marginLeft: '5px', marginBottom: '5px', backgroundColor: 'rgb(33,37,41)', color: 'rgb(248, 249, 250)', borderColor: 'rgb(33,37,41)' }} onClick={handleSearchSubmit}>
                            Hľadať
                        </Button>
                    </Col>
                </Row>

                <Row id="timeRecord" style={{marginTop: '5px', marginLeft: '30px', marginRight: '30px'}}>
                    <Table striped bordered hover>
                        <thead>
                        <tr>
                            <th>Názov</th>
                            <th>Začiatok</th>
                            <th>Koniec</th>
                            <th>Popis</th>
                        </tr>
                        </thead>
                        <tbody>
                        {allEventsRecord.length > 0 ? allEventsRecord.map((event, index) => (
                            <tr key={index}>
                                <td>{event.title}</td>
                                <td>{event.start.toLocaleString('sk')}</td>
                                <td>{event.end.toLocaleString('sk')}</td>
                                <td>{event.description}</td>
                            </tr>
                        )) : <tr><td>Žiadne záznamy</td></tr>}
                        </tbody>
                    </Table>
                </Row> 
            </Container>
        </div>
    )
}

export default Logged