<?php
	include 'config.php';

	$tables_to_drop = [
		'photos', 'catalog', 'payments', 'reservations', 'reviews', 'rooms',
		'test_payments', 'test_reservations', 'test_reviews', 'test_users', 'users'
	];

	// Start transaction
	mysqli_begin_transaction($link);

	try {
		foreach ($tables_to_drop as $table) {
			// Prepare the DROP TABLE statement to execute
			$drop_sql = "DROP TABLE IF EXISTS $table"; // Backticks around table name to avoid issues with reserved words

			if (!mysqli_query($link, $drop_sql)) {
				// If there is an error with the table dropping, throw an exception
				throw new Exception("Error dropping table $table: " . mysqli_error($link));
			}
		}

		// Commit the transaction if all DROP statements are executed successfully
		mysqli_commit($link);
		echo "All specified tables have been dropped successfully.";
		echo "</br>";

	} catch (Exception $e) {
		// Rollback the transaction in case of an error
		mysqli_rollback($link);
		echo "Error: " . $e->getMessage();
	}

	// Users table
	$users_table = "CREATE TABLE IF NOT EXISTS users (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		username VARCHAR(30) NOT NULL UNIQUE,
		password VARCHAR(512) NOT NULL,
		salt VARCHAR(128) NOT NULL,
		email VARCHAR(50) NOT NULL UNIQUE,
		role ENUM('admin', 'user') DEFAULT 'user',
		public_key VARCHAR(1000),
		
		created_at TIMESTAMP
	)";

	// Testing users table
	$test_users_table = "CREATE TABLE IF NOT EXISTS test_users (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		username VARCHAR(30) NOT NULL,
		password VARCHAR(512) NOT NULL,
		salt VARCHAR(128) NOT NULL,
		email VARCHAR(50) NOT NULL UNIQUE,
		public_key VARCHAR(1000),
		created_at TIMESTAMP
	)";

	// Reviews table
	$reviews_table = "CREATE TABLE IF NOT EXISTS reviews (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		user_id INT(6) UNSIGNED NOT NULL,
		reservation_id INT(6) UNSIGNED NOT NULL,
		review VARCHAR(255) NOT NULL,
		stars INT(1) NOT NULL,
		created_at TIMESTAMP
	)";

	// Testing reviews table
	$test_reviews_table = "CREATE TABLE IF NOT EXISTS test_reviews (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		user_id INT(6) UNSIGNED NOT NULL,
		reservation_id INT(6) UNSIGNED NOT NULL,
		review VARCHAR(255) NOT NULL,
		stars INT(1) NOT NULL,
		created_at TIMESTAMP
	)";

	// Payments table
	$payments_table = "CREATE TABLE IF NOT EXISTS payments (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		reservation_id INT(6) UNSIGNED NOT NULL,
		amount INT(6) NOT NULL,
		paid BOOLEAN DEFAULT FALSE,
		created_at TIMESTAMP
	)";

	// Testing payments table
	$test_payments_table = "CREATE TABLE IF NOT EXISTS test_payments (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		user_id INT(6) UNSIGNED NOT NULL,
		reservation_id INT(6) UNSIGNED NOT NULL,
		amount INT(6) NOT NULL,
		paid BOOLEAN DEFAULT FALSE,
		created_at TIMESTAMP
	)";

	// Catalog of accommodation table
	$catalog_table = "CREATE TABLE IF NOT EXISTS catalog (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		title VARCHAR(255) NOT NULL,
		description TEXT,
    	main_photo_url VARCHAR(255),
		created_at TIMESTAMP
	)";

	// Table for additional photos
	$photos_table = "CREATE TABLE IF NOT EXISTS photos (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		room_id INT(6) UNSIGNED NOT NULL,
		photo_url VARCHAR(255),  -- URL for additional photos
		created_at TIMESTAMP,
		FOREIGN KEY (room_id) REFERENCES rooms(id)
	)";

	// Rooms table
	$rooms_table = "CREATE TABLE IF NOT EXISTS rooms (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		title VARCHAR(255) NOT NULL,
		room_number INT(6) NOT NULL UNIQUE,
		price INT(6) NOT NULL,
		bed_count INT(6) NOT NULL,  -- Add the bed_count column
		main_photo_url VARCHAR(255),
		description TEXT,
		created_at TIMESTAMP
	)";

	// Reservations table
	$reservations_table = "CREATE TABLE IF NOT EXISTS reservations (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		user_id INT(6) UNSIGNED NOT NULL,
		room_id INT(6) UNSIGNED NOT NULL,
		status ENUM('PENDING', 'CONFIRMED', 'DONE', 'CANCELLED') DEFAULT 'pending',
		arrival_date DATE,
		departure_date DATE,
		created_at TIMESTAMP
	)";

	// Testing reservations table
	$test_reservations_table = "CREATE TABLE IF NOT EXISTS test_reservations (
		id INT(6) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
		user_id INT(6) UNSIGNED NOT NULL,
		payment_id INT(6) UNSIGNED NOT NULL,
		room_id INT(6) UNSIGNED NOT NULL,
		arrival_date DATE,
		departure_date DATE,
		created_at TIMESTAMP
	)";

	$table_execute = array(
		$users_table,
		$test_users_table,
		$reviews_table,
		$test_reviews_table,
		$payments_table,
		$test_payments_table,
		$catalog_table,
		$rooms_table,
		$reservations_table,
		$test_reservations_table,
		$photos_table
	);

	// Execute all tables if they don't exist in the database
	foreach ($table_execute as $table) {
		if ($link->query($table) === TRUE) {
			echo "Table created successfully";
			echo "</br>";
		} else {
			echo "Error creating table: " . $link->error;
		}
	}
?>
