<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Include your database config
require_once('db/config.php');

session_start();

// Define constants for delay settings
define('MAX_LOGIN_ATTEMPTS', 5); // Maximum allowed login attempts
define('DELAY_SECONDS', 60); // Delay in seconds

if ((isset($_SESSION["loggedin"]) && $_SESSION["loggedin"] === true)) {
  header("location: index.php");
  exit;
}

$loginErr = $passErr = $otherErr = "";


// $_SESSION['login_attempts']=0;

if ($_SERVER["REQUEST_METHOD"] == "POST") {
  $login = $_POST['username'];
  $password = $_POST['password'];

  // Check if the user has reached the maximum login attempts
  if (isset($_SESSION['login_attempts']) && $_SESSION['login_attempts'] >= MAX_LOGIN_ATTEMPTS) {
    // Check if the delay session variable is set
    if (!isset($_SESSION['delay_start'])) {
      // Set the delay session variable and store the current timestamp
      $_SESSION['delay_start'] = time();
    } else {
      // Calculate the remaining time of the delay
      $delay_elapsed = time() - $_SESSION['delay_start'];
      $remaining_delay = DELAY_SECONDS - $delay_elapsed;

      // If the remaining delay is over, reset the login attempts and delay
      if ($remaining_delay <= 0) {
        unset($_SESSION['login_attempts']);
        unset($_SESSION['delay_start']);
      } else {
        // Display the remaining delay to the user
        $otherErr = "Prilis mnoho nespravnych pokusov. Skuste znovu za $remaining_delay sekúnd.";
      }
    }
  } else {

    if (empty($loginErr) && empty($passErr)) {
      $sql = "SELECT id, email, username, password, salt, role, public_key, created_at FROM users WHERE (username = ? OR email = ?)";
      $stmt = mysqli_prepare($link, $sql);

      if ($stmt) {
        mysqli_stmt_bind_param($stmt, "ss", $login, $login);
        if (mysqli_stmt_execute($stmt)) {
          mysqli_stmt_store_result($stmt);

          if (mysqli_stmt_num_rows($stmt) == 1) {
            mysqli_stmt_bind_result($stmt, $id, $email, $dbLogin, $hashed_password, $salt, $role, $public_key, $created_at);
            mysqli_stmt_fetch($stmt);

            // Combine the entered password with the retrieved salt and hash it
            $saltedPassword = $salt . $password;

            if (password_verify($saltedPassword, $hashed_password)) {
              // Reset login attempts
              unset($_SESSION['login_attempts']);

			  $_SESSION["id"] = $id;
              $_SESSION["pubKey"] = $public_key;
              $_SESSION["loggedin"] = true;
              $_SESSION["name"] = $dbLogin;
              $_SESSION["email"] = $email;
			  $_SESSION["role"] = $role;
              $_SESSION["created_at"] = $created_at;
              header("location: index.php");
              exit();
            } else {
              recordLoginAttempt();
              $otherErr = "Nespravne meno alebo heslo";
            }
          } else {
            recordLoginAttempt();
            $otherErr = "Nespravne meno alebo heslo";
          }
        } else {
          recordLoginAttempt();
          $otherErr = "Ups. Nieco sa pokazilo!";
        }

        mysqli_stmt_close($stmt);
      } else {
        $otherErr = "Ups. Nieco sa pokazilo!";
      }
    }
  }
}

mysqli_close($link);


// Function to record the login attempt count
function recordLoginAttempt() {
  if (!isset($_SESSION['login_attempts'])) {
    $_SESSION['login_attempts'] = 1;
  } else {
    $_SESSION['login_attempts']++;
  }
}

?>



<!DOCTYPE html>
<html lang="en">

<head>
  <meta charset="UTF-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Document</title>

  <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-GLhlTQ8iRABdZLl6O3oVMWSktQOp6b7In1Zl3/Jr59b6EGGoI1aFkw7cmDA6j6gD" crossorigin="anonymous">
  <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/js/bootstrap.bundle.min.js" integrity="sha384-w76AqPfDkMBDXo30jS1Sgez6pr3x5MlQ1ZAGC+nuZB+EYdgRZgiwxhTBTkF7CXvN" crossorigin="anonymous"></script>
  <script src='https://kit.fontawesome.com/3b6394f049.js' crossorigin='anonymous'></script>


  <link rel="stylesheet" href="style.css">
</head>



<body class="d-flex flex-column min-vh-100">
  <nav class="navbar navbar-expand-lg navbar-dark bg-dark mb-2">
    <div class="container">
      <a class="navbar-brand" href="#">Zadanie 1</a>
      <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarNav" aria-controls="navbarNav" aria-expanded="false" aria-label="Toggle navigation">
        <span class="navbar-toggler-icon"></span>
      </button>

      <div class="collapse navbar-collapse justify-content-end" id="navbarNav">
        <ul class="navbar-nav">
          <li class="nav-item">
            <a class="active nav-link fw-bold" href="#">Login</a>
          </li>
        </ul>
      </div>

    </div>
  </nav>

  <div class="container-md ">
    <!-- <h1>Zadanie 1</h1>  -->
    <form action="#" method="post">
      <section class=" mt-3 gradient-custom">
        <div class="container py-0 h-100">
          <div class="row d-flex justify-content-center align-items-center h-100">
            <div class="col-12 col-md-8 col-lg-6 col-xl-5">
              <div class="card bg-dark text-white" style="border-radius: 1rem;">
                <div class="card-body p-4 text-center">

                  <div class="mb-md-5 mt-md-4 pb-0">

                    <h2 class="fw-bold mb-2 text-uppercase">Login</h2>
                    <p class="text-white-50 mb-5">Zadajte svoje prihlasovacie meno a heslo!</p>

                    <div class="form-outline form-white mb-4">
                      <input type="text" id="username" name="username" class="form-control form-control-lg" />
                      <label class="form-label" for="username">Prihlasovacie meno:</label>

                      <div id="usernameFeedback" class="invalid-feedback <?php echo $loginClass; ?>">
                        <?php echo $loginErr; ?>
                      </div>

                    </div>

                    <div class="form-outline form-white mb-4">
                      <input type="password" id="password" name="password" class="form-control form-control-lg" />
                      <label class="form-label" for="password">Heslo:</label>

                      <div id="passFeedback" class="invalid-feedback <?php echo $passClass; ?>" style="display: block">
                        <?php echo $otherErr; ?>
                      </div>
                    </div>


                    <button class="btn btn-outline-light btn-lg px-5" type="submit">Login</button>


                  </div>

                  <div>
                    <p class="mb-0">Nemáte účet? <a href="signUP.php" class="text-white-50 fw-bold">Zaregistrujte sa!</a>
                    </p>
                  </div>

                </div>
              </div>
            </div>
          </div>
        </div>
      </section>
    </form>


  </div>



</body>

</html>
