<?php
	header('Content-Type: application/json');

	if (!isset($_SESSION)) {
		session_start();
	}

	class BankAccount {
		public $cardNumber;
		public $csv;
		public $totalBalance;

		public function __construct($cardNumber, $csv, $totalBalance) {
			$this->cardNumber = $cardNumber;
			$this->csv = $csv;
			$this->totalBalance = $totalBalance;
		}

		public function canPay($amount) {
			return $amount <= $this->totalBalance;
		}

		public function realisePayment($amount) {
			if ($amount < 0 || $amount > $this->totalBalance) {
				return false;
			}

			$this->totalBalance -= $amount;

			return true;
		}
	}

	$bankAccounts = array(
		new BankAccount("123", 111, 500),
		new BankAccount("777", 111, 70),
		new BankAccount("437", 111, 100000000),
	);

	$response = array(
		'success' => false,
		'message' => 'Invalid data',
		'hash' => NULL
	);

	if ($_SERVER["REQUEST_METHOD"] == "POST") {
		if (!isset($_POST)) {
			$response['message'] = "Invalid data";
			echo json_encode($response);
			return;
		}

		$data = $_POST;
		$cardNumber = $data['cardNumber'];
		$csv = $data['csv'];
		$amount = $data['amount'];

		// Find bankAccount by cardNumber
		$bankAccount = null;
		foreach ($bankAccounts as $account) {
			if ($account->cardNumber == $cardNumber && $account->csv == $csv) {
				$bankAccount = $account;
				break;
			}
		}

		if ($bankAccount == null) {
            $response['message'] = "Invalid card number";
        } else if (!$bankAccount->canPay($amount)) {
            $response['message'] = "Insufficient balance";
        } else {
            $success = $bankAccount->realisePayment($amount);

            if ($success) {
                $hash = hash('sha256', random_bytes(7));
                $response = array(
					'success' => true,
					'message' => "Payment successful.",
					'hash' => $hash
				);
            } else {
                $response['message'] = "Payment failed";
            }
        }

		echo json_encode($response);
	}
?>
