<?php
	include_once("../db/config.php");
	require('../fpdf/fpdf.php');
	require('../encryptFunctions.php');

	if (!isset($_SESSION)) {
		session_start();
	}

	$invalid = false;

	// Initial request to get amount to be paid
	if ($_SERVER['REQUEST_METHOD'] == "GET" && isset($_GET["amount"]) && isset($_GET['reservation_id'])) {
		$escapedReservationId = mysqli_real_escape_string($link, $_GET['reservation_id']);
		$escapedAmount = mysqli_real_escape_string($link, $_GET['amount']);

		// Check if reservation exists
		$sql = "SELECT * FROM reservations WHERE id = '$escapedReservationId'";
		$result = mysqli_query($link, $sql);
		if (mysqli_num_rows($result) == 0) {
			$invalid = true;
			echo "Invalid reservation id";
			return;
		}

		// Check if reservation is already paid
		$sql = "SELECT * FROM payments WHERE reservation_id = '$escapedReservationId'";
		$result = mysqli_query($link, $sql);
		if (mysqli_num_rows($result) == 0) {
			$invalid = true;
			echo "Payment for following reservation doesn't exists";
			return;
		}

		$row = mysqli_fetch_assoc($result);
		$_SESSION['paymentId'] = $row['id'];

		// Check if amount is valid
		if ($escapedAmount <= 0) {
			$invalid = true;
			echo "Invalid amount";
			return;
		}

		$_SESSION['amount'] = $escapedAmount;
		$_SESSION['init'] = true;
		$invalid = false;
	} else if ($_SERVER["REQUEST_METHOD"] == "POST" &&
		isset($_SESSION['init']) &&
		$_SESSION['init'] == true &&
		isset($_SESSION['amount']) &&
		$_SESSION['amount'] > 0 &&
		isset($_POST['cardNumber']) &&
		isset($_POST['csv'])) {

		$amount = $_SESSION['amount'];
		$cardNumber = $_POST['cardNumber'];
		$csv = $_POST['csv'];

		$amount = mysqli_real_escape_string($link, $amount);
		$cardNumber = mysqli_real_escape_string($link, $cardNumber);
		$csv = mysqli_real_escape_string($link, $csv);

		// Check card number and csv regex
		$csvRegex = "/^[0-9]{3}$/";

		// We don't check cardNumber format as there exits many card number formats

		if (!preg_match($csvRegex, $csv)) {
			$invalid = true;
			echo "Invalid csv";
			return;
		}

		$data = array(
			'cardNumber' => $cardNumber,
			'csv' => $csv,
			'amount' => $amount
		);

		// Get current full url
		$bankURL = "http://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
		$bankURL = str_replace("index.php", "bank.php", $bankURL);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $bankURL);
		curl_setopt($ch, CURLOPT_PROXY, $_SERVER['SERVER_ADDR'] . ':' .  $_SERVER['SERVER_PORT']);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/x-www-form-urlencoded'));
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		$response = curl_exec($ch);

		if (curl_errno($ch)) {
			echo 'cURL error: ' . curl_error($ch);
			return;
		}

		curl_close($ch);

		$json_res = json_decode($response);

		if ($json_res->success == false) {
			echo ''. $json_res->message .'';
			return;
		}

		if ($json_res->hash != NULL) {
			$paymentId = $_SESSION['paymentId'];
			$paymentId = mysqli_escape_string($link, $paymentId);

			// Update reservation status
			$sql = "UPDATE payments SET paid = 1 WHERE id = '$paymentId'";
			mysqli_query($link, $sql);

			$sql = "SELECT r.title as room_name, r.room_number as room_no, rs.arrival_date as arrivalDate, rs.departure_date as departureDate, u.username as userName
					FROM payments p
					INNER JOIN reservations rs ON rs.id = p.reservation_id
					INNER JOIN rooms r ON r.id = rs.room_id
					INNER JOIN users u ON u.id = rs.user_id
					WHERE p.id = $paymentId;";

			$result = mysqli_query($link, $sql);

			if ($result) {
				$row = mysqli_fetch_assoc($result);

				if ($row) {
					$room_name = $row['room_name'];
					$room_no = $row['room_no'];
					$arrivalDate = $row['arrivalDate'];
					$departureDate = $row['departureDate'];
					$userName = $row['userName'];
					
					$reservation_data = array(
						'room_name' => $room_name,
						'room_no' => $room_no,
						'arrivalDate' => $arrivalDate,
						'departureDate' => $departureDate,
						'userName' => $userName
					);

    				$symmetricKey = openssl_random_pseudo_bytes(32);

					$room_name = encryptData($symmetricKey, $_SESSION['pubKey'], $room_name);
					$room_no = encryptData($symmetricKey, $_SESSION['pubKey'], $room_no);
					$arrivalDate = encryptData($symmetricKey, $_SESSION['pubKey'], $arrivalDate);
					$departureDate = encryptData($symmetricKey, $_SESSION['pubKey'], $departureDate);
					$userName = encryptData($symmetricKey, $_SESSION['pubKey'], $userName);

					class PDF extends FPDF {
						function createReservationConfirmation($symmetricKey, $room_no, $room_name, $arrivalDate, $departureDate, $userName) {
							$this->AddPage();
							
							$this->SetFont('Arial', '', 12);
							
							$this->Cell(0, 10, 'POTVRDENIE REZERVACIE', 0, 1, 'C');
							$this->Ln(10);
							
							$this->Cell(0, 10, 'Hotel Room: ' . $room_name . ' (Room No: ' . $room_no . ')', 0, 1);
							$this->Cell(0, 10, 'Arrival Date: ' . $arrivalDate, 0, 1);
							$this->Cell(0, 10, 'Departure Date: ' . $departureDate, 0, 1);
							$this->Cell(0, 10, 'User name: ' . $userName, 0, 1);

							// $this->Ln(10);
							// $this->Cell(0, 10, 'Symmetric key: ' . $symmetricKey, 0, 1);
						}
					}
		
					$pdf = new PDF();
		
					$pdf->createReservationConfirmation($symmetricKey, $room_no, $room_name, $arrivalDate, $departureDate, $userName);
					$pdf->Output('reservation.pdf', 'D'); // 'D' znamená, že PDF súbor bude stiahnutý priamo do prehliadača


					// echo "<script>alert('Payment Successful!'); window.location.href='../user/reservations.php';</script>";

					// echo "<span>Payment Successful!</span>";
					// echo "<a href='../user/reservations.php' class='btn btn-primary'>Návrat</a>";

		
					// header('Location: ../user/reservations.php');
					// exit();	
				}
			}

		}	
	} else {
		$invalid = true;
		echo "Invalid Request";
	}
?>

<!-- Terminate rendering if some error occur -->
<?php if ($invalid) return; ?>

<!DOCTYPE html>
<html lang="en">

<head>
  <meta charset="UTF-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Document</title>

  <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-GLhlTQ8iRABdZLl6O3oVMWSktQOp6b7In1Zl3/Jr59b6EGGoI1aFkw7cmDA6j6gD" crossorigin="anonymous">
  <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/js/bootstrap.bundle.min.js" integrity="sha384-w76AqPfDkMBDXo30jS1Sgez6pr3x5MlQ1ZAGC+nuZB+EYdgRZgiwxhTBTkF7CXvN" crossorigin="anonymous"></script>
  <script src='https://kit.fontawesome.com/3b6394f049.js' crossorigin='anonymous'></script>


  <link rel="stylesheet" href="style.css">
</head>



<body class="d-flex flex-column min-vh-100">
	<div class="container-sm mt-5" style="max-width: 576px; ">
		<form action='index.php' method='POST' class='p-4 border bg-light' style='border-radius: 8px;'>
			<div class='mb-3'>
				<label for='cardNumber' class='form-label'>Card Number:</label>
				<input type='text' class='form-control' id='cardNumber' name='cardNumber' required>
			</div>

			<div class='mb-3'>
				<label for='csv' class='form-label'>CSV:</label>
				<input type='text' class='form-control' id='csv' name='csv' required>
			</div>

			<button type='submit' class='btn btn-primary'>Submit Payment</button>
		</form>
	</div>
</body>
</html>
