<?php
	require_once('../db/config.php');
	require_once('../user/auth.php');
	session_start();
	ob_start();

	if (isAuth() == 0) {
		header('Location: ../login.php');
		exit;
	}

	$userId = getUserId();
	$userEmail = $_SESSION['email'];

	$today = date("Y-m-d");
	$rooms = array();

	$arrivalDate;

	if (isset($_GET['arrivalDate']) && isset($_GET['departureDate'])) {
		$arrivalDate = $_GET['arrivalDate'];
		$departureDate = $_GET['departureDate'];

		$arrivalDate = mysqli_real_escape_string($link, $arrivalDate);
		$departureDate = mysqli_real_escape_string($link, $departureDate);

		// Check if dates are valid
		if ($arrivalDate < $today || $departureDate < $today || $arrivalDate > $departureDate) {
			echo "Invalid dates";
			return;
		}

		// Fetch rooms that are not reserved in given date range
		$rooms = array();
		$sql = "SELECT * FROM rooms
				WHERE id NOT IN
				(SELECT room_id FROM reservations WHERE (arrival_date <= '$arrivalDate' AND departure_date >= '$arrivalDate') OR (arrival_date <= '$departureDate' AND departure_date >= '$departureDate'))";

		$result = mysqli_query($link, $sql);
		if (mysqli_num_rows($result) > 0) {
			while ($row = mysqli_fetch_assoc($result)) {
				array_push($rooms, $row);
			}
		}
	}

	if (isset($_GET['arrivalDate']) && isset($_GET['departureDate']) && isset($_GET['roomSelection'])) {
		$roomSelection = $_GET['roomSelection'];
		$roomSelection = mysqli_real_escape_string($link, $roomSelection);

		// Check if room is available within given date range
		$sql = "SELECT * FROM rooms
				WHERE id = '$roomSelection' AND id NOT IN
				(SELECT room_id FROM reservations WHERE (arrival_date <= '$arrivalDate' AND departure_date >= '$arrivalDate') OR (arrival_date <= '$departureDate' AND departure_date >= '$departureDate'))";

		$result = mysqli_query($link, $sql);

		if (mysqli_num_rows($result) == 0) {
			echo "Room is not available for given date range";
			return;
		}


		// Get total price for reservation based on room price and number of days
		$sql = "SELECT * FROM rooms WHERE id = '$roomSelection'";
		$result = mysqli_query($link, $sql);
		$row = mysqli_fetch_assoc($result);
		$price = $row['price'];
		$days = (strtotime($departureDate) - strtotime($arrivalDate)) / (60 * 60 * 24);
		$totalPrice = $price * $days;

		// Create reservation
		$sql = "INSERT INTO reservations (user_id, room_id, arrival_date, departure_date) VALUES ('$userId', '$roomSelection', '$arrivalDate', '$departureDate')";
		$result = mysqli_query($link, $sql);
		if ($result) {
			echo "Reservation created";


			// Get the ID of the last inserted reservation
			$reservationId = mysqli_insert_id($link);

			// Create payment
			$sql = "INSERT INTO payments (reservation_id, amount, paid) VALUES ($reservationId, '$totalPrice', 0)";
			$result = mysqli_query($link, $sql);

			if (!$result || mysqli_affected_rows($link) == 0) {
				echo "Error creating payment";
			} else {
				// Redirect to payment gateway
				header('Location: ../payment-gateway/index.php?reservation_id=' . $reservationId . '&amount=' . $totalPrice);
				exit();
			}
		} else {
			echo "Error creating reservation";
		}
	}
	ob_end_flush();


	// Function to truncate text and add three dots
function truncateText($text, $maxLength) {
    if (strlen($text) > $maxLength) {
        $truncatedText = substr($text, 0, $maxLength) . '...';
    } else {
        $truncatedText = $text;
    }

    return $truncatedText;
}

?>


<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Vytvoriť rezerváciu</title>

    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/css/bootstrap.min.css"
        rel="stylesheet" integrity="sha384-GLhlTQ8iRABdZLl6O3oVMWSktQOp6b7In1Zl3/Jr59b6EGGoI1aFkw7cmDA6j6gD"
        crossorigin="anonymous">
	<script src="https://unpkg.com/@popperjs/core@2"></script>
	<script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.7.1/jquery.min.js"></script>
	<script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/js/bootstrap.bundle.min.js"
        integrity="sha384-w76AqPfDkMBDXo30jS1Sgez6pr3x5MlQ1ZAGC+nuZB+EYdgRZgiwxhTBTkF7CXvN"
        crossorigin="anonymous"></script>
    <script src='https://kit.fontawesome.com/3b6394f049.js' crossorigin='anonymous'></script>

	<style>
		.cropped-img {
            object-fit: cover;
            width: 100%;
            height: 200px; /* Set your desired height */
            transition: transform 0.3s ease-in-out;

        }

        .card:hover .cropped-img {
            transform: scale(1.05);
        }

        .card {
            height: auto; /* Set a fixed height for all cards */
        }
        .card-body, .card-footer{
            padding: 0;
        }
        .btn{
            width: 100%;
        }
        .card-info{
            padding: 1%;
        }
	</style>
</head>
<body class="d-flex flex-column min-vh-100">
<nav class="navbar navbar-expand-lg navbar-dark bg-dark">
        <div class="container">
            <a class="navbar-brand" href="#"><?php
						// Check if the user is logged in
						if (isset($userEmail)) {
							echo 'Welcome, ' . $userEmail;
						} 
						?></a>
            <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarNav"
                aria-controls="navbarNav" aria-expanded="false" aria-label="Toggle navigation">
                <span class="navbar-toggler-icon"></span>
            </button>

            <div class="collapse navbar-collapse justify-content-end" id="navbarNav">
                <ul class="navbar-nav">
                    <li class="nav-item">
                        <a class="nav-link" href="../index.php">Home</a>
                    </li>
                    <li class="nav-item">
                        <a class="nav-link" href="../user/reservations.php">My reservations</a>
                    </li>
					<li class="nav-item">
						<?php
						// Check if the user is logged in
						if (!isset($userEmail)) {
							echo '<a class="nav-link" href="../login.php">Login</a>';
						}
						?>
					</li>
                    <li class="nav-item">
						<?php
						if (!isset($_SESSION['pubKey'])) {
							echo '<a class="nav-link" href="../userCreateKey.php">Add Keys</a>';
						}
						?>
					</li>
					<?php
						if (isset($_SESSION["role"]) && $_SESSION["role"]=="admin") {
							echo '<li class="nav-item">
							<a class="nav-link" href="../dashboard.php">Dashboard</a>
							</li>
							<li class="nav-item">
							<a class="nav-link" href="../createRoom.php">Create Room</a>
							</li>';
						}
						?>
					<li class="nav-item">
						<?php
						if (isset($userEmail)) {
							echo '<a class="nav-link" href="../logout.php">Log Out</a>';
						}
						?>
					</li>
                </ul>
            </div>
        </div>
    </nav>

	<div class="container mt-5">
		<!-- Generate bootstrap form to create new reservation - it should contains room selection, arrival date and departure date  -->
		<form action="create_reservation.php" method="GET" class='container'>

			<div class='row mb-2'>
				<div class="form-group col-6">
					<label for="arrivalDate">Arrival Date:</label>
					<input type="date" class="form-control" id="arrivalDate" name="arrivalDate" required value="<?php echo $arrivalDate; ?>">
				</div>

				<!-- Departure Date -->
				<div class="form-group col-6">
					<label for="departureDate">Departure Date:</label>
					<input type="date" class="form-control" id="departureDate" name="departureDate" required value="<?php echo $departureDate; ?>">
				</div>
			</div>

			<button type="submit" class="btn btn-primary">
				Check Availability
			</button>
			<!-- <button type="submit" class="btn btn-primary">
				<?php
					if (!isset($arrivalDate) || !isset($departureDate)) {
						echo "Check Availability";
					} else {
						echo "Create Reservation";
					}
				?>
			</button> -->
		</form>

		<div class="row">
			<?php foreach ($rooms as $room) : ?>
				<div class="col-md-4 mb-4">
					<div class="card">

						<div class="overflow-hidden">
							<?php
							$imagePath = "../uploads/photos/{$room['main_photo_url']}";
							$defaultImagePath = "../uploads/photos/unknown.png"; // Set the correct path to your default image

							// Check if the image file exists
							if (file_exists($imagePath) && $room['main_photo_url'] != '') {
								echo "<img src='{$imagePath}' class='card-img-top cropped-img' alt='Room Photo'>";
							} else {
								echo "<img src='{$defaultImagePath}' class='card-img-top cropped-img' alt='Default Photo'>";
							}
							?>
						</div>

						<div class="card-body">
							<div class="card-info">
								<h5 class="card-title"><?php echo $room['title']; ?></h5>
								<?php
								echo empty($room['description']) ? "Description will be added soon" : truncateText($room['description'], 45);
								?>
							</div>
						</div>

						<ul class="list-group list-group-flush">
							<li class="list-group-item">Room Number: <?php echo $room['room_number']; ?></li>
							<li class="list-group-item">Price: <?php echo $room['price']; ?> EUR</li>
							<li class="list-group-item">Bed Count: <?php echo $room['bed_count']; ?></li>
						</ul>

						<div class="card-body">
							<a href="room_details.php?roomSelection=<?php echo $room['id']; ?>&arrivalDate=<?php echo $arrivalDate; ?>&departureDate=<?php echo $departureDate; ?>" class="btn btn-primary">View Details</a>
						</div>
						<div class="card-footer">
							<?php
								if (isAdmin() == true) {
									echo "<a href='../editRoom.php?roomSelection=" . $room['id'] . "' class='btn btn-secondary'>Edit Room</a>";
								} else {
									echo "<a href='create_reservation.php?roomSelection=" . $room['id'] . "&arrivalDate=$arrivalDate&departureDate=$departureDate' class='btn btn-secondary'>Create reservation</a>";
								}
							?>
						</div>
					</div>
				</div>
			<?php endforeach; ?>
		</div>
	</div>

	<div class="row">
            <?php foreach ($rooms as $room) : ?>
                <div class="col-md-4 mb-4">
                    <div class="card">

                        <div class="overflow-hidden">
                            <?php
                            $imagePath = "../uploads/photos/{$room['main_photo_url']}";
                            $defaultImagePath = "../uploads/photos/unknown.png"; // Set the correct path to your default image

                            // Check if the image file exists
                            if (file_exists($imagePath) && $room['main_photo_url'] != '') {
                                echo "<img src='{$imagePath}' class='card-img-top cropped-img' alt='Room Photo'>";
                            } else {
                                echo "<img src='{$defaultImagePath}' class='card-img-top cropped-img' alt='Default Photo'>";
                            }
                            ?>
                        </div>

                        <div class="card-body">
                            <div class="card-info">
                                <h5 class="card-title"><?php echo $room['title']; ?></h5>
                                <?php
                                echo empty($room['description']) ? "Description will be added soon" : truncateText($room['description'], 45);
                                ?>
                            </div>
                        </div>

                        <ul class="list-group list-group-flush">
                            <li class="list-group-item">Room Number: <?php echo $room['room_number']; ?></li>
                            <li class="list-group-item">Price: <?php echo $room['price']; ?> EUR</li>
                            <li class="list-group-item">Bed Count: <?php echo $room['bed_count']; ?></li>
                        </ul>

                        <div class="card-body">
                            <a href="room_details.php?room_id=<?php echo $room['id']; ?>" class="btn btn-primary">View Details</a>
                        </div>
                              <!-- Edit button -->
                        <div class="card-footer">
                            <a href="editRoom.php?room_id=<?php echo $room['id']; ?>" class="btn btn-secondary">Edit Room</a>
                        </div>
                    </div>
                </div>
            <?php endforeach; ?>
        </div>


</body>

