<?php
session_start();

if ((isset($_SESSION["loggedin"]) && $_SESSION["loggedin"] === true)) {
  header("Location: index.php");
  exit;
}

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('db/config.php');

$username_value = "";
$email_value = "";
$emailErr = "";
$usernameErr = "";
$passwordErr = "";
$emailClass = "";

function userExist($link, $username, $email)
{
  $exist = false;
  $param_username = trim($username);
  $param_email = trim($email);
  $sql = "SELECT id FROM users WHERE username = ? OR email = ?";
  $stmt = mysqli_prepare($link, $sql);
  mysqli_stmt_bind_param($stmt, "ss", $param_username, $param_email);
  mysqli_stmt_execute($stmt);
  mysqli_stmt_store_result($stmt);
  if (mysqli_stmt_num_rows($stmt) == 1) {
    $exist = true;
  }
  mysqli_stmt_close($stmt);
  return $exist;
}

function validateEmail($email) {
    $email = trim($email);

    if (empty($email)) {
        return "Zadajte email";
    } elseif (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
        return "E-mail nie je platný";
    }

    return "";
}

function validateLogin($login) {
    $login = trim($login);

    if (empty($login)) {
        return "Zadajte login";
    } elseif (!preg_match('/^[a-zA-Z0-9_]+$/', $login)) {
        return "Login moze obsahovat iba velke, male pismena, cislice a podtrznik";
    } elseif (strlen($login) < 6 || strlen($login) > 32) {
        return "Login musí mať min. 6 a max. 32 znakov";
    }

    return "";
}

function validatePassword($password) {
    $password = trim($password);

    if (empty($password)) {
        return "Zadajte heslo";
    } elseif (isPasswordInDictionary($password)) {
        return "Zadané heslo nie je bezpečné";
    } elseif (strlen($password) < 8) {
        return "Heslo musí mať aspoň 8 znakov";
    } elseif (!preg_match('/^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*\W)/', $password)) {
        return "Heslo musí obsahovať aspoň jedno veľké písmeno, jedno malé písmeno, jednu číslicu a jeden špeciálny znak";
    }

    return "";
}

function isPasswordInDictionary($password) {
    // Path to the dictionary file
    $dictionaryFile = '10-million-password-list-top-1000000.txt';

    // Read the dictionary file line by line
    $file = fopen($dictionaryFile, 'r');
    if ($file) {
        while (($line = fgets($file)) !== false) {
            $line = trim($line); // Remove leading/trailing whitespace
            if (strcasecmp($password, $line) === 0) {
                fclose($file);
                return true; // Password is in the dictionary
            }
        }
        fclose($file);
    }

    return false; // Password is not in the dictionary
}

if ($_SERVER["REQUEST_METHOD"] == "POST") {
    $errmsg = '';


    if(validateEmail($_POST['email'])==""){
        $email_value = $_POST['email'];
    }else{
        $emailErr = validateEmail($_POST['email']);
        $errmsg = validateEmail($_POST['email']);
    }

    if(validateLogin($_POST['username'])==""){
        $username_value = $_POST['username'];
    }else{
        $usernameErr = validateLogin($_POST['username']);
        $errmsg = validateLogin($_POST['username']);
    }

    if(validatePassword($_POST['password'])==""){
        // $password_value = $_POST['password'];
    }else{
        $passwordErr = validatePassword($_POST['password']);
        $errmsg = validatePassword($_POST['password']);
    }
    
    

    if (userExist($link, $_POST['username'], $_POST['email'])) {
      $errmsg .= "Pouzivatel s tymto e-mailom / uzivatelskym menom uz existuje.</p>";
  
      $emailErr = "Pouzivatel s tymto e-mailom / uzivatelskym menom uz existuje";
      $emailClass = "showFeedback";
  
      $email_value = $_POST['email'];
      $username_value = $_POST['username'];
    }
  
    if (empty($errmsg)) {
        // Generate a random salt
        $salt = bin2hex(random_bytes(16)); // You can adjust the length of the salt

        // Combine the password with the salt
        $salted_password = $salt . $_POST['password'];

        // Hash the salted password
        $hashed_password = password_hash($salted_password, PASSWORD_ARGON2ID);

        $sql = "INSERT INTO users (username, email, password, salt) VALUES (?, ?, ?, ?)";
        $stmt = mysqli_prepare($link, $sql);

        $email = $_POST['email'];
        $username = $_POST['username'];

        mysqli_stmt_bind_param($stmt, "ssss", $username, $email, $hashed_password, $salt);

        if (mysqli_stmt_execute($stmt)) {
            // session_start(); // Start or resume the session
            $_SESSION["loggedin"] = true; // User is logged in
            $_SESSION["username"] = $username; // Store the username in the session
            $_SESSION["email"] = $email; // Store the username in the session
            $_SESSION["pubKey"] = null; 
            header("Location: index.php");
            exit();
        } else {
            echo "Ups. Nieco sa pokazilo";
        }
  
      mysqli_stmt_close($stmt);
    }
    mysqli_close($link);
  }
?>




<!DOCTYPE html>
<html lang="sk">

<head>
  <meta charset="UTF-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Document</title>

  <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-GLhlTQ8iRABdZLl6O3oVMWSktQOp6b7In1Zl3/Jr59b6EGGoI1aFkw7cmDA6j6gD" crossorigin="anonymous">
  <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0-alpha1/dist/js/bootstrap.bundle.min.js" integrity="sha384-w76AqPfDkMBDXo30jS1Sgez6pr3x5MlQ1ZAGC+nuZB+EYdgRZgiwxhTBTkF7CXvN" crossorigin="anonymous"></script>
  <script src='https://kit.fontawesome.com/3b6394f049.js' crossorigin='anonymous'></script>




  <link rel="stylesheet" href="style.css">
</head>



<body class="d-flex flex-column min-vh-100">
  <nav class="navbar navbar-expand-lg navbar-dark bg-dark mb-2">
    <div class="container">
      <a class="navbar-brand" href="#">Zadanie 1</a>
      <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarNav" aria-controls="navbarNav" aria-expanded="false" aria-label="Toggle navigation">
        <span class="navbar-toggler-icon"></span>
      </button>

      <div class="collapse navbar-collapse justify-content-end" id="navbarNav">
        <ul class="navbar-nav">
          <li class="nav-item ">
            <a class="nav-link " href="../index.php">Home</a>
          </li>
          <li class="nav-item ">
            <a class="nav-link" href="login.php">Login</a>
          </li>
        </ul>
      </div>

    </div>
  </nav>

  <div class="container-md ">
    <!-- <h1>Zadanie 1</h1>  -->


    <form name="myForm" action="<?php echo htmlspecialchars($_SERVER['PHP_SELF']); ?>" method="post" onsubmit="return validateForm() ">
      <section class=" mt-5 mb-5 gradient-custom">
        <div class="container py-0 h-100">
          <div class="row d-flex justify-content-center align-items-center h-100">
            <div class="col-12 col-md-8 col-lg-6 col-xl-5">
              <div class="card bg-dark text-white" style="border-radius: 1rem;">
                <div class="card-body p-5 text-center">

                  <div class="mb-md-5 mt-md-4 pb-0">

                    <h2 class="fw-bold mb-2 text-uppercase">Zaregistrujte sa!</h2>
                    <p class="text-white-50 mb-5">Prosím, zadajte svoje údaje!</p>


                    <div class="form-outline form-white mb-4">
                        <input type="text" id="username" name="username" value="<?php echo $username_value; ?>" class="form-control form-control-lg" />
                        <label class="form-label" for="username">Username</label>
                        <div id="usernameFeedback" class="invalid-feedback" style="display: block">
                            <?php echo $usernameErr; ?>
                        </div>
                    </div>


                    <div class="form-outline form-white mb-4">
                      <input type="email" id="email" name="email" value="<?php echo $email_value; ?>" class="form-control form-control-lg" />
                      <label class="form-label" for="email">Email</label>
                      <!-- <div id="emailFeedback" class="invalid-feedback"> -->

                      <div id="emailFeedback" class="invalid-feedback <?php echo $emailClass; ?>" style="display: block">
                        <?php echo $emailErr; ?>
                      </div>

                    </div>

                    <div class="form-outline form-white mb-4">
                      <input type="password" id="password" name="password" class="form-control form-control-lg" />
                      <label class="form-label" for="password">Heslo</label>
                      <div id="passwordFeedback" class="invalid-feedback" style="display: block">
                        <?php echo $passwordErr; ?>
                      </div>
                    </div>

                    <!-- <p class="small mb-4 pb-lg-2"><a class="text-white-50" href="#!">Forgot password?</a></p> -->

                    <button class="btn btn-outline-light btn-lg px-5" type="submit">Zaregistrovať sa</button>


                  </div>

                  <div>
                    <p class="mb-0">Už máte účet? <a href="login.php" class="text-white-50 fw-bold">Prihláste sa!</a>
                    </p>
                  </div>

                </div>
              </div>
            </div>
          </div>
        </div>
      </section>
    </form>

    


  </div>


  <script src='script.js'></script>
</body>

</html>