const db = require('better-sqlite3')('./collection.db');
db.pragma('journal_mode = WAL');

try {
  // Create tables
  const stmtCandidatesTable = db.prepare(`
        CREATE TABLE IF NOT EXISTS candidates (
        id INTEGER PRIMARY KEY,
        name TEXT NOT NULL,
        surname TEXT NOT NULL,
        votes INTEGER DEFAULT 0,
        UNIQUE (name, surname)
    )`);
  stmtCandidatesTable.run();
  console.log('Candidates table created successfully.');

  const stmtUsersTable = db.prepare(`
        CREATE TABLE IF NOT EXISTS users (
        id INTEGER PRIMARY KEY,
        name TEXT NOT NULL,
        surname TEXT NOT NULL,
        email TEXT NOT NULL UNIQUE,
        address TEXT,
        voted INTEGER DEFAULT 0,
        password TEXT NOT NULL,
        publicKey TEXT NOT NULL,
        loginAttempts INTEGER DEFAULT 3,
        lastLoginAttempt INTEGER DEFAULT 0
    )`);
  stmtUsersTable.run();

  console.log('Users table created successfully.');

  // Insert data
  const insertCandidates = db.prepare(`
        INSERT OR IGNORE INTO candidates (name, surname)
        VALUES (?, ?);
    `);

  const initDb = db.transaction(() => {
    insertCandidates.run('Zuzana', 'Čaputová');
    insertCandidates.run('Robert', 'Fico');
    insertCandidates.run('Ivan', 'Korčok');
    insertCandidates.run('Peter', 'Pellegrini');
    insertCandidates.run('Jožko', 'Ferko');
  });

  initDb();
  console.log('Initial data inserted successfully.');
} catch (error) {
  console.error('An error occurred:', error);
}
