'use client';
import { useGetLoginToken } from '@/utils/hooks/useGetToken';
import { useMutation, useQuery, useQueryClient } from '@tanstack/react-query';
import { UserUpdate, UserUpdateSchema } from '@/entities/UserUpdate';
import { useForm } from 'react-hook-form';
import styles from '../register/page.module.css';
import { zodResolver } from '@hookform/resolvers/zod';
import { useEffect } from 'react';
import { toast } from 'react-hot-toast';

async function getUserInfo(token: string | null) {
  const response = await fetch('http://localhost:3000/dashboard/api', {
    method: 'GET',
    headers: {
      'Content-Type': 'application/json',
      Authorization: `Bearer ${token}`,
    },
  });

  return response.json();
}

async function changeUserInfo(token: string, userUpdate: UserUpdate) {
  return fetch('http://localhost:3000/dashboard/api', {
    method: 'PUT',
    headers: {
      'Content-Type': 'application/json',
      Authorization: `Bearer ${token}`,
    },
    body: JSON.stringify(userUpdate),
  }).then((response) => response.json());
}

const Dashboard = () => {
  const [token] = useGetLoginToken();
  const queryClient = useQueryClient();
  const { data, isLoading } = useQuery<UserUpdate>({
    queryKey: ['stream-dashboard'],
    queryFn: () => getUserInfo(localStorage.getItem('token')),
  });

  useEffect(() => {
    if (!isLoading) {
      reset(data);
    }
  }, [isLoading]);

  const { mutate } = useMutation({
    mutationFn: (userUpdateData: UserUpdate) =>
      changeUserInfo(token, userUpdateData),
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: ['stream-dashboard'] });
      toast.success('Data changed');
    },
  });

  const handleSubmitForm = (newData: UserUpdate) => {
    if (!data) return;
    mutate({ ...data, address: newData.address, email: newData.email });
  };

  const {
    register,
    handleSubmit,
    formState: { errors },
    reset,
  } = useForm<UserUpdate>({
    resolver: zodResolver(UserUpdateSchema),
  });

  return (
    <main className={styles.main}>
      <div>
        <h1 style={{ textAlign: 'center' }}>
          Welcome, {data?.name} {data?.surname}!
        </h1>
        <h2 style={{ textAlign: 'center', marginBottom: '2rem' }}>
          Your Profile
        </h2>
        <a href={'/'}>Go to voting</a>
        <form
          onSubmit={handleSubmit(handleSubmitForm)}
          className={styles.regform}
        >
          <label>
            Address:
            <input type="text" {...register('address')} />
          </label>
          <span className={styles['error-message']}>
            {errors.address ? errors.address.message : '\u00A0'}
          </span>
          <label>
            Email:
            <input type="text" {...register('email')} />
          </label>
          <span className={styles['error-message']}>
            {errors.email ? errors.email.message : '\u00A0'}
          </span>
          <button type="submit">Change information</button>
        </form>
      </div>
    </main>
  );
};

export default Dashboard;
