'use client';
import styles from './page.module.css';
import { useForm } from 'react-hook-form';
import { LoginUser, LoginUserSchema } from '@/entities/LoginUserDto';
import { zodResolver } from '@hookform/resolvers/zod';
import { useState } from 'react';
import {
  decryptAESKeyWithRSA,
  decryptDataAES,
} from '@/utils/cipher/cryptoUtils';

export default function Home() {
  const [welcomeMessage, setWelcomeMessage] = useState('');
  const [errorMessage, setErrorMessage] = useState('');
  //const [itemsPost, setItemsPost] = useState([]);
  //const exampleData = {
  //  email: 'xantalp1@stuba.sk',
  //  password: 'Qwertuyioplk486.<',
  //};

  const {
    register,
    handleSubmit,
    formState: { errors },
  } = useForm<LoginUser>({
    resolver: zodResolver(LoginUserSchema),
  });

  const submitForm = (data: LoginUser) => {
    console.log('Form data:', data);
    fetch('http://localhost:3000/login/api', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json', // Set the request headers to indicate JSON format
      },
      body: JSON.stringify(data),
    })
      .then((res) => {
        if (res.status === 200) {
          return res.json().then((data) => {
            localStorage.setItem('token', data.token);
            const pk = localStorage.getItem('privateKey');
            if (pk) {
              const decryptedSymetricKey = decryptAESKeyWithRSA(
                data.symmetricKey,
                pk,
              );
              const decryptedData = {
                name: decryptDataAES(data.user.name, decryptedSymetricKey),
                surname: decryptDataAES(
                  data.user.surname,
                  decryptedSymetricKey,
                ),
              };
              setWelcomeMessage(
                `Welcome ${decryptedData.name} ${decryptedData.surname}`,
              );
              setErrorMessage(''); // Clear any previous error messages
              window.location.href = '/';
            }
          });
        } else if (res.status === 401) {
          return res.json().then((data) => {
            setErrorMessage(data.msg);
            setWelcomeMessage('');
          });
        } else if (res.status === 403) {
          return res.text().then((data) => {
            setErrorMessage(data);
            setWelcomeMessage('');
          });
        } else {
          console.log('Unexpected response code:', res.status);
        }
      })
      .catch((error) => {
        console.error('Error:', error);
      });
  };

  return (
    <main className={styles.main}>
      <div>
        {welcomeMessage && (
          <div className={styles.welcome}>{welcomeMessage}</div>
        )}
        {errorMessage && <div className={styles.error}>{errorMessage}</div>}
        <form onSubmit={handleSubmit(submitForm)} className={styles.regform}>
          <label> Email: </label>
          <input type="text" {...register('email')} />
          <span className={styles['error-message']}>
            {errors.email ? errors.email.message : '\u00A0'}
          </span>
          <label> Password: </label>
          <input type="password" {...register('password')} />
          <span className={styles['error-message']}>
            {errors.password ? errors.password.message : '\u00A0'}
          </span>
          <input type="submit" className={styles.btn} />
        </form>
      </div>
    </main>
  );
}

