'use client';
import styles from './page.module.css';
import { useState } from 'react';
import { useQuery } from '@tanstack/react-query';
import { Candidate } from '@/entities/Canditate';
import { useGetLoginToken } from '@/utils/hooks/useGetToken';
import { toast } from 'react-hot-toast';

async function getCandidates() {
  return (await fetch('http://localhost:3000/api', {
    method: 'GET',
    headers: {
      'Content-Type': 'application/json',
    },
  }).then((res) => res.json())) as Candidate[];
}

async function vote(token: string, id: number) {
  const request = {
    id: id.toString(),
  };
  console.log(token);
  fetch('http://localhost:3000/api', {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
      Authorization: `Bearer ${token}`,
    },
    body: JSON.stringify(request),
  })
    .then((res) => {
      if (res.status === 403) {
        res.text().then((resErr) => {
          toast.error(resErr);
        });
      }

      return res.json();
    })
    .then((data) => {
      console.log(data);
      window.location.href = '/pdf';
    })
    .catch((error) => console.error('Error:', error));
}

const Canditate = (props: { candidate: Candidate }) => {
  const [token] = useGetLoginToken();
  return (
    <div>
      {props.candidate.name} {props.candidate.surname}
      {token && (
        <button
          onClick={() => vote(token, props.candidate.id)}
          style={{
            marginLeft: '10px',
          }}
        >
          Vote
        </button>
      )}
    </div>
  );
};

const handleLogout = () => {
  localStorage.removeItem('token');
  window.location.reload();
};

export default function Home() {
  // redirect('/register');
  const { data } = useQuery({
    queryKey: ['stream-hydrate-users'],
    queryFn: () => getCandidates(),
    staleTime: 5 * 1000,
  });
  const [search, setSearch] = useState('');
  const [token] = useGetLoginToken();
  return (
    <main className={styles.main}>
      {token ? (
        <div style={{ display: 'flex', flexDirection: 'column' }}>
          <p>Logged in</p>
          <a href={'/dashboard'}>Dashboard</a>
          <button onClick={handleLogout}>Logout</button>
        </div>
      ) : (
        <div>
          <a href={'/login'}>Login</a>
          <p></p>
          <a href={'/register'}>Signup</a>
        </div>
      )}
      <div>
        <p>Search:</p>
        <input
          type={'text'}
          onChange={(event) => setSearch(event.target.value)}
        />
      </div>

      <div
        style={{
          marginTop: '10px',
        }}
      >
        {data ? (
          data
            .filter((candidate) =>
              `${candidate.name} ${candidate.surname}`
                .toLowerCase()
                .includes(search.toLowerCase()),
            )
            .map((candidate) => (
              <Canditate key={candidate.id} candidate={candidate} />
            ))
        ) : (
          <div>Loading</div>
        )}
      </div>
    </main>
  );
}
