'use client';

import styles from './page.module.css';
import { useGetLoginToken } from '@/utils/hooks/useGetToken';

async function downloadPDF(token: string): Promise<void> {
  const response = await fetch('http://localhost:3000/pdf/api', {
    method: 'GET',
    headers: {
      'Content-Type': 'application/pdf',
      Authorization: `Bearer ${token}`,
    },
  });
  if (!response.ok) {
    throw new Error(`Failed to download PDF: ${response.statusText}`);
  }

  const pdfData = await response.arrayBuffer();

  // Create a Blob from the PDF data
  const blob = new Blob([pdfData], { type: 'application/pdf' });

  // Create a download link
  const url = URL.createObjectURL(blob);
  const link = document.createElement('a');
  link.href = url;
  link.download = 'VotingCertificate.pdf';

  // Trigger the download
  link.click();

  // Clean up
  URL.revokeObjectURL(url);
}

export default function Home() {
  const [token] = useGetLoginToken();
  return (
    <main className={styles.main}>
      {token && (
        <button onClick={() => downloadPDF(token)}>
          PRESS to Download voting certificate
        </button>
      )}
      {/*<a href={'/pdf/api'} className={styles.btn} style={{ fontSize: '20px' }}>*/}
      {/*  PRESS to Download voting certificate*/}
      {/*</a>*/}
      <a href={'/'} className={styles.btn} style={{ fontSize: '15px' }}>
        Go to home page
      </a>
    </main>
  );
}
