'use client';

import styles from './page.module.css';
import { useForm } from 'react-hook-form';
import { useState } from 'react';
import {
  NewUserRequest,
  NewUserRequestSchema,
} from '@/entities/NewUserRequestDto';
import { zodResolver } from '@hookform/resolvers/zod';
// Import the necessary functions from cryptoUtils.ts
import {
  decryptAESKeyWithRSA,
  decryptDataAES,
  generateUsersKeyPair,
  getUsersPrivateKey,
  getUsersPublicKey,
} from '@/utils/cipher/cryptoUtils';

export default function Home() {
  const [itemsPost, setItemsPost] = useState([]);
  const [buttonClicked, setButtonClicked] = useState(false);
  const [privateKeyDisp, setPrivateKey] = useState('');
  const [dataFromServer, setDataFromServer] = useState({
    name: '',
    surname: '',
    email: '',
  });
  const [errorFromServer, setErrorFromServer] = useState('');
  const [symmetricKey, setSymmetricKey] = useState('');
  // const exampleData = {
  //   name: 'Patrik',
  //   surname: 'Antal',
  //   email: 'xantalp1@stuba.sk',
  //   password: 'Qwertuyioplk486.<',
  // };
  //
  // console.log(exampleData);

  const {
    register,
    handleSubmit,
    formState: { errors },
  } = useForm<NewUserRequest>({
    resolver: zodResolver(NewUserRequestSchema),
  });

  const submitForm = (data: NewUserRequest) => {
    setButtonClicked(false);
    console.log('submit started');
    const keyPair = generateUsersKeyPair();
    console.log(keyPair);
    const pk = getUsersPrivateKey(keyPair);
    setPrivateKey(pk);
    localStorage.setItem('privateKey', pk);

    const request = {
      user: data,
      publicKey: getUsersPublicKey(keyPair),
    };

    console.log('Form data:', data);
    fetch('http://localhost:3000/register/api', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json', // Set the request headers to indicate JSON format
      },
      body: JSON.stringify(request),
    })
      .then((res) => {
        if (res.status === 400) {
          setErrorFromServer('Password is too common');
          return undefined;
        } else if (res.status === 401) {
          setErrorFromServer('Email already exists');
          return undefined;
        }
        setErrorFromServer('');
        return res.json();
      }) // Parse the response data as JSON
      .then((data) => {
        if (!data) return;
        localStorage.setItem('token', data.token);
        console.log('response data: ' + data);
        const encryptedSymmetricKey = data.symmetricKey;
        const userEncrypted = data.user;

        const setData = {
          name: userEncrypted.name,
          surname: userEncrypted.surname,
          email: userEncrypted.email,
        };

        setDataFromServer(setData);
        console.log(data.user);

        console.log('response received: ');
        console.log('user: ' + userEncrypted);
        console.log('encryptedSymmetricKey: ' + encryptedSymmetricKey);

        console.log('privateKey: ' + pk);
        const decryptedSymmetricKey = decryptAESKeyWithRSA(
          encryptedSymmetricKey,
          pk,
        );

        setSymmetricKey(decryptedSymmetricKey);

        console.log('decryptedSymmetricKey: ' + decryptedSymmetricKey);

        setItemsPost(userEncrypted);
      });
    console.log(itemsPost);
  };

  const handleDecrypt = () => {
    console.log('Decrypt button clicked');
    console.log(buttonClicked);
    console.log(symmetricKey);

    if (buttonClicked) {
      console.log('Button clicked more then once');
      console.log(buttonClicked);
      return;
    }
    setButtonClicked(true);
    console.log(buttonClicked);

    if (dataFromServer) {
      const decryptedUser: any = {
        name: decryptDataAES(dataFromServer.name, symmetricKey),
        surname: decryptDataAES(dataFromServer.surname, symmetricKey),
        email: decryptDataAES(dataFromServer.email, symmetricKey),
      };
      setDataFromServer(decryptedUser);
    }
  };

  return (
    <main className={styles.main}>
      <form onSubmit={handleSubmit(submitForm)} className={styles.regform}>
        <label> Name: </label>
        <input type="text" {...register('name')} />
        <span className={styles['error-message']}>
          {errors.name ? errors.name.message : '\u00A0'}
        </span>
        <label> Surname: </label>
        <input type="text" {...register('surname')} />
        <span className={styles['error-message']}>
          {errors.surname ? errors.surname.message : '\u00A0'}
        </span>
        <label> Email: </label>
        <input type="text" {...register('email')} />
        <span className={styles['error-message']}>
          {errors.email ? errors.email.message : '\u00A0'}
        </span>
        <label> Password: </label>
        <input type="password" {...register('password')} />
        <span className={styles['error-message']}>
          {errors.password ? errors.password.message : '\u00A0'}
          {errorFromServer ? errorFromServer : '\u00A0'}
        </span>
        <input type="submit" className={styles.btn} />
      </form>
      <a href={'/'}>Go to home page</a>
      {dataFromServer.name !== '' && (
        <button
          onClick={handleDecrypt}
          className={styles.btn}
          style={{ fontSize: '20px' }}
        >
          PRESS to Decrypt
        </button>
      )}
      {dataFromServer.name !== '' && (
        <div className={styles.privateKeyContainer}>
          <h2>Data Received from Server:</h2>
          <p>Name: {dataFromServer.name}</p>
          <p>Surname: {dataFromServer.surname}</p>
          <p>Email: {dataFromServer.email}</p>
        </div>
      )}
      {privateKeyDisp && (
        <div className={styles.privateKeyContainer}>
          <div className={styles.privateKey}>
            <p>Private Key:</p>
            <p>{privateKeyDisp}</p>
          </div>
        </div>
      )}
    </main>
  );
}
