import { z } from 'zod';

const pwdStrengthMessage = {
  length: 'Your password should be at least 12 characters long',
  lowerUpperCase:
    'Your password should include both upper and lower case characters',
  specialCharacters:
    'Your password should include at least one special character: !@#$%^&*(),.?":{}|<>',
  numbers: 'Your password should include at least one number',
};

export const NewUserRequestSchema = z.object({
  name: z.string().min(1),
  surname: z.string().min(1),
  email: z.string().email(),
  password: z
    .string()
    .refine((value) => value.length >= 12, {
      message: pwdStrengthMessage.length,
    })
    .refine((value) => /[a-z]/.test(value) && /[A-Z]/.test(value), {
      message: pwdStrengthMessage.lowerUpperCase,
    })
    .refine((value) => /[!@#\$%\^&\*\(\),\.\?"\{\}:{}|<>]/.test(value), {
      message: pwdStrengthMessage.specialCharacters,
    })
    .refine((value) => /\d/.test(value), {
      message: pwdStrengthMessage.numbers,
    }),
});

export type NewUserRequest = z.infer<typeof NewUserRequestSchema>;
