import * as crypto from 'crypto-ts';
import forge from 'node-forge';
import * as cryptojs from 'crypto';
const generateRandomKey = () => {
  const key = crypto.lib.WordArray.random(16); // 128 bits key for AES-128
  return key.toString();
};

const generateUsersKeyPair = () => {
  return forge.pki.rsa.generateKeyPair(2048);
};

// the key would normally be obtained, for our uses, this will do
const getUsersPublicKey = (rsaKeyPair: forge.pki.rsa.KeyPair) => {
  // Convert the public key to a PEM format
  return forge.pki.publicKeyToPem(rsaKeyPair.publicKey);
};

const getUsersPrivateKey = (rsaKeyPair: forge.pki.rsa.KeyPair) => {
  // Convert the public key to a PEM format
  return forge.pki.privateKeyToPem(rsaKeyPair.privateKey);
};

const encryptAESKeyWithRSA = (symmetricKey: string, usersPublicKey: string) => {
  const publicKey = forge.pki.publicKeyFromPem(usersPublicKey);
  const encryptedSymmetricKey = publicKey.encrypt(symmetricKey, 'RSA-OAEP', {
    md: forge.md.sha256.create(),
  });
  return forge.util.encode64(encryptedSymmetricKey);
};

const decryptAESKeyWithRSA = (
  encryptedSymmetricKey: string,
  usersPrivateKey: string,
) => {
  const privateKey = forge.pki.privateKeyFromPem(usersPrivateKey);
  const decodedSymmetricKey = forge.util.decode64(encryptedSymmetricKey);
  const decryptedSymmetricKey = privateKey.decrypt(
    decodedSymmetricKey,
    'RSA-OAEP',
    {
      md: forge.md.sha256.create(),
    },
  );
  return decryptedSymmetricKey;
};

const encryptDataAES = (data: string, key: string) => {
  return crypto.AES.encrypt(data, key).toString();
};

const decryptDataAES = (encryptedData: string, key: string) => {
  const decryptedBytes = crypto.AES.decrypt(encryptedData, key);
  return decryptedBytes.toString(crypto.enc.Utf8);
};

function calculateHash(data: unknown) {
  return cryptojs
    .createHash('sha256')
    .update(JSON.stringify(data))
    .digest('hex');
}

export {
  generateRandomKey,
  encryptDataAES,
  encryptAESKeyWithRSA,
  decryptAESKeyWithRSA,
  decryptDataAES,
  generateUsersKeyPair,
  getUsersPublicKey,
  getUsersPrivateKey,
  calculateHash,
};
