import * as fs from 'fs';
import * as crypto from 'crypto';

export const hashPassword = (password: string): string => {
  // Generate a random salt
  const salt = crypto.randomBytes(16).toString('hex');

  // Hash password with salt
  const hash = crypto.createHmac('sha256', salt).update(password).digest('hex');

  // Return salt and hash separated by a '$' symbol
  return [salt, hash].join('$');
};

export const verifyPassword = (
  storedPassword: string,
  providedPassword: string,
): boolean => {
  // Split the stored password string into salt and hash
  const [salt, storedHash] = storedPassword.split('$');

  // Hash the provided password with the retrieved salt
  const hash = crypto
    .createHmac('sha256', salt)
    .update(providedPassword)
    .digest('hex');

  // Compare the hash of the provided password with the hash of the stored password
  return storedHash === hash;
};

export function checkPasswordInFile(password: string): Promise<boolean> {
  return new Promise<boolean>((resolve, reject) => {
    const passwordSet = new Set<string>();

    const readStream = fs.createReadStream(
      './100k-most-used-passwords-NCSC.txt',
      'utf-8',
    );

    readStream.on('data', (chunk: string) => {
      const lines = chunk.split('\n');
      lines.forEach((line) => {
        passwordSet.add(line.trim());
      });
    });

    readStream.on('end', () => {
      resolve(passwordSet.has(password));
    });

    readStream.on('error', (err) => {
      reject(err);
    });
  });
}
