<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\DB;

class CustomMigrationCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'migrate:custom';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command for migrating in docker.';

    /**
     * Execute the console command.
     */
    public function handle()
    {   
        $connected = false;

        Artisan::call('optimize');
        $this->info('Optimized!');
        do{
            try{
                $this->info('Checking database connection...');
                DB::connection()->getPdo();
                $connected = true;
            }catch(\Exception $e){
                $this->error($e->getMessage());
                $connected = false;
                sleep(3);
            }
        }while(!$connected);
        $this->info('Database connected!');
        try {
            $this->info('Composer install...');
            shell_exec('composer install');
            $this->info('Composer installed!');

            $this->info('NPM install...');
            shell_exec('npm install');
            $this->info('NPM installed!');

            $this->info('Migrating...');
            Artisan::call('migrate:fresh', [
                '--seed' => true,
            ]);
            $this->info('Migrated!');

            $this->info('Finalizing...');
            Artisan::call('optimize:clear');
            Artisan::call('key:generate');
            Artisan::call('config:cache');
            shell_exec('chmod -R 777 storage');
            shell_exec('npm run build');
        } catch (\Exception $e) {
            $this->error($e->getMessage());
            $this->error('Something failed!');
        }
    }
}