<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\Str;


class LoginController extends Controller
{
    public function index(){
        if(auth()->check()){
            return redirect('/profile');
        }
        return view('auth.login');
    }

    public function login(Request $request){
        $credentials = $request->validate([
            'email' => 'required|email',
            'password'=> 'required',
            ]);

        if (RateLimiter::tooManyAttempts($this->throttleKey($request), 3)) {
            return back()->withErrors([
                'email' => 'Too many login attempts. Please try again in '.RateLimiter::availableIn($this->throttleKey($request)).' seconds.',
            ])->onlyInput('email');
        }

        if (Auth::attempt($credentials)) {
            $request->session()->regenerate();
            RateLimiter::clear($this->throttleKey($request));
            return redirect()->intended('/profile');
        }

        RateLimiter::hit($this->throttleKey($request), 60);

        return back()->withErrors([
            'email' => 'The provided credentials do not match our records.',
        ])->onlyInput('email');
    }

    protected function throttleKey(Request $request = null)
    {
        return Str::lower($request->input('email')).'|'.$request->ip();
    }
}
