<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;

use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;

class RegisterController extends Controller
{
    /**
     * Get the error messages for the defined validation rules.
     *
     * @return array<string, string>
     */
    public function messages(): array
    {
        return [
            'password.regex' => 'Must contain 1 uppercase, 1 lowercase, 1 number, 1 special character',
        ];
    }

    protected $rules = [
        'first_name' => 'required|min:1|max:30',
        'last_name' => 'required|min:1|max:30',
        'email' => 'required|email|unique:users,email',
        'password' => 'required|min:8|max:20|regex:/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&.])[A-Za-z\d@$!%*?&]{8,}$/'
    ];

    public function index(){
        if(auth()->check()){
            return redirect('/login');
        }
        return view('auth.register');
    }

    public function register(Request $request){
        $data = $request->validate($this->rules, $this->messages());
        User::create([
            'first_name' => $data['first_name'],
            'last_name' => $data['last_name'],
            'email' => $data['email'],
            'password' => Hash::make($data['password']),
        ]);
        return redirect('/');
    }
}
