<?php

namespace App\Http\Controllers;

use App\Models\Review;
use App\Models\Item;
use Illuminate\Http\Request;

class ListingsController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $search = request()->query('search');
        if ($search) {
            $items = Item::where('name', 'LIKE', "%{$search}%")->paginate(12);
        } else {
            $items = Item::paginate(12);
        }
        return view('shop.listings', ['items' => $items]);
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        $item = Item::findOrFail($id);
        $reviews = Review::where('product_id', $id)->get();
        return view('shop.listing', ['item' => $item, 'reviews' => $reviews]);
    }

    public function showCart()
    {
        // Get the cart data from the cookie
        $cart = request()->cookie('cart');

        // Check if the cart cookie is set and is a serialized string
        if ($cart && is_string($cart) && @unserialize($cart) !== false) {
            $cart = unserialize($cart);

            // Get the items from the database
            $items = Item::whereIn('id', array_column($cart, 'product_id'))->get();

            // Attach the quantity information to each item
            foreach ($items as $item) {
                $key = array_search($item->id, array_column($cart, 'product_id'));
                $item->qty = $cart[$key]['quantity'];
            }
        } else {
            $items = [];
        }


        return view('shop.cart', [
            'cartItems' => $items,
            'hasPublicKey' => $this->checkIfHasPublicKey(),
        ]);
    }

    /**
     * Display a listing of the resource.
     */
    public function updateCart(Request $request)
    {
        // Get the existing cart
        $cart = request()->cookie('cart');
        $cart = $cart ? unserialize($cart) : [];

        // Add the new item to the cart
        $itemId = $request->itemId;
        $key = array_search($itemId, array_column($cart, 'product_id'));

        if ($key !== false && $request->update == 'increment') {
            // If the item is already in the cart, increment its quantity
            $cart[$key]['quantity']++;
        }
        else if($key !== false && $request->update == 'decrement') {
            // If the item is already in the cart, decrement its quantity
            $cart[$key]['quantity']--;
            // If the item quantity is zero or less, remove the item from the cart
            if ($cart[$key]['quantity'] <= 0) {
                unset($cart[$key]);
            }
        }
        else {
            // If the item is not in the cart, add it
            $cart[] = ['product_id' => $itemId, 'quantity' => 1];
        }

        // Save the updated cart back to the cookie
        cookie()->queue('cart', serialize($cart), 60);
    }

    private function checkIfHasPublicKey(){
        $user = auth()->user();
        if($user === null || $user->public_key == null || $user->public_key == ""){
            return false;
        }
        return true;
    }


}
