<?php

namespace App\Http\Controllers;

use App\Models\Item;
use App\Models\Order;
use Illuminate\Http\Request;

use function Sodium\bin2hex;

class OpenSSLController extends Controller
{

    public function encryptData($id)
    {
        // ALICE = SERVER
        // Create symmetric key
        $symm_key = openssl_random_pseudo_bytes(32);
        // file_put_contents('symm.key', $symm_key);

        // Fetch data from DB and encrypt it with symmetric key
        //$dataToEncrypt = $this->fetchDataFromDB(10);
        $dataToEncrypt = $this->invoiceContract($id);
        $iv = openssl_random_pseudo_bytes(16);
        // file_put_contents('iv.txt', $iv);
        $encrypted_data = openssl_encrypt($dataToEncrypt, 'aes-256-cbc', $symm_key, OPENSSL_RAW_DATA, $iv);
        // file_put_contents('encryptedData.txt', $encrypted_data);

        // Create asymmetric keys and save paths to the files
        $server_keys = $this->createAsymmKeypair();
        $user = auth()->user();
        $userPublicKey = $user->public_key;

        // Encrypt the symmetric key with the user's public key
        $server_private_key = $server_keys['private_key'];
        openssl_public_encrypt($symm_key, $encrypted_symm_key, $userPublicKey, OPENSSL_PKCS1_OAEP_PADDING);
        // file_put_contents('encryptedSymmKey.txt', $encrypted_symm_key);
        // file_put_contents('encryptedSymmKeyHEX.txt', bin2hex($encrypted_symm_key));


        openssl_sign($dataToEncrypt, $signatureForEncdSymmKey, $server_private_key, OPENSSL_ALGO_SHA256);
        // file_put_contents('signatureForEncdSymmKey.txt', $signatureForEncdSymmKey);
        // file_put_contents('signatureForEncdSymmKeyHEX.txt', bin2hex($signatureForEncdSymmKey));


        $server_public_key = $server_keys['public_key'];
        // file_put_contents('serverPublicKey.txt', $server_public_key);
        // file_put_contents('serverPublicKeyHEX.txt', bin2hex($server_public_key));



        $encryptionFileJson = $this->createEncryptionFile($server_public_key, $encrypted_symm_key, $encrypted_data, $iv, $signatureForEncdSymmKey, $dataToEncrypt);

        return $encryptionFileJson;
    }

    public function createEncryptionFile($server_public_key, $encrypted_symm_key, $encrypted_data, $iv, $signatureForEncdSymmKey, $dataToEncrypt)
    {
        $iv = bin2hex($iv);
        $encrypted_symm_key = bin2hex($encrypted_symm_key);
        $signatureForEncdSymmKey = bin2hex($signatureForEncdSymmKey);
        $server_public_key = bin2hex($server_public_key);

        // Symmetric Encryption Information
        $encryptionFormat['header']['Assymetric_encryption_method'] = 'RSA-OAEP';
        $encryptionFormat['header']['symmetric_encryption_method'] = 'AES-256-CBC';
        $encryptionFormat['header']['iv'] = $iv;

        //Info for Symmetric Key  Decryption and Verification
        $encryptionFormat['header']['server_public_key'] = $server_public_key;
        $encryptionFormat['header']['encrypted_symmetric_key'] = $encrypted_symm_key;
        $encryptionFormat['header']['signature_for_encd_symm_key'] = $signatureForEncdSymmKey;

        $encryptionFormat['data_file'] = bin2hex($encrypted_data);


        $encryptionFileJson = json_encode($encryptionFormat);

        //SIMULACIA CO JE V JS
        $decryptoinFile = json_decode($encryptionFileJson, true);
        $data = $dataToEncrypt;
        $signatureForEncdSymmKey = hex2bin($decryptoinFile['header']['signature_for_encd_symm_key']);
        $server_public_key = hex2bin($decryptoinFile['header']['server_public_key']);
        if (openssl_verify($data, $signatureForEncdSymmKey, $server_public_key, OPENSSL_ALGO_SHA256)) {
            $test = "Signature is valid";
        } else {
            $test = "Signature is invalid";
        }
        $encryptionFormat['test'] = $test;

        $encryptionFileJson = json_encode($encryptionFormat);

        return $encryptionFileJson;
    }


    public static function createAsymmKeypair()
    {

        // Generate a new RSA private key
        $private_key  = openssl_pkey_new([
            'private_key_bits' => 2048,
            'private_key_type' => OPENSSL_KEYTYPE_RSA,
        ]);


        // Extract the public key from the private key
        $public_key = openssl_pkey_get_details($private_key);
        $public_key = $public_key['key'];

        // Get private key in PEM format
        openssl_pkey_export($private_key, $private_key_pem);

        // Save the keys into file storage
        // file_put_contents("{$name}_public.key", $public_key);
        // file_put_contents("{$name}_private.key", $private_key_pem);

        // Return paths to the keys so we can work with them
        return [
            'public_key' => $public_key,
            'private_key' => $private_key,
        ];
    }

    public function fetchDataFromDB($numberOfRows)
    {
        $items = Item::select('name', 'description', 'price')
            ->limit($numberOfRows)
            ->get();
        // file_put_contents('dataToEncrypt.txt', $items);
        return $items;
    }

    public function invoiceContract($id)
    {
        // Fetch the order with the specified ID
        $order = Order::find($id);
        if (!$order) {
            abort(404);
        }

        // Check if the logged-in user has permission to download this invoice
        $user = auth()->user();
        if ($order->user_id !== $user->id) {
            abort(403, 'Unauthorized'); // Or handle the case where the user is not authorized
        }
        // Fetch the order with the specified ID for the logged-in user
        $order = Order::where('user_id', $user->id)->where('id', $id)->select('data')->first();
        
        $jsonData = $order->toArray();
        file_put_contents('invoice.txt', $jsonData);
        return file_get_contents('invoice.txt');
    }
}
