@extends('layouts.layout-default')

@section('title') Register @endsection

@section('content')
<div class="container-fluid h-100 py-5">

    <div class="container-fluid d-flex align-items-center justify-content-center w-25 h-100
        rounded" style="min-height: 80vh; background-color: #d5dbdc">
        <form method="POST" action="{{ route('auth.register') }}" class="container-fluid">
            @method('POST')
            @csrf
            <div class="form-group">
                <label for="first_name">First name</label>
                <input id="first_name" type="text" name="first_name" class="form-control form-control-sm"
                       value="{{ old('first_name') }}" required autofocus>
                @error('first_name')
                <div class="text-danger">{{ $message }}</div>
                @enderror
            </div>
            <div class="form-group">
                <label for="last_name">Last name</label>
                <input id="last_name" type="text" name="last_name" class="form-control form-control-sm"
                       value="{{ old('last_name') }}" required autofocus>
                @error('last_name')
                <div class="text-danger">{{ $message }}</div>
                @enderror
            </div>
            <div class="form-group">
                <label for="email">Email</label>
                <input id="email" type="email" name="email" class="form-control form-control-sm"
                       value="{{ old('email') }}" required autofocus>
                @error('email')
                <div class="text-danger">{{ $message }}</div>
                @enderror
            </div>
            <div class="form-group">
                <label for="password">Password</label>
                <input type="password" name="password" id="password" class="form-control form-control-sm"
                       required autofocus>
                @error('password')
                <div class="text-danger">{{ $message }}</div>
                @enderror
            </div>
            <button type="submit" class="btn btn-primary"
                    style="background-color: #39b4ae; border: none;"> Register
            </button>
        </form>
    </div>

</div>

<script>
    async function isPasswordCommon(password) {
        const sha1Hash = await sha1(password);
        const prefix = sha1Hash.substring(0, 5);
        const hashSuffix = sha1Hash.substring(5).toUpperCase(); // The API expects the suffix in uppercase

        const response = await fetch(`https://api.pwnedpasswords.com/range/${prefix}`);
        const responseBody = await response.text();

        const commonPasswords = responseBody.split('\n').map(line => line.split(':')[0]);

        return commonPasswords.includes(hashSuffix);
    }

    // Function to compute the SHA-1 hash of a string
    async function sha1(input) {
        const buffer = new TextEncoder().encode(input);
        const hashBuffer = await crypto.subtle.digest('SHA-1', buffer);
        const hashArray = Array.from(new Uint8Array(hashBuffer));
        return hashArray.map(byte => byte.toString(16).padStart(2, '0')).join('');
    }

    const passwordInput = document.getElementById('password');

    document.getElementById("password").addEventListener("blur", async function() {
    const password = passwordInput.value; // Get the entered password
    const isCommon = await isPasswordCommon(password);

    const p_err = document.getElementById('passwordError')
    if (isCommon) {
        p_err.innerHTML = '';
        p_err.display = 'block';
        p_err.innerHTML = 'Password is common. Please choose a stronger password.';
        p_err.style.color = 'red';
    } else {
        p_err.innerHTML = '';
        p_err.display = 'block';
        p_err.innerHTML = 'Password is not common. Proceed with registration.';
        p_err.style.color = 'green';
    }
});
</script>

@endsection
