package com.example.upbz5.rest;

import com.example.upbz5.exception.ErrorResponse;
import com.example.upbz5.exception.LoginException;
import com.example.upbz5.rest.dto.AccommodationFilter;
import com.example.upbz5.rest.dto.AccommodationResponse;
import com.example.upbz5.rest.dto.AccommodationsResponse;
import com.example.upbz5.service.AccommodationService;
import com.example.upbz5.service.UserService;
import lombok.AllArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/accommodation")
@AllArgsConstructor
public class AccommodationRest {

    private final AccommodationService accommodationService;

    private final UserService userService;

    @GetMapping("/all")
    public ResponseEntity<Object> allAccommodations(@RequestHeader("Authorization") String jwt) {
        try{
            userService.getUserIdByToken(jwt);
        }
        catch(LoginException e){
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(new ErrorResponse(e.getMessage(), HttpStatus.UNAUTHORIZED.value(), e.getClass().getName()));
        }
        AccommodationsResponse accommodations = accommodationService.getAllAccommodations();
        return ResponseEntity.status(HttpStatus.OK).body(accommodations);
    }

    @GetMapping("/top")
    public ResponseEntity<Object> topAccommodations(@RequestHeader("Authorization") String jwt) {
        try{
            userService.getUserIdByToken(jwt);
        }
        catch(LoginException e){
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(new ErrorResponse(e.getMessage(), HttpStatus.UNAUTHORIZED.value(), e.getClass().getName()));
        }
        AccommodationsResponse accommodations = accommodationService.getTopAccommodations();
        return ResponseEntity.status(HttpStatus.OK).body(accommodations);
    }

    @PostMapping("/filter")
    public ResponseEntity<Object> filterAccommodations(@RequestBody AccommodationFilter filter, @RequestHeader("Authorization") String jwt) {
        try{
            userService.getUserIdByToken(jwt);
        }
        catch(LoginException e){
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(new ErrorResponse(e.getMessage(), HttpStatus.UNAUTHORIZED.value(), e.getClass().getName()));
        }
        AccommodationsResponse accommodations = accommodationService.getAccommodationsByFilter(filter);
        return ResponseEntity.status(HttpStatus.OK).body(accommodations);
    }

    @GetMapping("/detail/{id}")
    public ResponseEntity<Object> accommodationDetail(@PathVariable("id") Long id, @RequestHeader("Authorization") String jwt) {
        try{
            userService.getUserIdByToken(jwt);
        }
        catch(LoginException e){
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(new ErrorResponse(e.getMessage(), HttpStatus.UNAUTHORIZED.value(), e.getClass().getName()));
        }
        AccommodationResponse accommodation = accommodationService.getAccommodationDetail(id);
        return ResponseEntity.status(HttpStatus.OK).body(accommodation);
    }

}
