package com.example.upbz5.rest;

import com.example.upbz5.exception.ErrorResponse;
import com.example.upbz5.exception.LoginException;
import com.example.upbz5.exception.NotFoundException;
import com.example.upbz5.rest.dto.ReservationRequest;
import com.example.upbz5.rest.dto.ReservationResponse;
import com.example.upbz5.rest.dto.ReservationSecretData;
import com.example.upbz5.service.ReservationService;
import com.example.upbz5.service.UserService;
import lombok.AllArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

@RestController
@RequestMapping("/reservation")
@AllArgsConstructor
public class ReservationRest {

    ReservationService reservationService;

    UserService userService;

    @PostMapping("/new")
    public ResponseEntity<Object> newReservation(@RequestBody ReservationRequest request, @RequestHeader("Authorization") String jwt) {
        try{
            request.setUserId(userService.getUserIdByToken(jwt));
        }
        catch(LoginException e){
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(new ErrorResponse(e.getMessage(), HttpStatus.UNAUTHORIZED.value(), e.getClass().getName()));
        }
        try {
            reservationService.createReservation(request);
            return ResponseEntity.status(HttpStatus.OK).build();

        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ErrorResponse(e.getMessage(), HttpStatus.BAD_REQUEST.value(), e.getClass().getName()));
        }
    }
    @GetMapping("/detail/{id}")
    public ResponseEntity<Object> reservationDetail(@PathVariable("id") Long id, @RequestHeader("Authorization") String jwt) {
        try{
            Long userId = userService.getUserIdByToken(jwt);
            if (!Objects.equals(reservationService.getReservationOwner(id), userId)){
                throw new LoginException("You do not have rights to see this reservation");
            }
        }
        catch(LoginException e){
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(new ErrorResponse(e.getMessage(), HttpStatus.UNAUTHORIZED.value(), e.getClass().getName()));
        }
        try{
            ReservationSecretData reservation = reservationService.reservationDetail(id);
            return ResponseEntity.status(HttpStatus.OK).body(reservation);
        } catch (NotFoundException e){
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(new ErrorResponse(e.getMessage(), HttpStatus.NOT_FOUND.value(), e.getClass().getName()));
        }
    }

    @GetMapping("/all")
    public ResponseEntity<Object> reservationsList(@RequestHeader("Authorization") String jwt) {
        Long userId;
        try{
            userId = userService.getUserIdByToken(jwt);
        }
        catch(LoginException e){
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(new ErrorResponse(e.getMessage(), HttpStatus.UNAUTHORIZED.value(), e.getClass().getName()));
        }
        try{
            List<ReservationResponse> reservations = reservationService.getUserReservations(userId);
            return ResponseEntity.status(HttpStatus.OK).body(reservations);
        } catch (NotFoundException e){
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(new ErrorResponse(e.getMessage(), HttpStatus.NOT_FOUND.value(), e.getClass().getName()));
        }
    }
}
