package com.example.upbz5.rest;

import com.example.upbz5.exception.DataEncryptionException;
import com.example.upbz5.exception.ErrorResponse;
import com.example.upbz5.exception.NotFoundException;
import com.example.upbz5.exception.RegistrationException;
import com.example.upbz5.rest.dto.*;
import com.example.upbz5.service.UserService;
import lombok.AllArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/api")
@AllArgsConstructor
public class UserRest {

    private final UserService userService;

    @PostMapping("/register")
    public ResponseEntity<Object> registerUser(@RequestBody UserRegistrationRequest registrationRequest) {
        try {
            UserDto registeredUser = userService.registerUser(registrationRequest);
            return ResponseEntity.status(HttpStatus.CREATED).body(registeredUser);
        } catch (RegistrationException e) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ErrorResponse(e.getMessage(), HttpStatus.BAD_REQUEST.value(), e.getClass().getName()));
        }
    }

    @PostMapping("/login")
    public ResponseEntity<Object> loginUser(@RequestBody UserLoginRequest loginRequest) {
        try {
            UserDto loggedUser = userService.loginUser(loginRequest);
            return ResponseEntity.status(HttpStatus.OK).body(loggedUser);
        } catch (NotFoundException e) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(new ErrorResponse(e.getMessage(), HttpStatus.BAD_REQUEST.value(), e.getClass().getName()));
        }
    }

    @GetMapping("/validation")
    public ResponseEntity<ValidationDto> isValidToken(@RequestHeader("Authorization") String jwt) {
        return ResponseEntity.status(HttpStatus.OK).body(userService.isValidToken(jwt));
    }
}

