package com.example.upbz5.service;

import com.example.upbz5.exception.NotFoundException;
import com.example.upbz5.exception.RegistrationException;
import com.example.upbz5.models.Accommodation;
import com.example.upbz5.models.User;
import com.example.upbz5.repository.AccommodationRepository;
import com.example.upbz5.rest.dto.AccommodationFilter;
import com.example.upbz5.rest.dto.AccommodationResponse;
import com.example.upbz5.rest.dto.AccommodationsResponse;
import com.example.upbz5.rest.dto.DatesPair;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
@Slf4j
@AllArgsConstructor
public class AccommodationsServiceImpl implements AccommodationService{

    private final AccommodationRepository accommodationRepository;

    @Override
    public AccommodationsResponse getAllAccommodations() {
        List <Accommodation> accommodations = accommodationRepository.findAll();
        return new AccommodationsResponse(accommodations.size(), accommodations);
    }

    @Override
    public AccommodationsResponse getAccommodationsByFilter(AccommodationFilter accommodationFilter) {
        Specification<Accommodation> specification = accommodationRepository.getAccommodationSpecification(accommodationFilter);
        List <Accommodation> accommodations = accommodationRepository.findAll(specification);
        return new AccommodationsResponse(accommodations.size(), accommodations);
    }

    @Override
    public AccommodationsResponse getTopAccommodations() {
        List <Accommodation> accommodations = accommodationRepository.findByTopTrue();
        return new AccommodationsResponse(accommodations.size(), accommodations);
    }

    @Override
    public AccommodationResponse getAccommodationDetail(Long id) {
        Optional<Accommodation> accommodation = accommodationRepository.findById(id);
        if(accommodation.isEmpty()){
            throw  new NotFoundException("Accommodation with this id:" + id + " not found");
        }
        List<DatesPair> reservationDates = accommodationRepository.findDatesPairsByAccommodation(accommodation.get());
        return new AccommodationResponse(accommodation.get(), reservationDates);
    }

    public Accommodation getAccommodationById(Long id) throws NotFoundException{
        Optional<Accommodation> accommodation = accommodationRepository.findById(id);
        if(accommodation.isEmpty()){
            throw new NotFoundException("Accommodation with this id:" + id + " doesn't exist");
        }
        return accommodation.get();
    }

    public List<DatesPair> getReservationDates(Accommodation accommodation){
        return accommodationRepository.findDatesPairsByAccommodation(accommodation);
    }

}
