import {RegistrationForm} from "./registration/registration-form";
import Navbar from "./navbar/navbar";
import React, {useState} from 'react';
import {BrowserRouter as Router, Navigate, Route, Routes} from 'react-router-dom';
import {SignInForm} from "./signin/sign-in-form";
import {ToastContainer} from "react-toastify";
import 'react-toastify/dist/ReactToastify.css';

import {EncryptorForm} from "./encryptor/encryptor-form";

import {ProtectedRoute} from "./private-r";
import {AccommodationPage} from "./accommodation/accommodation-page";
import {AccommodationDetailPage} from "./accommodation/detail/accommodation-detail-page";
import {ReservationPage} from "./reservation/reservation-page";


export function setCookie(name, value, expirationInMinutes) {
    const expirationDate = new Date();
    expirationDate.setTime(expirationDate.getTime() + (expirationInMinutes * 60 * 1000));
    const expires = "expires=" + expirationDate.toUTCString();
    document.cookie = name + "=" + encodeURIComponent(value) + ";" + expires + ";path=/";
}


export function getCookie(name) {
    const cookies = document.cookie.split(';');

    for (let i = 0; i < cookies.length; i++) {
        const cookie = cookies[i].trim();
        if (cookie.startsWith(name + '=')) {
            return decodeURIComponent(cookie.substring(name.length + 1));
        }
    }

    return null;
}
export function removeCookie(name) {
    document.cookie = name + '=; expires=Thu, 01 Jan 1970 00:00:00 UTC; path=/;';
}



function App() {

    return (
        <div className="App">
            <ToastContainer/>
            <Router>
                <Navbar/>
                <Routes>
                    <Route exact path="/registration" element={<RegistrationForm></RegistrationForm>}></Route>
                    <Route exact path="/signin" element={<SignInForm></SignInForm>}></Route>
                    <Route exact path="/accommodation" element={<AccommodationPage></AccommodationPage>}></Route>
                    <Route exact path="/accommodation/detail/:detailId" element={<AccommodationDetailPage></AccommodationDetailPage>}></Route>
                    <Route exact path="/reservations" element={<ReservationPage></ReservationPage>}></Route>
                    <Route path="/" element={<Navigate to="/accommodation" />} />
                </Routes>
            </Router>

        </div>
    );
}

export default App;
