import {
    Button, Card, CardContent, CardMedia, Grid, InputAdornment, MenuItem, Select, TextField, Typography
} from "@mui/material";
import React, {useEffect, useState} from "react";
import {DatePicker, LocalizationProvider} from "@mui/x-date-pickers";
import {AdapterDayjs} from "@mui/x-date-pickers/AdapterDayjs";
import {useFormik} from "formik";
import {SummaryDialog} from "../../modal/summary-dialog";
import {useNavigate, useParams} from 'react-router-dom';
import {getCookie} from "../../App";
import {toast} from "react-toastify";
import * as Yup from "yup";
import ConfirmDialog from "../../modal/confirmation-dialog";

const url = process.env.REACT_APP_API_URL
export const AccommodationDetailPage = () => {
    const navigate = useNavigate()
    const [selectedFromDate, setSelectedFromDate] = useState(null);
    const [selectedToDate, setSelectedToDate] = useState(null)
    const [guests, setGuests] = useState("default");
    const [price, setPrice] = useState(0);
    const [openDialog, setOpenDialog] = useState(false)
    const [accommodationDetail, setAccommodationDetail] = useState(null)
    const [loaded, setLoaded] = useState(false)
    const {detailId} = useParams();
    const [dates, setDates] = useState([])
    const [openConfirmDialog, setOpenConfirmDialog] = useState(false)


    useEffect(() => {
        fetchData();
    }, []);
    const fetchData = async () => {
        const token = getCookie('auth');
        try {
            const response = await fetch(url + '/accommodation/detail/' + detailId, {
                method: 'GET', headers: {
                    'Authorization': `${token}`,
                },
            },)

            if (!response.ok) {
                if (response.status === 401) {
                    navigate("/signin")
                } else {
                    toast.error(response.message)
                }
            } else {
                const result = await response.json();
                setAccommodationDetail(result.accommodation)
                setDates(result.reservations)
                setLoaded(true)

            }

        } catch (error) {
            console.error('Error fetching data:', error);
        }
    };

    const disableKeyboardEntry = (e) => {
        if (e?.preventDefault) {
            e?.preventDefault();
            e?.stopPropagation();
        }
    }
    const generateMenuItems = (num) => {
        const items = [];
        for (let i = 1; i <= num; i++) {
            items.push(<MenuItem key={i} value={i}>{i} {i === 1 ? 'guest' : 'guests'}</MenuItem>);
        }
        return items;
    };
    const getDayDiff = (date1, date2) => {
        const oneDay = 24 * 60 * 60 * 1000; // hours * minutes * seconds * milliseconds
        const timeDiff = date2.getTime() - date1.getTime();
        const dayDiff = timeDiff / oneDay;
        return dayDiff;
    };

    const formik = useFormik({
        initialValues: {
            price: undefined, guests: undefined, dateFrom: undefined, dateTo: undefined,
        }, onSubmit: (values) => {

        }
    });

    const handleBook = () => {
        setOpenDialog(true)
    }

    const handleCloseDialog = () => {
        setOpenDialog(false)
    }


    const reloadPage = () => {
        formik.resetForm();
        fetchData().then(r => {
            setPrice(0)
            setSelectedFromDate(null)
            setSelectedToDate(null)
            setGuests("default")
            setOpenConfirmDialog(true)
        })

    }

    return (<>
        {loaded && <Grid container justifyContent="center" style={{marginTop: '13vh'}}>

            <Grid item xs={11} md={8}>

                <Card>
                    <CardMedia component="img" height="250rem" image={accommodationDetail.url}
                               alt={accommodationDetail.name}/>
                    <CardContent>
                        <Typography variant="h5" gutterBottom>
                            {accommodationDetail.name}
                        </Typography>
                        <Typography variant="body1" gutterBottom>
                            {accommodationDetail.description}
                        </Typography>

                        <Typography variant="body2" color="textSecondary">
                            Address: 123 Main Street, City, Country
                        </Typography>
                        <Typography variant="body2" color="textSecondary">
                            Price: {accommodationDetail.price} € per night
                        </Typography>
                        <div style={{
                            paddingTop: '1rem',
                            paddingBottom: '1rem',
                            display: 'flex',
                            gap: '1rem',
                            flexWrap: 'wrap'
                        }}>
                            <LocalizationProvider dateAdapter={AdapterDayjs}>
                                <DatePicker
                                    label="From"
                                    value={selectedFromDate}
                                    onChange={(newValue) => {
                                        setSelectedFromDate(newValue);
                                        formik.setFieldValue('dateFrom', newValue)
                                        if (new Date(newValue) >= formik.values.dateTo) {
                                            formik.setFieldValue('price', undefined)
                                            setPrice(0)
                                        } else {
                                            formik.setFieldError('dateFrom', ''); // Clear the error if the condition is no longer met
                                            if (formik.values.dateTo !== undefined && formik.values.guests !== undefined) {
                                                const diffDays = getDayDiff(new Date(newValue), new Date(formik.values.dateTo));
                                                setPrice(diffDays * accommodationDetail.price)
                                                formik.setFieldValue('price', diffDays * accommodationDetail.price)

                                            }
                                        }

                                    }}
                                    slotProps={{
                                        textField: {
                                            onBeforeInput: disableKeyboardEntry,
                                            variant: 'standard',
                                            error: (formik.values.dateFrom !== undefined && formik.values.dateTo !== undefined) && formik.values.dateFrom >= formik.values.dateTo,
                                        }
                                    }}
                                    shouldDisableDate={(dateValue) => {
                                        let date = new Date(dateValue)
                                        if (date < new Date()) {
                                            return true;
                                        }
                                        for (const range of dates) {
                                            let dateFromFormated = new Date(range.dateFrom)
                                            if ((date >= new Date(dateFromFormated - 24 * 60 * 60 * 1000) && date <= new Date(range.dateTo))) {
                                                return true;
                                            }

                                        }
                                        return false;
                                    }}
                                />
                                <DatePicker
                                    label="To"
                                    value={selectedToDate}
                                    onChange={(newValue) => {
                                        setSelectedToDate(newValue);
                                        formik.setFieldValue('dateTo', newValue)
                                        if (formik.values.dateFrom >= new Date(newValue)) {
                                            formik.setFieldValue('price', undefined)
                                            setPrice(0)
                                        } else {
                                            if (formik.values.dateFrom !== undefined && formik.values.guests !== undefined) {
                                                const diffDays = getDayDiff(new Date(formik.values.dateFrom), new Date(newValue),);
                                                setPrice(diffDays * accommodationDetail.price)
                                                formik.setFieldValue('price', diffDays * accommodationDetail.price)

                                            }
                                        }

                                    }}
                                    shouldDisableDate={(dateValue) => {
                                        let date = new Date(dateValue)
                                        if (date < new Date()) {
                                            return true;
                                        }
                                        for (const range of dates) {
                                            let dateFromFormated = new Date(range.dateFrom)
                                            if ((date >= new Date(dateFromFormated - 24 * 60 * 60 * 1000) && date <= new Date(range.dateTo))) {
                                                return true;
                                            }
                                        }
                                        return false;
                                    }}
                                    slotProps={{
                                        textField: {
                                            onBeforeInput: disableKeyboardEntry,
                                            error: (formik.values.dateFrom !== undefined && formik.values.dateTo !== undefined) && formik.values.dateFrom >= formik.values.dateTo,
                                            variant: 'standard'
                                        }
                                    }}
                                />

                            </LocalizationProvider>
                            <Select
                                value={guests}
                                onChange={(e) => {
                                    setGuests(e.target.value)
                                    console.log("change")
                                    formik.setFieldValue('guests', e.target.value)
                                    if (formik.values.dateFrom >= formik.values.dateTo) {

                                    } else {
                                        if (formik.values.dateFrom !== undefined && formik.values.dateTo !== undefined) {
                                            const diffDays = getDayDiff(new Date(formik.values.dateFrom), new Date(formik.values.dateTo));
                                            setPrice(diffDays * accommodationDetail.price)
                                            formik.setFieldValue('price', diffDays * accommodationDetail.price)
                                        }
                                    }
                                }}
                                variant="standard"
                                label="Guests"
                                style={{width: '10rem'}}
                            >
                                <MenuItem value="default" key="default" disabled selected>
                                    Number of Guests
                                </MenuItem>
                                {generateMenuItems(accommodationDetail.capacity)}
                            </Select>

                        </div>
                        <div style={{
                            display: 'flex',
                            justifyContent: 'space-between',
                            alignItems: 'center',
                            marginTop: '1rem',
                            flexWrap: 'wrap',
                            gap: '1rem'
                        }}>

                            <TextField
                                id="price"
                                name="price"
                                type="number"
                                label="Price"
                                style={{width: '15rem'}}
                                disabled={true}
                                value={price}
                                InputProps={{
                                    endAdornment: <InputAdornment position="end">€</InputAdornment>,
                                }}
                            />
                            <Button variant="contained" onClick={handleBook}
                                    disabled={formik.values.price === undefined}>
                                Book Now
                            </Button>
                        </div>
                    </CardContent>
                </Card>
            </Grid>


            <SummaryDialog open={openDialog} handleClose={handleCloseDialog}
                           accommodation={accommodationDetail} numberOfPeople={guests} formik={formik}
                           priceSum={price} reloadPage={reloadPage}></SummaryDialog>
            <ConfirmDialog open={openConfirmDialog}
                           setOpen={setOpenConfirmDialog}></ConfirmDialog>

        </Grid>}</>);


}