import {useFormik} from "formik";
import React, {useState} from 'react';
import {Button, Grid, TextField, Typography} from "@mui/material";
import * as Yup from 'yup';
import {toast} from "react-toastify";
import forge from "node-forge";
import {getCookie} from "../App";
import {saveAs} from "file-saver"
import CryptoJS from "crypto-js";


const url = process.env.REACT_APP_API_URL
export const EncryptorForm = arrayLike => {
    const [maxRequests, setMaxRequest] = useState(3);
    const [requestInterval, setRequestInterval] = useState(5000)
    const [requestCount, setRequestCount] = useState(0)
    const [lastRequestTime, setLastRequestTime] = useState(0)

    const formik = useFormik({
        initialValues: {
            privateKey: '',
            data: ''
        },
        onSubmit: async (values) => {
            try {
                await fetchWithRateLimit(values)
            } catch (error) {
                if (error instanceof Error) {
                    toast.error(error.message); // Display the error message in a toast
                } else {
                    const errorMessage = await error.text();
                    const errorObject = JSON.parse(errorMessage);
                    toast.error(errorObject.error); // Display the custom backend exception message in a toast
                }
            }
        }
    });

    const fetchWithRateLimit = async (values) => {


        return new Promise((resolve, reject) => {

            const makeRequest = async arrayLike => {
                const currentTime = new Date().getTime();
                if (currentTime - lastRequestTime < requestInterval) {
                    setRequestCount(requestCount + 1);
                    setLastRequestTime(currentTime)
                    if (requestCount > maxRequests) {
                        reject(new Error('Too many requests, please try again later.'));
                        return;
                    }
                } else {
                    setRequestCount(1)
                    setLastRequestTime(currentTime)
                }
                const token = getCookie('auth');
                try {

                    /* const response = await fetch(url + '/data', {
                         method: 'GET',
                         headers: {
                             'Authorization': `${token}`,
                         },
                     });
                     const jsonData = await response.json();

                     if (!response.ok) {
                         toast.error(jsonData.message)
                     } else {
                         const forge = require('node-forge');
                          const privateKK = `-----BEGIN PRIVATE KEY-----\n${values.privateKey}\n-----END PRIVATE KEY-----`;
                         const privateKey = forge.pki.privateKeyFromPem(privateKK);
                         const encryptedBytes = forge.util.decode64(jsonData.encrypted);
                         const decrypted = privateKey.decrypt(encryptedBytes);
                         await formik.setFieldValue('data', decrypted)
                         //toast.success(decrypted);
                     }*/
                    const response = await fetch(url + '/reservation/detail/67', {
                        method: 'GET',
                        headers: {
                            'Authorization': `${token}`,
                        },
                    });
                    const jsonData = await response.json();

                    if (!response.ok) {
                        toast.error(jsonData.message)
                    } else {
                        const CryptoJS = require('crypto-js');
                        const forge = require('node-forge');
                        const privateKK = `-----BEGIN PRIVATE KEY-----\n${values.privateKey}\n-----END PRIVATE KEY-----`;
                        const privateKey = forge.pki.privateKeyFromPem(privateKK);
                        const encryptedKeyAES = forge.util.decode64(jsonData.encryptedAes);
                        const decryptedKey = privateKey.decrypt(encryptedKeyAES);
                        const encryptedDataBinary = atob(jsonData.pdf);
                        var key = CryptoJS.enc.Utf8.parse(decryptedKey);

                        const encryptedDataWordArray = CryptoJS.lib.WordArray.create(Uint8Array.from(encryptedDataBinary, c => c.charCodeAt(0)));

                        var decryptedBytes = CryptoJS.AES.decrypt(
                            { ciphertext: CryptoJS.enc.Base64.parse(jsonData.pdf) },
                            key,
                            { mode: CryptoJS.mode.ECB, padding: CryptoJS.pad.Pkcs7 }
                        );

                        var decryptedString = CryptoJS.enc.Utf8.stringify(decryptedBytes);

                        const uint8Array = new Uint8Array(JSON.parse(decryptedString));

                        const pdfBlob = new Blob([uint8Array], {type: 'application/pdf'});
                        saveAs(pdfBlob, 'reservation.pdf');

                    }


                } catch (error) {

                    console.log("ERRRRRROOOOOOOOOOOOORRRRRR" + error)
                    if (error.response) {

                        const errorData = await error.response.json();
                        toast.error(errorData.message); // Extract the error message from the response
                    } else if (error.request) {

                        toast.error('No response received from server. ');
                    } else {
                        toast.error('An error occured.')

                    }
                }

            };

            makeRequest();
        });
    };

    return (
        <Grid
            container
            spacing={2}
            direction="column"
            alignItems="center"
            justifyContent="center"
            style={{minHeight: '100vh', width: '80%', maxWidth: '1200px', margin: '0 auto'}}
        >
            <form onSubmit={formik.handleSubmit}>
                <Grid container justifyContent="center" alignItems="center" width="100%" spacing={2}>
                    <Grid item xs={12} md={8} lg={8}>
                        <TextField
                            id="privateKey"
                            name="privateKey"
                            label="Private Key"
                            fullWidth
                            value={formik.values.privateKey}
                            onChange={formik.handleChange}
                            error={formik.touched.privateKey && Boolean(formik.errors.privateKey)}
                            helperText={formik.touched.privateKey && formik.errors.privateKey}
                        />
                    </Grid>
                    <Grid item xs={12} md={4} lg={2}>
                        <Button type="submit" fullWidth variant="contained" style={{height: '100%'}}>
                            Decode
                        </Button>
                    </Grid>
                    <Grid item>
                        <TextField
                            id="data"
                            name="data"
                            label="Decrypted Data"
                            disabled
                            fullWidth
                            multiline
                            rows={5}
                            value={formik.values.data}
                            error={formik.touched.data && Boolean(formik.errors.data)}
                            helperText={formik.touched.data && formik.errors.data}
                        />
                    </Grid>
                </Grid>
            </form>
        </Grid>
    );

};
