import React, {useEffect, useState} from "react";
import DialogTitle from "@mui/material/DialogTitle";
import {Button, IconButton, Typography} from "@mui/material";
import {styled} from "@mui/material/styles";
import Dialog from "@mui/material/Dialog";
import CloseIcon from "@mui/icons-material/Close";
import DialogContent from "@mui/material/DialogContent";
import DialogActions from "@mui/material/DialogActions";
import {useNavigate} from "react-router-dom";

const BootstrapDialog = styled(Dialog)(({ theme }) => ({
    '& .MuiDialogContent-root': {
        padding: theme.spacing(2),
    },
    '& .MuiDialogActions-root': {
        padding: theme.spacing(1),
    },
}));



export default function ConfirmDialog(props) {
    const navigate=useNavigate()
     const handleClose = () => {
        props.setOpen(false);
    };

    return (
        <React.Fragment>
            <BootstrapDialog
                onClose={handleClose}
                aria-labelledby="customized-dialog-title"
                open={props.open}
            >
                <DialogTitle sx={{ m: 0, p: 2 }} id="customized-dialog-title">
                    Successful reservation!
                </DialogTitle>
                <IconButton
                    aria-label="close"
                    onClick={handleClose}
                    sx={{
                        position: 'absolute',
                        right: 8,
                        top: 8,
                        color: (theme) => theme.palette.grey[500],
                    }}
                >
                    <CloseIcon />
                </IconButton>
                <DialogContent dividers>
                    <Typography gutterBottom>
                        Hooray! Your accommodation reservation was successful.
                    </Typography>
                    <Typography gutterBottom>
                        You can find it in the list of your reservations. Just click on the button "My reservations" or move to the page using the navigation bar.
                    </Typography>
                </DialogContent>
                <DialogActions>
                    <Button autoFocus variant="contained" onClick={()=>{
                        navigate("/reservations")
                    }}>
                        My reservations
                    </Button>
                    <Button autoFocus onClick={handleClose}>
                        OK
                    </Button>
                </DialogActions>
            </BootstrapDialog>
        </React.Fragment>
    );
}