import * as React from 'react';
import Button from '@mui/material/Button';
import Dialog from '@mui/material/Dialog';
import DialogTitle from '@mui/material/DialogTitle';
import DialogContent from '@mui/material/DialogContent';
import DialogActions from '@mui/material/DialogActions';
import IconButton from '@mui/material/IconButton';
import CloseIcon from '@mui/icons-material/Close';
import Typography from '@mui/material/Typography';
import {styled} from '@mui/material/styles';
import {Grid} from "@mui/material";
import PlaceIcon from '@mui/icons-material/Place';
import {getCookie} from "../App";
import {toast} from "react-toastify";
import {useState} from "react";
import {useNavigate} from "react-router-dom";

const BootstrapDialog = styled(Dialog)(({theme}) => ({
    '& .MuiDialogContent-root': {
        padding: theme.spacing(2),
    },
    '& .MuiDialogActions-root': {
        padding: theme.spacing(1),
    },
    '& .MuiPaper-root': {
        width: '70%', // Adjust the width as per your requirement
    },
}));

const url = process.env.REACT_APP_API_URL
export const SummaryDialog = (props) => {
    const [maxRequests, setMaxRequest] = useState(3);
    const [requestInterval, setRequestInterval] = useState(5000)
    const [requestCount, setRequestCount] = useState(0)
    const [lastRequestTime, setLastRequestTime] = useState(0)
    const navigate = useNavigate()

    const formateDate = (dateToFormat) => {
        const [day, month, year] = dateToFormat.split('/');
        const formattedDateFrom = `${year}-${month}-${day}`;
        const dateObject = new Date(formattedDateFrom);
        const formattedDate = dateObject.toLocaleDateString();
    }

    const handleBook = async () => {

        try {
            const existFrom = new Date(props.formik.values.dateFrom)
            existFrom.setHours(1)
            const existTo = new Date(props.formik.values.dateTo)
            existTo.setHours(1)
            await fetchWithRateLimit({
                userId: null,
                accommodationId: props.accommodation.id,
                checkInDate: existFrom.toISOString(),
                checkOutDate: existTo.toISOString(),
                guestsCount: props.numberOfPeople
            })
        } catch (error) {

            if (error instanceof Error) {
                toast.error(error.message); // Display the error message in a toast
            } else {
                const errorMessage = await error.text();
                const errorObject = JSON.parse(errorMessage);
                toast.error(errorObject.error); // Display the custom backend exception message in a toast
            }
        }

    }

    const fetchWithRateLimit = async (body) => {
        return new Promise((resolve, reject) => {
            const makeRequest = async () => {
                const currentTime = new Date().getTime();
                if (currentTime - lastRequestTime < requestInterval) {
                    setRequestCount(requestCount + 1);
                    setLastRequestTime(currentTime)
                    if (requestCount > maxRequests) {
                        reject(new Error('Too many requests, please try again later.'));
                        return;
                    }
                } else {
                    setRequestCount(1)
                    setLastRequestTime(currentTime)
                }
                try {
                    const token = getCookie('auth');
                    const response = await fetch(url + '/reservation/new', {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json',
                            'Authorization': `${token}`
                        },
                        body: JSON.stringify(body)
                    });
                    if (!response.ok) {
                        if (response.status === 401) {
                            navigate("/signin")
                        } else {
                            const jsonData = await response.json();
                            toast.error(jsonData.message)
                        }
                    } else {
                        props.handleClose()
                        props.reloadPage()
                    }
                } catch (error) {

                    reject(error);
                }
            };

            makeRequest();
        });
    };


    return (<>

        <BootstrapDialog
            onClose={props.handleClose}
            aria-labelledby="customized-dialog-title"
            open={props.open}
        >
            <DialogTitle sx={{m: 0, p: 2, backgroundColor: '#1565c0', color: 'white'}} id="customized-dialog-title">
                BookIT Summarization
            </DialogTitle>
            <IconButton
                aria-label="close"
                onClick={props.handleClose}
                sx={{
                    position: 'absolute',
                    right: 8,
                    top: 8,
                    color: (theme) => theme.palette.grey[500],
                }}
            >
                <CloseIcon/>
            </IconButton>
            <DialogContent dividers sx={{p: 4}}>
                <Grid container spacing={2} alignItems="center">
                    <Grid item>
                        <img src={props.accommodation.url} alt={props.accommodation.name}
                             style={{width: 100, height: 100, objectFit: 'cover'}}/>
                    </Grid>
                    <Grid item xs>
                        <Typography variant="h6" gutterBottom>
                            Accommodation:
                        </Typography>
                        <Typography gutterBottom>{props.accommodation.name}</Typography>
                        <Typography gutterBottom> <PlaceIcon></PlaceIcon>{props.accommodation.address}</Typography>
                    </Grid>
                </Grid>
                <Typography variant="h6" style={{marginTop: '0.5rem'}} gutterBottom>
                    Dates:
                </Typography>
                <Typography
                    gutterBottom> {new Date(props.formik.values.dateFrom).toLocaleDateString()} - {new Date(props.formik.values.dateTo).toLocaleDateString()}</Typography>
                <Typography variant="h6" gutterBottom>
                    Number of People:
                </Typography>
                <Typography gutterBottom>{props.formik.values.guests} Guests</Typography>
                <Typography variant="h6" gutterBottom>
                    Price:
                </Typography>
                <Typography gutterBottom>{props.priceSum} €</Typography>

            </DialogContent>
            <DialogActions>
                <Button autoFocus onClick={handleBook}>
                    Book Now
                </Button>
            </DialogActions>
        </BootstrapDialog>


    </>)

}