import {Navigate, Outlet} from "react-router-dom";
import React, {useEffect, useState} from "react";
import {toast} from "react-toastify";
import {EncryptorForm} from "./encryptor/encryptor-form";
import {getCookie} from "./App";

const url = process.env.REACT_APP_API_URL
export const ProtectedRoute = (props) => {
    const [auth, setAuth] = useState(true)
    const [loading, setLoading] = useState(true);
    useEffect(() => {
        const fetchData = async () => {

            if (!getCookie('auth')) {
                setAuth(false);
                setLoading(false)
            } else {

                const token = getCookie('auth');
                try {
                    const response = await fetch('http://localhost:8443/api/validation', {
                        method: 'GET',
                        headers: {
                            'Authorization': `${token}`,
                        },
                    });
                    const jsonData = await response.json();

                    if (!response.ok) {
                        toast.error(jsonData.message);
                    } else {

                        if (jsonData.valid) {
                            setAuth(true);
                            setLoading(false)

                        } else {
                            setAuth(false);
                            setLoading(false)

                        }
                    }
                } catch (error) {

                }
            }
        };

        fetchData();
    }, [])

    if (loading) {
        return <div>Loading...</div>;
    }

    if (!auth) {
        return <Navigate to="/signin" replace/>;
    }

    return props.children;
};




