import React, {useEffect, useState} from "react";
import DialogTitle from "@mui/material/DialogTitle";
import {Button, Grid, IconButton, TextField, Typography} from "@mui/material";
import {styled} from "@mui/material/styles";
import Dialog from "@mui/material/Dialog";
import CloseIcon from "@mui/icons-material/Close";
import DialogContent from "@mui/material/DialogContent";
import DialogActions from "@mui/material/DialogActions";
import {useNavigate} from "react-router-dom";
import {useFormik} from "formik";
import {toast} from "react-toastify";
import {getCookie} from "../App";
import {saveAs} from "file-saver"
const url = process.env.REACT_APP_API_URL

const BootstrapDialog = styled(Dialog)(({theme}) => ({
    '& .MuiDialogContent-root': {
        padding: theme.spacing(2),
    },
    '& .MuiDialogActions-root': {
        padding: theme.spacing(1),
    },
    '& .MuiPaper-root': {
        width: '50%', // Adjust the width as per your requirement
    },
}));


export default function PrivateDialog(props) {
    const navigate = useNavigate()
    const [maxRequests, setMaxRequest] = useState(3);
    const [requestInterval, setRequestInterval] = useState(5000)
    const [requestCount, setRequestCount] = useState(0)
    const [lastRequestTime, setLastRequestTime] = useState(0)
    const handleClose = () => {
        props.setOpen(false);
    };

    const formik = useFormik({
        initialValues: {
            privateKey: '',
        },
        onSubmit: async (values) => {
            try {
                await fetchWithRateLimit(values)
            } catch (error) {
                if (error instanceof Error) {
                    toast.error(error.message); // Display the error message in a toast
                } else {
                    const errorMessage = await error.text();
                    const errorObject = JSON.parse(errorMessage);
                    toast.error(errorObject.error); // Display the custom backend exception message in a toast
                }
            }
        }
    });


    const fetchWithRateLimit = async (values) => {


        return new Promise((resolve, reject) => {

            const makeRequest = async arrayLike => {
                const currentTime = new Date().getTime();
                if (currentTime - lastRequestTime < requestInterval) {
                    setRequestCount(requestCount + 1);
                    setLastRequestTime(currentTime)
                    if (requestCount > maxRequests) {
                        reject(new Error('Too many requests, please try again later.'));
                        return;
                    }
                } else {
                    setRequestCount(1)
                    setLastRequestTime(currentTime)
                }
                const token = getCookie('auth');
                try {
                    const response = await fetch(url + '/reservation/detail/' + props.id, {
                        method: 'GET',
                        headers: {
                            'Authorization': `${token}`,
                        },
                    });
                    const jsonData = await response.json();

                    if (!response.ok) {
                        if (response.status === 401) {
                            navigate("/signin")
                        } else {
                            toast.error(response.message)
                        }
                    } else {
                        /*const CryptoJS = require('crypto-js');
                        const forge = require('node-forge');
                        console.log(jsonData)
                        console.log(JSON.parse(jsonData.pdf))

                        const uint8Array = new Uint8Array(JSON.parse(jsonData.pdf));

                        const pdfBlob = new Blob([uint8Array], {type: 'application/pdf'});

                        saveAs(pdfBlob, 'reservation.pdf');*/

                        const CryptoJS = require('crypto-js');
                        const forge = require('node-forge');
                        const privateKK = `-----BEGIN PRIVATE KEY-----\n${values.privateKey}\n-----END PRIVATE KEY-----`;
                        const privateKey = forge.pki.privateKeyFromPem(privateKK);
                        const encryptedKeyAES = forge.util.decode64(jsonData.encryptedAes);
                        const decryptedKey = privateKey.decrypt(encryptedKeyAES);
                        const encryptedDataBinary = atob(jsonData.pdf);
                        var key = CryptoJS.enc.Utf8.parse(decryptedKey);
                        const encryptedDataWordArray = CryptoJS.lib.WordArray.create(Uint8Array.from(encryptedDataBinary, c => c.charCodeAt(0)));
                        var decryptedBytes = CryptoJS.AES.decrypt(
                            {ciphertext: CryptoJS.enc.Base64.parse(jsonData.pdf)},
                            key,
                            {mode: CryptoJS.mode.ECB, padding: CryptoJS.pad.Pkcs7}
                        );
                        var decryptedString = CryptoJS.enc.Utf8.stringify(decryptedBytes);
                        const uint8Array = new Uint8Array(JSON.parse(decryptedString));

                        const pdfBlob = new Blob([uint8Array], {type: 'application/pdf'});
                        saveAs(pdfBlob, 'reservation.pdf');
                        formik.setFieldValue("privateKey", "")
                        props.setOpen(false)

                    }


                } catch (error) {

                    console.log("Error" + error)
                    if (error.response) {

                        const errorData = await error.response.json();
                        toast.error(errorData.message); // Extract the error message from the response
                    } else if (error.request) {

                        toast.error('No response received from server. ');
                    } else {
                        toast.error('An error occured.')

                    }
                }

            };

            makeRequest();
        });
    };

    const handleDownload = () => {
        formik.handleSubmit()

    }

    return (
        <React.Fragment>
            <BootstrapDialog
                onClose={handleClose}
                aria-labelledby="customized-dialog-title"
                open={props.open}
            >
                <DialogTitle sx={{m: 0, p: 2}} id="customized-dialog-title">
                    Enter your private key
                </DialogTitle>
                <IconButton
                    aria-label="close"
                    onClick={handleClose}
                    sx={{
                        position: 'absolute',
                        right: 8,
                        top: 8,
                        color: (theme) => theme.palette.grey[500],
                    }}
                >
                    <CloseIcon/>
                </IconButton>
                <DialogContent dividers>
                    <Grid item xs={12} md={12} lg={12}>
                        <TextField
                            id="privateKey"
                            name="privateKey"
                            label="Private Key"
                            fullWidth
                            value={formik.values.privateKey}
                            onChange={formik.handleChange}
                            error={formik.touched.privateKey && Boolean(formik.errors.privateKey)}
                            helperText={formik.touched.privateKey && formik.errors.privateKey}
                        />
                    </Grid>
                </DialogContent>
                <DialogActions>
                    <Button autoFocus variant="contained"
                            disabled={formik.values.privateKey === "" || formik.values.privateKey === undefined}
                            onClick={handleDownload}>
                        Download
                    </Button>
                </DialogActions>
            </BootstrapDialog>
        </React.Fragment>
    );
}