import {
    Button,
    Card,
    CardContent,
    CardHeader,
    CardMedia,
    Grid,
    Table,
    TableBody,
    TableCell,
    TableContainer,
    TableHead,
    TableRow, Typography,
} from "@mui/material";
import React, {useEffect, useState} from "react";
import {getCookie} from "../App";
import {toast} from "react-toastify";
import {useNavigate} from "react-router-dom";
import DescriptionIcon from '@mui/icons-material/Description';
import PrivateDialog from "./private-dialog";
const url = process.env.REACT_APP_API_URL;

export const ReservationPage = () => {
    const [reservations, setReservations] = useState([]);
    const [loaded, setLoaded] = useState(false)
    const navigate = useNavigate()
    const [openPrivate, setOpenPrivate]= useState(false)
    const [selectedID, setSelectedID]=useState(null)

    useEffect(() => {
        fetchData();
    }, []);
    const fetchData = async () => {
        const token = getCookie('auth');
        try {
            const response = await fetch(url + '/reservation/all', {
                method: 'GET',
                headers: {
                    'Authorization': `${token}`,
                },
            },)

            if (!response.ok) {
                if (response.status === 401) {
                    navigate("/signin")
                } else {
                    toast.error(response.message)
                }
            } else {
                const result = await response.json();
                setReservations(result)
                setLoaded(true)
            }

        } catch (error) {
            console.error('Error fetching data:', error);
        }
    };

    return (
        <>
            <Grid container justifyContent="center" style={{marginTop: "13vh"}}>
                <Grid item xs={11} md={8}>
                    <Card>
                        <CardHeader style={{marginTop:"1rem", marginLeft:"1rem"}} title="My Reservations"/>
                        <CardContent>
                            {reservations.length > 0 ?
                                <TableContainer>
                                    <Table>
                                        <TableHead >
                                            <TableRow >
                                                <TableCell style={{ fontWeight: "bold", fontSize: "16px" }} >Accomodation</TableCell>
                                                <TableCell style={{ fontWeight: "bold", fontSize: "16px" }}>Address</TableCell>
                                                <TableCell style={{ fontWeight: "bold", fontSize: "16px" }}>Guests</TableCell>
                                                <TableCell style={{ fontWeight: "bold", fontSize: "16px" }}>Check In</TableCell>
                                                <TableCell style={{ fontWeight: "bold", fontSize: "16px" }}>Check Out</TableCell>
                                                <TableCell style={{ fontWeight: "bold", fontSize: "16px" }}>Total Price</TableCell>
                                                <TableCell style={{ fontWeight: "bold", fontSize: "16px" }}></TableCell>
                                                {/* Add more columns as needed */}
                                            </TableRow>
                                        </TableHead>
                                        <TableBody>
                                            {reservations.map((reservation) => (
                                                <TableRow key={reservation.id}>
                                                    <TableCell>{reservation.accommodation.name}</TableCell>
                                                    <TableCell>{reservation.accommodation.address}</TableCell>
                                                    <TableCell>{reservation.guestsCount}</TableCell>
                                                    <TableCell>{reservation.checkInDate}</TableCell>
                                                    <TableCell>{reservation.checkOutDate}</TableCell>
                                                    <TableCell>{reservation.totalPrice} €</TableCell>
                                                    <TableCell onClick={()=>{
                                                        setSelectedID(reservation.id)
                                                        setOpenPrivate(true)
                                                    }} style={{ cursor: "pointer" }}><DescriptionIcon></DescriptionIcon>{reservation.pdf}</TableCell>

                                                </TableRow>
                                            ))}
                                        </TableBody>
                                    </Table>
                                </TableContainer> : <Typography gutterBottom align="center">
                                    You don´t have any reservations.
                                </Typography>}
                        </CardContent>
                    </Card>
                </Grid>
                <PrivateDialog open={openPrivate} setOpen={setOpenPrivate} id={selectedID}></PrivateDialog>
            </Grid>
        </>
    );
};
