<?php

namespace App\Http\Controllers;

use App\Models\Cart;
use App\Models\Order;
use App\Models\OrderProduct;
use App\Utils\Security;
use Dompdf\Dompdf;
use App\Models\Address;
use Illuminate\Http\Request;

class OrderController extends Controller
{
    use Security;

    protected $user;

    public function __construct()
    {
        $this->user = auth()->user();
    }

    public function index()
    {
        $orders = $this->user->orders;

        $orders = $orders->map(function ($order) {
            $order->products = $order->products->map(function ($product) {
                $product->product;
                return $product;
            });
            return $order;
        });

        return response()->json($orders, 200);
    }

    public function show($id)
    {
        $order = $this->user->orders()->with('products')->find($id);

        $order->products = $order->products->map(function ($product) {
            $product->product;
            return $product;
        });


        if (!$order) {
            return response()->json([
                "message" => "Order not found"
            ], 404);
        }

        return $order;
    }

    public function store(Request $request)
    {
        $cart = Cart::where('user_id', $this->user->id)->first();

        if (!$cart) {
            return response()->json([
                "message" => "Cart not found"
            ], 404);
        }

        $order = new Order();
        $order->user_id = $this->user->id;
        $order->address_id = $request->address_id;
        $order->total_price = $cart->total_price;
        $order->save();

        $total_price = 0;

        foreach ($cart->products as $product) {
            $order_product = new OrderProduct();
            $order_product->order_id = $order->id;
            $order_product->product_id = $product->product_id;
            $order_product->quantity = $product->quantity;
            $order_product->price = $product->product->price;
            $order_product->save();
            $total_price += $product->price * $product->quantity;
        }

        $cart->products()->delete();
        $cart->delete();

        return response()->json($order, 201);
    }

    public function invoice(int $id)
    {
        $user = auth()->user();

        if ($user->public_key == null) {
            return response()->json([
                "message" => "You must set your public key first",
            ], 400);
        }

        $order = $this->user->orders()->with('products')->find($id);

        $order->products = $order->products->map(function ($product) {
            $product->product;
            return $product;
        });

        $order->address = Address::find($order->address_id);

        $html = view()->make('invoice', ['order' => $order])->render();

        $dompdf = new Dompdf();
        $dompdf->loadHtml($html);
        $dompdf->setPaper('A4');
        $dompdf->render();

        $data = $dompdf->output();
        $publicKey = $this->validatePublicKey($user->public_key);
        $encryptedData = $this->encrypt($publicKey, $data);

        return response($encryptedData)->withHeaders([
            'Content-Type' => 'application/octet-stream',
        ]);
    }
}
