<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Validation\ValidationException;

class Address extends Model
{
    use HasFactory;

    protected $fillable = [
        'user_id',
        'name',
        'surname',
        'street',
        'city',
        'country',
        'zip_code',
    ];

    protected static function boot()
    {
        parent::boot();

        static::saving(function ($model) {
            $validator = validator($model->attributes, [
                'user_id' => 'required',
                'name' => 'required|alpha|min:2|max:255',
                'surname' => 'required|alpha|min:2|max:255',
                'street' => 'required|min:2|max:255',
                'city' => 'required|min:2|max:255',
                'country' => 'required|min:2|max:255',
                'zip_code' => 'required|numeric|digits:5',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }
        });
    }


    public function user()
    {
        return $this->belongsTo(User::class);
    }

    public function orders()
    {
        return $this->hasMany(Address::class);
    }
}
