<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Foundation\Auth\User as Authenticatable;

use Tymon\JWTAuth\Contracts\JWTSubject;


class User extends Authenticatable implements JWTSubject
{
    use HasFactory;

    protected $fillable = [
        "email",
        "password",
        "first_name",
        "last_name",
        "remember_token"
    ];

    protected $hidden = [
        "password",
        "remember_token"
    ];

    protected $rules = [
        "email"=> "required|email",
        "password"=> "required|min:8",
        "first_name"=> "required",
        "last_name"=> "required",
    ];

    protected $casts = [
        'password' => 'hashed',
    ];

    public function rewiews()
    {
        return $this->hasMany(Rewiew::class);
    }

    public function carts()
    {
        return $this->hasMany(Cart::class);
    }

    public function orders()
    {
        return $this->hasMany(Order::class);
    }

    public function addresses()
    {
        return $this->hasMany(Address::class);
    }
    
    public function getJWTIdentifier() {
        return $this->getKey();
    }

    public function getJWTCustomClaims() {
        return [];
    }  
}
