<?php

namespace App\Utils;

use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use phpseclib3\Crypt\AES;
use phpseclib3\Crypt\Common\PrivateKey;
use phpseclib3\Crypt\Common\PublicKey;
use phpseclib3\Crypt\Random;
use phpseclib3\Crypt\RSA;

const KEY_LENGTH = 16; // 128 bits

trait Security
{
    private static function getCipher(): AES
    {
        return new AES('gcm');
    }

    public function validatePublicKey($str): ?PublicKey
    {
        try {
            // TODO: validate public key length?
            return RSA::loadPublicKey($str);
        } catch (\Exception $e) {
            return null;
        }
    }

    public function encrypt($publicKey, $data): string
    {
        $cipher = self::getCipher();

        $key = Random::string(KEY_LENGTH);
        $iv = Random::string(KEY_LENGTH);

        $cipher->setKey($key);
        $cipher->setNonce($iv);

        $ciphertext = $cipher->encrypt($data);
        $tag = $cipher->getTag();

        $a = pack('C3', KEY_LENGTH, KEY_LENGTH, strlen($tag)) . $key . $iv . $tag;
        $enc = $publicKey->encrypt($a);

        return pack('v', strlen($enc)) . $enc . $ciphertext;
    }
}
