<?php

use App\Http\Controllers\AddressController;
use App\Http\Controllers\CartController;
use App\Http\Controllers\CategoryController;
use App\Http\Controllers\OrderController;
use App\Http\Controllers\ProductController;
use App\Http\Controllers\RewiewController;
use App\Http\Controllers\TestController;
use App\Http\Middleware\auth;
use App\Models\Product;
use App\Models\Rewiew;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "api" middleware group. Make something great!
|
*/

Route::group([
    'prefix' => 'users',
], function () {
    Route::post("/register", "App\Http\Controllers\UsersController@register");
    Route::post("/login", "App\Http\Controllers\UsersController@login");
    Route::post("/check-strength", "App\Http\Controllers\UsersController@checkStrength");

    Route::group([
        'middleware' => ['auth'],
    ], function () {
        Route::post("/refresh", "App\Http\Controllers\UsersController@refresh");
        Route::post("/logout", "App\Http\Controllers\UsersController@logout");
        Route::post("/logout-from-all", "App\Http\Controllers\UsersController@logoutFromAll");
        Route::post('/public-key', 'App\Http\Controllers\UsersController@savePublicKey');
    });
});

Route::group([
    'prefix' => 'v1',
], function () {
    Route::group([
        'middleware' => ['jwt.auth'],
    ], function () {
        Route::resource('addresses', AddressController::class);
        Route::post('products/search', [ProductController::class, 'search']);
        Route::resource('cart', CartController::class);
        Route::resource('orders', OrderController::class);
        Route::put('add_to_cart', [CartController::class, 'update']);
        Route::get("orders/{id}/invoice", [OrderController::class, 'invoice']);
        Route::resource('reviews', RewiewController::class)->only([
            'store', 'update', 'destroy'
        ]);
       
    });
    
    Route::resource('products', ProductController::class);
    Route::resource('categories', CategoryController::class);
    Route::resource('reviews', RewiewController::class)->only([
        'index'
    ]);
    Route::get('product/{id}/reviews', [RewiewController::class, 'getProductReviews']);
    


});
