import { createApp } from 'vue'
import { createPinia } from 'pinia'
import { createRouter, createWebHistory } from 'vue-router'
import Toast, { PluginOptions } from 'vue-toastification'
import { useUserStore } from './stores/user'
import routes from './routes'
import App from './App.vue'

import 'vue-toastification/dist/index.css'
import './style.css'

const pinia = createPinia()
const router = createRouter({
  history: createWebHistory(),
  routes,
})

const app = createApp(App)

app.use(Toast, {
  position: 'bottom-right',
  toastClassName: 'custom-toast',
} as PluginOptions)

app.use(pinia)
app.use(router)
app.mount('#app')

const userStore = useUserStore()

router.beforeEach((to, _, next) => {
  if (!userStore.authPromise) {
    next()
    return
  }

  userStore.authPromise.then(() => {
    if (to.meta.requiresAuth && !userStore.token) {
      next({ name: 'login' })
    } else if (to.meta.requiresGuest && userStore.token) {
      next({ name: 'home' })
    } else {
      next()
    }
  })
})
