import { ref } from 'vue'
import { defineStore } from 'pinia'
import { useUserStore } from './user'
import { useRouter } from 'vue-router'

export const useCartStore = defineStore('cart', () => {
  const total = ref(0.0)
  const products = ref<any[]>([])
  const userStore = useUserStore()
  const router = useRouter()

  async function addToCart(product: any) {
    if (!userStore.user) {
      router.push({ name: 'login' })
      return Promise.reject()
    }

    await userStore.client
      .url('v1/add_to_cart')
      .put({ product_id: product.id, quantity: 1 })
      .then((data: any) => {
        total.value = data.total_price
        products.value = data.products
      })
  }

  async function getCart() {
    await userStore.client
      .url('v1/cart')
      .get()
      .then((data: any) => {
        if (!data[0]) {
          total.value = 0
          products.value = []
          return
        }

        total.value = data[0].total_price
        products.value = data[0].products
      })
  }

  async function remove(product_id: any) {
    await userStore.client
      .url('v1/add_to_cart')
      .put({ product_id: product_id, quantity: 0 })
      .then((data: any) => {
        total.value = data.total_price
        products.value = data.products
      })
  }

  async function change_quantity(product_id: any, quantity: any) {
    await userStore.client
      .url('v1/add_to_cart')
      .put({ product_id: product_id, quantity: quantity })
      .then((data: any) => {
        total.value = data.total_price
        products.value = data.products
      })
  }

  return { total, products, addToCart, getCart, remove, change_quantity }
})
