import { defineStore } from 'pinia'
import { computed, ref } from 'vue'
import wretch from 'wretch'

export interface User {
  email: string
  name: string
  first_name: string
  last_name: string
  public_key: string | null
  public_key_fingerprint?: string
}

export const useUserStore = defineStore('user', () => {
  const user = ref<null | User>(null)
  const token = ref<null | string>(null)
  const authPromise = ref<Promise<any> | null>(null)
  const loading = ref(true)
  const client = computed(() =>
    createClient()
      .resolve((r) => r.json())
      .headers({
        Accept: 'application/json',
      }),
  )

  function createClient() {
    return wretch(import.meta.env.VITE_BE_URL, {
      headers: {
        Authorization: token.value ? `Bearer ${token.value}` : '',
      },
    }).errorType('json')
  }

  return { user, token, client, authPromise, loading, createClient }
})
