from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization, hashes
from flask import Flask, render_template, request, jsonify, session, redirect, url_for, sessions
import psycopg2
import sys
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives.asymmetric import padding
import base64
import re
import os
from flask_sqlalchemy import SQLAlchemy
import hashlib
import hmac
from OpenSSL import SSL
from datetime import datetime, timedelta
from models import db, User, Account, PaymentTransactions, PersonalDetails, Transactions, Transactions_Encrypted
from flask_session import Session
from reportlab.pdfgen import canvas
from flask import send_file
from flask import render_template, request, redirect, url_for

app = Flask(__name__)
app.config['SQLALCHEMY_DATABASE_URI'] = 'postgresql://postgres:heslo123@postgres:5432/upb'
app.config['SESSION_TYPE'] = 'filesystem'  # You can choose another storage method if needed
Session(app)
context = SSL.Context(SSL.SSLv23_METHOD)
context.use_privatekey_file('keys/key.pem')
context.use_certificate_file('keys/cert.pem')

# Force SSL for the entire application
app.config['FORCE_SSL'] = True

# Initialize Flask-SQLAlchemy
db.init_app(app)


# PUBLIC KEY
def add_pem_headers_and_footers(public_key_str):
    header = "-----BEGIN PUBLIC KEY-----"
    footer = "-----END PUBLIC KEY-----"
    return f"{header}\n{public_key_str}\n{footer}"


#
@app.route('/get_encrypted_data', methods=['GET'])
def get_encrypted_data():
    data = Transactions_Encrypted.query.all()
    data_list = [{'id': item.id, 'from_name': item.from_name, 'to_name': item.to_name, 'fernet_key': item.fernet_key}
                 for item in data]
    return jsonify(data_list)


def generate_fernet_key():
    return Fernet.generate_key()


@app.route('/encrypt_data')
def encryption_of_data():
    user_name = session.get('user_name')
    all_data = Transactions.query.all()
    user = User.query.filter_by(username=user_name).first()

    fernet_key = generate_fernet_key()

    user_public_key = user.public_key
    public_key_pem = base64.b64decode(user_public_key).decode('utf-8')
    loaded_public_key = serialization.load_pem_public_key(public_key_pem.encode())

    for row in all_data:
        encrypt_from, encrypt_to, encrypt_key = encrypt_data(row.from_name, row.to_name, fernet_key)

        encrypt_from_b64 = base64.b64encode(encrypt_from).decode('utf-8')
        encrypt_to_b64 = base64.b64encode(encrypt_to).decode('utf-8')
        encrypt_key_b64 = base64.b64encode(encrypt_key).decode('utf-8')
        # print(encrypt_key_b64,file=sys.stderr)
        encrypted_key = loaded_public_key.encrypt(
            encrypt_key_b64.encode('utf-8'),
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None
            )
        )

        encrypted_key_b64 = base64.b64encode(encrypted_key).decode('utf-8')

        transaction_crypt = Transactions_Encrypted(
            from_name=encrypt_from_b64,
            to_name=encrypt_to_b64,
            fernet_key=encrypted_key_b64
        )

        db.session.add(transaction_crypt)
        db.session.commit()

    return user_name


def encrypt_data(from_name, to_name):
    encryption_key = Fernet.generate_key()
    fernet = Fernet(encryption_key)
    encrypted_from = fernet.encrypt(from_name.encode('utf-8'))
    encrypted_to = fernet.encrypt(to_name.encode('utf-8'))
    return encrypted_from, encrypted_to, encryption_key


@app.route('/generate_rsa_key', methods=['POST'])
def generate_rsa_key():
    data = request.get_json()
    public_key_json = data.get('public_key')

    if public_key_json:
        public_key_base64 = base64.b64encode(public_key_json.encode('utf-8')).decode('utf-8')
        user = User.query.filter_by(username=session['user_name']).first()
        user.public_key = public_key_base64
        db.session.commit()
        return jsonify({'message': 'Public key saved successfully'})
    else:
        return jsonify({'error': 'Invalid or missing public key data'}), 400


@app.route('/login', methods=['POST'])
def login():
    username = request.form['username']
    password = request.form['password']

    # Query the database to retrieve the user's record by username
    user = User.query.filter_by(username=username).first()

    message = "Chybne meno alebo heslo"
    if user is None:
        return render_template('error.html', message=message)

    # Check if the user's account is on cooldown
    if user.last_login_attempt:
        cooldown_time = timedelta(minutes=1)  # 1 minute cooldown
        if (user.last_login_attempt + cooldown_time > datetime.now()) and (user.login_attempts >= 5):
            return "Ucet je docasne zablokovany skuste za 60 sekund"

    # Validate the password
    stored_pwd = user.hashed_password
    salt = user.salt

    if validate_password(password, stored_pwd, salt):
        # Reset login attempts upon successful login
        user.login_attempts = 0
        user.last_login_attempt = None
        db.session.commit()
        session['user_name'] = user.username
        session['user_id'] = user.id
        return redirect('/dashboard')
    else:
        # Increase login attempts and update last login attempt timestamp
        user.login_attempts += 1
        user.last_login_attempt = datetime.now()
        db.session.commit()

        # If login attempts exceed 5, show account on cooldown message
        if user.login_attempts >= 5:
            return "Vela nesprávnych pokusov."
        message = "Nesprávne heslo alebo meno"
        return render_template('error.html', message=message)


@app.route('/logout')
def logout():
    # Clear the user session
    session.pop('user_name', None)
    return redirect('/')


@app.route('/back_to_homepage', methods=['GET'])
def home_page():
    return render_template('public_page.html')


def validate_password(input_password, stored_password, salt):
    input_password = input_password
    pwd = input_password + salt

    hashed_log_pwd = hashlib.sha256(pwd.encode('utf-8')).hexdigest()

    return hmac.compare_digest(hashed_log_pwd, stored_password)


import random
import string


def generate_random_iban():
    # Generate two random uppercase letters
    random_letters = ''.join(random.choices(string.ascii_uppercase, k=2))

    # Generate 22 random digits
    random_digits = ''.join(random.choices(string.digits, k=22))

    # Assemble the IBAN with the specified format
    iban = f"SK{random_digits[:2]} {random_digits[:4]} {random_digits[4:8]} {random_digits[8:12]} {random_digits[12:16]} {random_digits[16:20]}"

    return iban


@app.route('/register', methods=['POST'])
def register():
    username = request.form['new-username']
    password = request.form['new-password']
    confirm_password = request.form['confirm-password']

    # Validate the password (your existing validation code)
    email = request.form['email']

    if not is_secure_password(password):
        return "Password must contain at least one uppercase letter, one lowercase letter, and one digit."

    if password != confirm_password:
        return "Passwords do not match."

    with open('files/100k-most-used-passwords-NCSC.txt', 'r') as password_file:
        common_passwords = password_file.read().splitlines()

    if password in common_passwords:
        return "Common password detected. Please choose a stronger password."

    # Hash the password and generate a salt
    salt = os.urandom(16)  # Generate a random salt
    password = request.form['new-password']  # Get the password as a string
    salt_hex = salt.hex()
    salted_password = password + salt_hex  # Combine password and salt as a string

    hashed_password = hashlib.sha256(salted_password.encode('utf-8')).hexdigest()

    user = User.query.filter_by(username=username).first()

    if user is not None:
        message = "Neuspesna registracia. Uzivatelske meno uz existuje"
        return render_template('error.html', message=message)

    # Store the hashed password and salt in the database
    user = User(username=username, hashed_password=hashed_password, email=email, salt=salt_hex)
    db.session.add(user)
    db.session.commit()

    # Generate a random IBAN
    iban = generate_random_iban()

    # Create an account for the user with the generated IBAN
    account = Account(user_id=user.id, iban=iban)
    db.session.add(account)
    db.session.commit()

    message = "Uspesna registracia"
    return render_template("success.html", message=message)


@app.route('/update_profile', methods=['GET', 'POST'])
def update_profile():
    user_id = session.get('user_id')
    if request.method == 'POST':
        # Retrieve form data
        name = request.form['name']
        surname = request.form['surname']
        birth_date = request.form['birth_date']
        country = request.form['country']
        city = request.form['city']
        street = request.form['street']
        postal_code = request.form['postal_code']
        phone_number = request.form['phone_number']

        # Update personal details in the database
        personal_details = PersonalDetails.query.filter_by(user_id=user_id).first()
        personal_details.name = name
        personal_details.surname = surname
        personal_details.birth_date = birth_date
        personal_details.country = country
        personal_details.city = city
        personal_details.street = street
        personal_details.postal_code = postal_code
        personal_details.phone_number = phone_number

        db.session.commit()

        return dashboard()

    return render_template('update_profile.html')


def is_secure_password(password):
    # Password must contain at least one uppercase letter, one lowercase letter, and one digit
    return bool(re.search(r'^(?=.*[A-Z])(?=.*[a-z])(?=.*\d)', password))


# Endpoint pre zobrazenie stavu účtu, Ak bude treba tak dorobit aby returnlo dalsie veci z db
@app.route('/get_account_balance', methods=['GET'])
def get_account_balance(user_id):
    try:
        session.get('user_id')
        user = User.query.get(user_id)
        if user:
            accounts = Account.query.filter_by(user_id=user.id).all()
            account_info = [{'account_id': acc.account_id, 'balance': acc.balance, 'iban': acc.iban} for acc in
                            accounts]
            return jsonify({'user': user.username, 'accounts': account_info})
        else:
            return jsonify({'error': 'User not found'}), 404
    except Exception as e:
        return jsonify({'error': str(e)}), 500


@app.route('/transaction', methods=['POST'])
def make_transaction():
    try:
        data = request.get_json(force=True)
        sender_account_number = session.get('user_id')
        receiver_account_number = data.get('receiver')
        amount = float(data.get('amount'))  # Convert amount to float

        sender_account = Account.query.filter_by(user_id=sender_account_number).first()
        receiver_account = Account.query.filter_by(account_id=receiver_account_number).first()

        if sender_account and receiver_account and sender_account.balance >= amount:
            transactions = PaymentTransactions(
                user_id=sender_account_number,
                account_id=sender_account_number,
                sender_account_number=sender_account_number,
                receiver_account_number=receiver_account_number,
                amount=amount
            )
            # Realize the payment and update records in the database
            sender_account.balance -= amount
            receiver_account.balance += amount
            db.session.add(transactions)
            db.session.commit()
            return jsonify({'message': 'Transaction successful'})
        else:
            return jsonify({'error': 'Insufficient funds or invalid accounts'}), 400
    except Exception as e:
        return jsonify({'error': str(e)}), 500


@app.route('/update_page', methods=['GET'])
def redirect_to_update_page():
    return render_template('update_profile.html')


@app.route('/new_transaction', methods=['GET'])
def new_transaction_form():
    # Retrieve user and account information
    user_name = session.get('user_name')
    user = User.query.filter_by(username=user_name).first()

    if user:
        # Fetch a list of potential receivers (users other than the current user)
        potential_receivers = User.query.filter(User.id != user.id).all()

        return render_template('new_transaction.html', user_name=user_name, receivers=potential_receivers)
    else:
        return jsonify({'error': 'User not found'}), 404


@app.route('/dashboard')
def dashboard():
    # Access the user's name from the session
    user_name = session.get('user_name')

    # Retrieve user and account information
    user = User.query.filter_by(username=user_name).first()

    if user:
        # Fetch user's account information
        accounts = Account.query.filter_by(user_id=user.id).all()

        account_info = [{'account_id': acc.account_id, 'balance': acc.balance, 'iban': acc.iban} for acc in accounts]

        profiles = PersonalDetails.query.filter_by(user_id=user.id).all()

        profile_info = [{'name':  p_info.name, 'surname': p_info.surname, 'birth_date': p_info.birth_date,
                         'country': p_info.country, 'city': p_info.city, 'street': p_info.street,
                         'postal_code': p_info.postal_code, 'phone_number': p_info.phone_number} for p_info in profiles]

        # Fetch both outgoing and incoming transactions for the user
        sent_transactions = PaymentTransactions.query.filter_by(sender_account_number=user.id).all()
        received_transactions = PaymentTransactions.query.filter_by(receiver_account_number=user.id).all()

        # Combine the sent and received transactions into a single list
        all_transactions = sent_transactions + received_transactions

        transaction_info = []
        for transaction in all_transactions:
            sender = User.query.filter_by(id=transaction.sender_account_number).first()
            receiver = User.query.filter_by(id=transaction.receiver_account_number).first()

            transaction_info.append({
                'id': transaction.id,
                'sender_name': sender.username if sender else 'Unknown',
                'receiver_name': receiver.username if receiver else 'Unknown',
                'amount': transaction.amount,
                'timestamp': transaction.timestamp.strftime("%d-%m-%Y %H:%M:%S")
            })

        # Render the dashboard template with the user's name and transactions
        return render_template('dashboard_secure.html', accounts=account_info, profiles=profile_info,
                               user_name=user_name, transactions=transaction_info)
    else:
        return jsonify({'error': 'User not found'}), 404
    return render_template('dashboard.html')


@app.route('/generate_pdf', methods=['GET'])
def generate_pdf():
    # Retrieve user and transaction information
    user_name = session.get('user_name')
    user = User.query.filter_by(username=user_name).first()

    if user:
        # Fetch user's transaction information
        transactions = PaymentTransactions.query.filter_by(user_id=user.id).all()

        # Create a PDF document
        pdf_filename = f"{user_name}_transactions.pdf"
        pdf_path = f"pdfs/{pdf_filename}"

        # Set up the PDF canvas
        c = canvas.Canvas(pdf_path)
        c.setFont("Helvetica", 12)

        # Add user information
        c.drawString(100, 750, f"User: {user_name}")
        c.drawString(100, 730, "Transaction History")

        # Add transaction details
        y_position = 700
        for transaction in transactions:
            sender = User.query.filter_by(id=transaction.sender_account_number).first()
            receiver = User.query.filter_by(id=transaction.receiver_account_number).first()

            transaction_info = (
                f"Transaction ID: {transaction.id}",
                f"Sender: {sender.username if sender else 'Unknown'}",
                f"Receiver: {receiver.username if receiver else 'Unknown'}",
                f"Amount: {transaction.amount}",
                f"Timestamp: {transaction.timestamp.strftime('%d-%m-%Y %H:%M:%S')}",
                "-" * 30
            )

            for line in transaction_info:
                c.drawString(100, y_position, line)
                y_position -= 15

        # Save the PDF file
        c.save()

        # Provide the file for download
        return send_file(pdf_path, as_attachment=True)
    else:
        return jsonify({'error': 'User not found'}), 404


@app.route('/')
def index():
    return render_template("public_page.html")


if __name__ == '__main__':
    context = ('keys/cert.pem', 'keys/key.pem')
    app.run(host="0.0.0.0", port=5000, ssl_context=context)
