CREATE TABLE transactions (
    id serial PRIMARY KEY,
    from_name VARCHAR(255),
    to_name VARCHAR(255)
);

CREATE TABLE transactions_encrypted (
    id serial PRIMARY KEY,
    from_name VARCHAR(2048),
    to_name VARCHAR(2048),
    fernet_key VARCHAR(2048)
);

CREATE TABLE users (
    id serial PRIMARY KEY,
    username VARCHAR(255) NOT NULL UNIQUE,
    hashed_password VARCHAR(255) NOT NULL,
    email VARCHAR(255) NOT NULL UNIQUE,
    salt VARCHAR(255) NOT NULL ,
    last_login_attempt TIMESTAMP,
    login_attempts INT DEFAULT 0,
    public_key VARCHAR(4096)
);

CREATE TABLE account (
    account_id serial PRIMARY KEY,
    user_id INT,
    balance FLOAT,
    iban VARCHAR(34),
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    FOREIGN KEY (user_id) REFERENCES users(id)
);

CREATE TABLE payment_transactions (
    id serial PRIMARY KEY,
    account_id INT NOT NULL,
    user_id INT,
    sender_account_number INT NOT NULL,
    receiver_account_number INT NOT NULL,
    amount FLOAT,
    timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    FOREIGN KEY (account_id) REFERENCES account(account_id),
    FOREIGN KEY (user_id) REFERENCES users(id)
);

CREATE TABLE personal_details (
    personal_id serial PRIMARY KEY,
    user_id INT,
    name VARCHAR(100) NOT NULL,
    surname VARCHAR(100) NOT NULL ,
    address VARCHAR(255) NOT NULL ,
    phone_number VARCHAR(20) NOT NULL,
    birth_date DATE NOT NULL,
    country VARCHAR(255),
    postal_code VARCHAR(20),
    city VARCHAR(255),
    street VARCHAR(255),
    building_number VARCHAR(10),
    FOREIGN KEY (user_id) REFERENCES users(id)
);


-- Insert some data --
-- Insert transactions
INSERT INTO transactions (from_name, to_name) VALUES ('martin', 'peter');
INSERT INTO transactions (from_name, to_name) VALUES ('peter', 'martin');
INSERT INTO transactions (from_name, to_name) VALUES ('martin', 'gregor');
INSERT INTO transactions (from_name, to_name) VALUES ('hobo', 'peter');
INSERT INTO transactions (from_name, to_name) VALUES ('kamil', 'laco');

-- Insert users
INSERT INTO users (username, hashed_password, email, salt, public_key)
VALUES ('user1', '74efc20946a2b088261e0c9dd3706a721429331a66fb60de368025cc92e33902', 'user1@example.com', '296c30b0cccf978d4e853604e43f41c3', 'public_key_1');

INSERT INTO users (username, hashed_password, email, salt, public_key)
VALUES ('user2', 'd8d61b6676d5696877d85140567e4e184377a12890417996933893d6a94cb3c5', 'user2@example.com', '3bc36cd236731503c7939ef362b9197c', 'public_key_2');

-- Insert account
INSERT INTO account (user_id, balance, iban) VALUES (1, 1000.00, 'SK1234567890');
INSERT INTO account (user_id, balance, iban) VALUES (2, 500.00, 'SK5555567879');

-- Insert payment_transactions
INSERT INTO payment_transactions (account_id, user_id, sender_account_number, receiver_account_number, amount, timestamp) VALUES (2, 1, 2, 1, 50.00, '2022-11-11 15:15:15.22222');
INSERT INTO payment_transactions (account_id, user_id, sender_account_number, receiver_account_number, amount, timestamp) VALUES (1, 2, 1, 2, 30.00, '2022-11-11 15:15:15.22222');

-- Insert personal_details
INSERT INTO personal_details (user_id, name, surname, address, phone_number, birth_date, country, postal_code, city, street, building_number)
VALUES (1, 'John', 'Doe', '123 Main St', '123-456-7890', '1990-01-01', 'USA', '12345', 'Anytown', 'Maple St', '42');

INSERT INTO personal_details (user_id, name, surname, address, phone_number, birth_date, country, postal_code, city, street, building_number)
VALUES (2, 'Jane', 'Smith', '456 Oak St', '987-654-3210', '1985-05-15', 'Canada', '56789', 'Somecity', 'Oak St', '7');