# models.py
from flask_sqlalchemy import SQLAlchemy

db = SQLAlchemy()


class User(db.Model):
    __tablename__ = 'users'
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(255), nullable=False, unique=True)
    hashed_password = db.Column(db.String(255), nullable=False)
    email = db.Column(db.String(255), nullable=False, unique=True)
    salt = db.Column(db.String(255), nullable=False)
    last_login_attempt = db.Column(db.TIMESTAMP, nullable=True, default=None)
    login_attempts = db.Column(db.Integer, default=0)
    public_key = db.Column(db.String(4096), nullable=True)
    accounts = db.relationship('Account', back_populates='user', cascade='all, delete-orphan')
    payment_transactions = db.relationship('PaymentTransactions', back_populates='user', cascade='all, delete-orphan')
    personal_details = db.relationship('PersonalDetails', back_populates='user', cascade='all, delete-orphan')


class Account(db.Model):
    __tablename__ = 'account'
    account_id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey('users.id'), nullable=False)
    balance = db.Column(db.Float, default=100.0)
    iban = db.Column(db.String(34))
    created_at = db.Column(db.TIMESTAMP, default=db.func.current_timestamp())
    updated_at = db.Column(db.TIMESTAMP, default=db.func.current_timestamp(), onupdate=db.func.current_timestamp())
    user = db.relationship('User', back_populates='accounts') #hodi v User tab update na ccount pre daneho usera
    payment_transactions = db.relationship('PaymentTransactions', back_populates='account',
                                           cascade='all, delete-orphan')


class PaymentTransactions(db.Model):
    __tablename__ = 'payment_transactions'
    id = db.Column(db.Integer, primary_key=True)
    account_id = db.Column(db.Integer, db.ForeignKey('account.account_id'), nullable=False)
    user_id = db.Column(db.Integer, db.ForeignKey('users.id'))
    amount = db.Column(db.Float)
    sender_account_number = db.Column(db.Integer)
    receiver_account_number = db.Column(db.Integer)
    timestamp = db.Column(db.TIMESTAMP, default=db.func.current_timestamp())
    account = db.relationship('Account', back_populates='payment_transactions')
    user = db.relationship('User', back_populates='payment_transactions')


class PersonalDetails(db.Model):
    __tablename__ = 'personal_details'
    personal_id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey('users.id'))
    name = db.Column(db.String(100), nullable=False)
    surname = db.Column(db.String(100), nullable=False)
    address = db.Column(db.String(255), nullable=False)
    phone_number = db.Column(db.String(20), nullable=False)
    birth_date = db.Column(db.Date, nullable=False)
    country = db.Column(db.String(255))
    postal_code = db.Column(db.String(20))
    city = db.Column(db.String(255))
    street = db.Column(db.String(255))
    building_number = db.Column(db.String(10))
    user = db.relationship('User', back_populates='personal_details')


class Transactions(db.Model):
    __tablename__ = 'transactions'
    id = db.Column(db.Integer, primary_key=True)
    from_name = db.Column(db.String(255), nullable=True)
    to_name = db.Column(db.String(255), nullable=True)


class Transactions_Encrypted(db.Model):
    __tablename__ = 'transactions_encrypted'
    id = db.Column(db.Integer, primary_key=True)
    from_name = db.Column(db.String(255), nullable=True)
    to_name = db.Column(db.String(255), nullable=True)
    fernet_key = db.Column(db.String(2048), nullable=True)
